@@ -16,7 +16,7 @@ from collections import Counter
 from IPython import embed
 
 import numpy as np
-from scipy import ndimage, signal, interpolate
+from scipy import ndimage, signal, optimize
 from matplotlib import pyplot as plt
 
 from astropy.stats import sigma_clipped_stats, sigma_clip
@@ -24,7 +24,9 @@ from astropy.stats import sigma_clipped_stats, sigma_clip
 from pypeit import msgs
 from pypeit import utils
 from pypeit import sampling
-from pypeit.core import moment, pydl, arc
+from pypeit.core import arc
+from pypeit.core import moment
+from pypeit.core import pydl
 
 # TODO: Some of these functions could probably just live in pypeit.edges
 
@@ -603,8 +605,7 @@ def follow_centroid(flux, start_row, start_cen, ivar=None, bpm=None, fwgt=None,
             Object used to flag the feature traces. If None,
             assessments use a boolean array to flag traces. If not
             None, errors will be raised if the object cannot
-            interpret the correct flag names defined. In addition to
-            flags used by :func:`_recenter_trace_row`, this function
+            interpret the correct flag names defined. This function
             uses the DISCONTINUOUS flag.
 
     Returns:
@@ -911,10 +912,10 @@ def fit_trace(flux, trace_cen, order, ivar=None, bpm=None, trace_bpm=None, weigh
         maxdev (:obj:`float`, optional):
             If provided, reject points with `abs(data-model) >
             maxdev` during the fitting. If None, no points are
-            rejected. See :func:`pypeit.utils.robust_polyfit_djs`.
+            rejected. See :func:`~pypeit.core.fitting.robust_fit`.
         maxiter (:obj:`int`, optional):
             Maximum number of rejection iterations allowed during the
-            fitting. See :func:`pypeit.utils.robust_polyfit_djs`.
+            fitting. See :func:`~pypeit.core.fitting.robust_fit`.
         niter (:obj:`int`, optional):
             The number of iterations for this method; i.e., the
             number of times the two-step fitting algorithm described
@@ -929,10 +930,10 @@ def fit_trace(flux, trace_cen, order, ivar=None, bpm=None, trace_bpm=None, weigh
             if `debug` is true for the plotting. Default is just a
             running number.
         xmin (:obj:`float`, optional):
-            Lower reference for robust_polyfit polynomial fitting.
+            Lower reference for robust_fit polynomial fitting.
             Default is to use zero
         xmax (:obj:`float`, optional):
-            Upper reference for robust_polyfit polynomial fitting.
+            Upper reference for robust_fit polynomial fitting.
             Default is to use the image size in nspec direction
         flavor (:obj:`str`, optional):
             Defines the type of fit performed. Only used by QA
@@ -1224,9 +1225,9 @@ def build_trace_bpm(flux, trace_cen, bpm=None, boxcar=None, thresh=None, median_
 # so it takes only the highest peaks from detect_lines
 def peak_trace(flux, ivar=None, bpm=None, trace_map=None, extract_width=None, smash_range=None,
                peak_thresh=100.0, peak_clip=None, trough=False, trace_median_frac=0.01,
-               trace_thresh=10.0, fwhm_uniform=3.0, fwhm_gaussian=3.0, maxshift=None,
-               maxerror=None, function='legendre', order=5, maxdev=5.0, maxiter=25,
-               niter_uniform=9, niter_gaussian=6, bitmask=None, debug=False):
+               trace_thresh=10.0, fwhm_uniform=3.0, fwhm_gaussian=3.0, min_pkdist_frac_fwhm=5.0,
+               maxshift=None, maxerror=None, function='legendre', order=5, maxdev=5.0, maxiter=25,
+               niter_uniform=9, niter_gaussian=6, bitmask=None, show_fits=False, show_peaks=False):
     """
     Find and trace features in an image by identifying peaks/troughs
     after collapsing along the spectral axis.
@@ -1300,11 +1301,11 @@ def peak_trace(flux, ivar=None, bpm=None, trace_map=None, extract_width=None, sm
             length of the detector. The tuple gives the minimum and
             maximum in the fraction of the full spectral length
             (nspec). If None, the full image is collapsed.
-        peak_thresh (:obj:`float, optional):
+        peak_thresh (:obj:`float`, optional):
             The threshold for detecting peaks in the image. See the
             ``input_thresh`` parameter for
             :func:`~pypeit.core.arc.detect_lines`.
-        peak_clip (:obj:`float, optional):
+        peak_clip (:obj:`float`, optional):
             Sigma-clipping threshold used to clip peaks with small
             values; no large values are clipped. If None, no clipping
             is performed. Generally, if the peak detection algorithm
@@ -1333,6 +1334,10 @@ def peak_trace(flux, ivar=None, bpm=None, trace_map=None, extract_width=None, sm
             The ``fwhm`` parameter to use when using Gaussian
             weighting in the calls to :func:`fit_trace`. See
             description of the algorithm above.
+        min_pkdist_frac_fwhm (:obj:`float`, optional):
+            Minimum allowed separation between same-side edge detections
+            expressed relative to fwhm_gaussian.  See
+            :func:`~pypeit.core.arc.detect_lines`.
         maxshift (:obj:`float`, optional):
             Maximum shift allowed between the input and recalculated
             centroid (see :func:`fit_trace`).
@@ -1363,8 +1368,10 @@ def peak_trace(flux, ivar=None, bpm=None, trace_map=None, extract_width=None, sm
         bitmask (:class:`~pypeit.bitmask.BitMask`, optional):
             Object used to toggle the returned bit masks in edge
             centroid measurements; see :func:`masked_centroid`.
-        debug (:obj:`bool`, optional):
-            Show plots useful for debugging.
+        show_fits (:obj:`bool`, optional):
+            Show (re)fits to edge traces.
+        show_peaks (:obj:`bool`, optional):
+            Show peaks detected in rectified and collapsed trace image.
 
     Returns:
         :obj:`tuple`: Returns four `numpy.ndarray`_ objects and the
@@ -1420,8 +1427,6 @@ def peak_trace(flux, ivar=None, bpm=None, trace_map=None, extract_width=None, sm
         # since that is the width of the sobel filter
         flux_extract = sampling.rectify_image(flux, trace_map, bpm=bpm, extract_width=fwhm_gaussian 
                                                 if extract_width is None else extract_width)[0]
-#        if debug:
-#            ginga.show_image(flux_extract, chname ='rectified image')
 
     # Collapse the image along the spectral direction to isolate peaks/troughs
     start, end = np.clip(np.asarray(smash_range)*nspec, 0, nspec).astype(int)
@@ -1458,7 +1463,7 @@ def peak_trace(flux, ivar=None, bpm=None, trace_map=None, extract_width=None, sm
         peak, _, _cen, _, _, best, _, _ \
                 = arc.detect_lines(s*flux_smash_mean, cont_subtract=False, fwhm=fwhm_gaussian,
                                    input_thresh=peak_thresh, max_frac_fwhm=4.0,
-                                   min_pkdist_frac_fwhm=5.0, debug=debug)
+                                   min_pkdist_frac_fwhm=min_pkdist_frac_fwhm, debug=show_peaks)
 
         if len(_cen) == 0 or not np.any(best):
             msgs.warn('No good {0}s found!'.format(l))
@@ -1507,7 +1512,7 @@ def peak_trace(flux, ivar=None, bpm=None, trace_map=None, extract_width=None, sm
                 = fit_trace(_flux, trace_peak, order, ivar=ivar, bpm=bpm,
                             trace_bpm=trace_peak_bpm, fwhm=fwhm_uniform, maxshift=maxshift,
                             maxerror=maxerror, function=function, maxdev=maxdev, maxiter=maxiter,
-                            niter=niter_uniform, bitmask=bitmask, debug=debug)
+                            niter=niter_uniform, bitmask=bitmask, debug=show_fits)
 
         # Reset the mask
         # TODO: Use or include `bad` resulting from fit_trace()?
@@ -1521,7 +1526,7 @@ def peak_trace(flux, ivar=None, bpm=None, trace_map=None, extract_width=None, sm
                 = fit_trace(_flux, trace_peak, order, ivar=ivar, bpm=bpm, trace_bpm=trace_peak_bpm,
                             weighting='gaussian', fwhm=fwhm_gaussian, maxshift=maxshift,
                             maxerror=maxerror, function=function, maxdev=maxdev, maxiter=maxiter,
-                            niter=niter_gaussian, bitmask=bitmask, debug=debug)
+                            niter=niter_gaussian, bitmask=bitmask, debug=show_fits)
 
         # Save the results
         fit = np.append(fit, trace_peak, axis=1)
@@ -1570,7 +1575,176 @@ def parse_user_slits(add_slits, this_det, rm=False):
             if det == this_det:
                 user_slits.append([xcen,yrow])
     # Finish
-    if len(user_slits) == 0:
-        return None
-    else:
-        return user_slits
+    return None if len(user_slits) == 0 else user_slits
+
+
+def find_missing_orders(cen, width_fit, gap_fit, tol=0.2):
+    """
+    Using simple models for the order width and order gap as a function of
+    spatial position, identify orders missed by the automated tracing.
+
+    Args:
+        cen (`numpy.ndarray`_):
+            The spatial pixel positions of the orders traced.
+        width_fit (:class:`~pypeit.core.fitting.PypeItFit`):
+            Model of the order width as a function of the order center.
+        gap_fit (:class:`~pypeit.core.fitting.PypeItFit`):
+            Model of the order gap *after* each order as a function of the order
+            center.
+        tol (:obj:`float`, optional):
+            Fraction of the order width used as the tolerance to identify missed
+            orders.
+
+    Returns:
+        :obj:`tuple`: Two arrays providing (1) the centers of all slit orders
+        and (2) a boolean array selecting orders that were missed by the tracing
+        algorithm.
+    """
+    # Start with the first order found
+    c = [cen[0]]
+    missing = [False]
+
+    # Only interpolate; i.e., only iterate through the region covered by
+    # successfully traced orders.
+    i = 1
+    while c[-1] < cen[-1]:
+        # Calculate where the model would predict the next order to land and add
+        # it to the list of centers.
+        l = width_fit.eval(c[-1])
+        c += [c[-1] + l + gap_fit.eval(c[-1])]
+        # Determine if its missing
+        missing += [True if i >= len(cen) else np.absolute(c[-1] - cen[i]) > tol*l]
+        if not missing[-1]:
+            # If not, reset the center to the measured value and increment the
+            # array index
+            c[-1] = cen[i]
+            i += 1
+    # Return arrays
+    return np.array(c), np.array(missing)
+
+
+def predicted_center_difference(lower_spat, spat, width_fit, gap_fit):
+    """
+    Return the difference between the predicted and true location of an order
+    center.
+
+    This is specifically implemented for :func:`extrapolate_orders` and its use
+    of an optimization algorithm to extrapolate the order locations to *lower*
+    spatial pixel values.
+
+    Args:
+        lower_spat (`numpy.ndarray`_):
+            Optimization parameter.  Must be an array with a single element
+            giving the predicted spatial location of the order toward lower
+            spatial pixels compared to the known order (``true``).
+        spat (:obj:`float`):
+            The true spatial location of the known order.
+        width_fit (:class:`~pypeit.core.fitting.PypeItFit`):
+            Model of the order width as a function of the order center.
+        gap_fit (:class:`~pypeit.core.fitting.PypeItFit`):
+            Model of the order gap *after* each order as a function of the order
+            center.
+
+    Returns:
+        :obj:`float`: The absolute value of the difference between the
+        prediction and the measure location of the order.
+    """
+    test_spat = lower_spat[0] + width_fit.eval(lower_spat[0]) + gap_fit.eval(lower_spat[0])
+    return np.absolute(spat - test_spat)
+
+
+def extrapolate_prev_order(cen, width_fit, gap_fit, tol=0.01):
+    """
+    Predict the location of the order to the spatial left (lower spatial pixel
+    numbers) of the order center provided.
+
+    This is largely a support function for :func:`extrapolate_orders`.
+
+    Args:
+        cen (:obj:`float`):
+            Center of the known order.
+        width_fit (:class:`~pypeit.core.fitting.PypeItFit`):
+            Model of the order width as a function of the order center.
+        gap_fit (:class:`~pypeit.core.fitting.PypeItFit`):
+            Model of the order gap *after* each order as a function of the order
+            center.
+        tol (:obj:`float`, optional):
+            Tolerance used when optimizing the order locations predicted toward
+            lower spatial pixels.
+
+    Returns:
+        :obj:`float`: Predicted center of the previous order.
+    """
+    # Guess the position of the previous order
+    w = width_fit.eval(cen)
+    guess = np.array([cen - w - gap_fit.eval(cen)])
+    # Set the bounds based on this guess and the expected order width
+    bounds = optimize.Bounds(lb=guess - w/2, ub=guess + w/2)
+    # Optimize the spatial position
+    res = optimize.minimize(predicted_center_difference, guess,
+                            args=(cen, width_fit, gap_fit),
+                            method='trust-constr', jac='2-point', bounds=bounds, tol=tol)    
+    return res.x[0]
+
+
+def extrapolate_orders(cen, width_fit, gap_fit, min_spat, max_spat, tol=0.01, bracket=False):
+    """
+    Predict the locations of additional orders by extrapolation.
+
+    Order centers are only predicted for those that fall between a minimum and
+    maximum spatial pixel value (see ``min_spat`` and ``max_spat``).
+
+    The models of the order width and gap are defined such that the location of
+    the orders are :math:`c_{i+1} = c_i + w_i + g_i`.  Extrapolation toward
+    larger spatial positions is, therefore, trivial.  Toward smaller pixels,
+    :math:`c_i` is unknown and the width and gap models are not required to be
+    linear; we use a simple minimiation algorithm to optimize the extrapolated
+    values so that the order location model is accurate (see ``tol``).
+
+    Args:
+        cen (`numpy.ndarray`_):
+            The spatial pixel positions of the orders traced.
+        width_fit (:class:`~pypeit.core.fitting.PypeItFit`):
+            Model of the order width as a function of the order center.
+        gap_fit (:class:`~pypeit.core.fitting.PypeItFit`):
+            Model of the order gap *after* each order as a function of the order
+            center.
+        min_spat (:obj:`float`):
+            The minimum spatial pixel for the extrapolation range.
+        max_spat (:obj:`float`):
+            The maximum spatial pixel for the extrapolation range.
+        tol (:obj:`float`, optional):
+            Tolerance used when optimizing the order locations predicted toward
+            lower spatial pixels.
+        bracket (:obj:`bool`, optional):
+            Bracket the added orders with one additional order on either side.
+            This can be useful for dealing with predicted overlap.
+
+    Returns:
+        :obj:`tuple`: Two arrays with orders centers (1) below the first and (2)
+        above the last measured center.  One or both of the arrays can be empty
+        if extrapolation leads to no orders outside the specified minimum and
+        maximum spatial range (``min_spat``, ``max_spat``).
+    """
+    # Extrapolate toward lower spatial positions
+    lower_spat = [cen[0]]
+    while lower_spat[-1] > min_spat:
+        lower_spat += [extrapolate_prev_order(lower_spat[-1], width_fit, gap_fit, tol=tol)]
+
+    # Extrapolate toward larger spatial positions
+    upper_spat = [cen[-1]]
+    while upper_spat[-1] < max_spat:
+        upper_spat += [upper_spat[-1] + width_fit.eval(upper_spat[-1]) 
+                        + gap_fit.eval(upper_spat[-1])]
+
+    # Return arrays after removing the first spatial position because it is a
+    # repeat of the input.  If not bracketting, also remove the last point
+    # because it will be outside the minimum or maximum spatial position (set by
+    # min_spat, max_spat).
+    if bracket:
+        return np.array(lower_spat[-1:0:-1]), np.array(upper_spat[1:])
+    return np.array(lower_spat[-2:0:-1]), np.array(upper_spat[1:-1])
+    
+
+
+
