@@ -1,38 +1,43 @@
-""" Module to generate templates for the PypeIt full_template wavelength calibration routine
+"""
+Module to generate templates for the PypeIt full_template wavelength calibration routine
+
+This function uses direct access to the ``path`` attribute in the relevant
+pypeit data paths.  This should *not* be replicated in other parts of the code.
+This ``templates.py`` script is a developer only script, and it really shouldn't
+be distributed as part of the code base.
 
 .. include:: ../include/links.rst
 """
 
 import os
+import pathlib
+
 import numpy as np
 from IPython import embed
 
 from matplotlib import pyplot as plt
 
-from pkg_resources import resource_filename
-
 from scipy.io import readsav
 from scipy.interpolate import interp1d
 
-
 from astropy.table import Table
 from astropy import units
 
-from linetools import utils as ltu
+import linetools.utils
 
 from pypeit import msgs
 from pypeit import utils
 from pypeit import io
 from pypeit import wavecalib
+from pypeit.core import arc
 from pypeit.core.wave import airtovac
 from pypeit.core.wavecal import waveio
 from pypeit.core.wavecal import wvutils
 from pypeit.core.wavecal import autoid
 from pypeit.core.wavecal import wv_fitting
 from pypeit.core import fitting
-from pypeit import msgs
+from pypeit import dataPaths
 
-from astropy.io import fits
 from pypeit.spectrographs.util import load_spectrograph
 
 # Data Model
@@ -42,26 +47,30 @@ from pypeit.spectrographs.util import load_spectrograph
 #
 # Meta must include BINNING of the template with 1=native
 if os.getenv('PYPEIT_DEV') is not None:
-    template_path = os.path.join(os.getenv('PYPEIT_DEV'), 'dev_algorithms/wavelengths/template_files/')
+    template_path = (
+        pathlib.Path(os.getenv('PYPEIT_DEV')) / 
+        'pypeitdev' / 'wavelengths' / 'template_files'
+    )
 else:
     # print("You may wish to set the PYPEIT_DEV environment variable")
     pass
 
-outpath = resource_filename('pypeit', 'data/arc_lines/reid_arxiv')
+# TODO: Rename the "Master" files here?
 
 
 def build_template(in_files, slits, wv_cuts, binspec, outroot, outdir=None,
                    normalize=False, subtract_conti=False, wvspec=None,
                    lowredux=False, ifiles=None, det_cut=None, chk=False,
                    miny=None, overwrite=True, ascii_tbl=False, in_vac=True,
-                   shift_wave=False, binning=None, micron=False):
+                   shift_wave=False, binning=None, micron=False,
+                   reid_files:bool=False):
     """
     Generate a full_template for a given instrument
 
     Args:
         in_files (list or str):
             Wavelength solution files, XIDL or PypeIt
-            If PypeIt, they can be a mix of MasterWaveCalib JSON and FITS files
+            If PypeIt, they can be a mix of WaveCalib JSON and FITS files
         slits (list):
             Slits in the archive files to use
         wv_cuts (list):
@@ -75,6 +84,8 @@ def build_template(in_files, slits, wv_cuts, binspec, outroot, outdir=None,
             Name of output directory
         lowredux (bool, optional):
             If true, in_files are from LowRedux
+        reid_files (bool, optional):
+            If True, in_files are reid_arxiv files
         wvspec (ndarray, optional):
             Manually input the wavelength values
         ifiles (list, optional):
@@ -104,7 +115,7 @@ def build_template(in_files, slits, wv_cuts, binspec, outroot, outdir=None,
             which is the PypeIt convention. Default=False
     """
     if outdir is None:
-        outdir = outpath
+        outdir = dataPaths.reid_arxiv.path
     if ifiles is None:
         ifiles = np.arange(len(in_files))
     if binning is None:
@@ -126,6 +137,8 @@ def build_template(in_files, slits, wv_cuts, binspec, outroot, outdir=None,
                 wv_vac, spec = xidl_arcspec(in_file, slit)
             elif ascii_tbl:
                 wv_vac, spec = read_ascii(in_file, in_vac=in_vac)
+            elif reid_files:
+                wv_vac, spec = read_reid(in_file)
             else:
                 wv_vac, spec, pypeitFit = pypeit_arcspec(in_file, slit, binspec, binning[kk])
             if micron:
@@ -199,9 +212,6 @@ def build_template(in_files, slits, wv_cuts, binspec, outroot, outdir=None,
         plt.clf()
         ax = plt.gca()
         ax.plot(nwwv, nwspec)
-        # DEBUGGING
-        #wave, flux, bin = waveio.load_template('keck_deimos_1200B.fits', 1)
-        #ax.plot(wave, flux)
         plt.show()
     # Generate the table
     wvutils.write_template(nwwv, nwspec, binspec, outdir, outroot, det_cut=det_cut, overwrite=overwrite)
@@ -237,24 +247,28 @@ def pypeit_arcspec(in_file, slit, binspec, binning=None):
     Load up the arc spectrum from an input JSON file
 
     Args:
-        in_file (str):
+        in_file (str or :obj:`pathlib.Path`):
             File containing the arc spectrum and or fit
         slit (int):
             slit index
 
     Returns:
-        tuple: np.ndarray, np.ndarray, PypeItFit:  wave, flux, pypeitFitting
+        tuple: `numpy.ndarray`_, `numpy.ndarray`_, PypeItFit:  wave, flux, pypeitFitting
 
     """
-    if 'json' in in_file:
-        wv_dict = ltu.loadjson(in_file)
+    # Enforce `str` type for input file name
+    in_file = str(in_file)
+
+    if '.json' in in_file:
+        # Force any possible pathlib.Path object to string before `loadjson`
+        wv_dict = linetools.utils.loadjson(in_file)
         iwv_calib = wv_dict[str(slit)]
         pypeitFitting = fitting.PypeItFit(fitc=np.array(iwv_calib['fitc']),
                                           func=iwv_calib['function'],
                                           minx=iwv_calib['fmin'], maxx=iwv_calib['fmax'])
 
         if binning is not None and binning != binspec:
-            embed(header='Not ready for this yet!')
+            raise NotImplementedError('Cannot rebin yet')
         else:
             x = np.arange(len(iwv_calib['spec']))
 
@@ -262,8 +276,9 @@ def pypeit_arcspec(in_file, slit, binspec, binning=None):
         #wv_vac = utils.func_val(iwv_calib['fitc'], x/iwv_calib['xnorm'], iwv_calib['function'],
         #                   minx=iwv_calib['fmin'], maxx=iwv_calib['fmax'])
         flux = np.array(iwv_calib['spec']).flatten()
-    elif 'fits' in in_file:
-        wvcalib = wavecalib.WaveCalib.from_file(in_file)
+    elif '.fits' in in_file:
+        # TODO: This should not default to chk_version=False ...
+        wvcalib = wavecalib.WaveCalib.from_file(in_file, chk_version=False)
         idx = np.where(wvcalib.spat_ids == slit)[0][0]
         flux = wvcalib.arc_spectra[:,idx]
         #
@@ -283,7 +298,6 @@ def pypeit_arcspec(in_file, slit, binspec, binning=None):
     return wv_vac, flux, pypeitFitting
 
 
-
 def pypeit_identify_record(iwv_calib, binspec, specname, gratname, dispangl, outdir=None):
     """From within PypeIt, generate a template file if the user manually identifies an arc spectrum
 
@@ -312,7 +326,7 @@ def pypeit_identify_record(iwv_calib, binspec, specname, gratname, dispangl, out
     extstr = ""
     while True:
         outroot = '{0:s}_{1:s}_{2:s}{3:s}.fits'.format(specname, gratname, dispangl, extstr)
-        if os.path.exists(os.path.join(outpath, outroot)):
+        if (dataPaths.reid_arxiv.path / outroot).exists():
             extstr = "_{0:02d}".format(cntr)
         else:
             break
@@ -352,12 +366,12 @@ def poly_val(coeff, x, nrm):
     IDL style function for polynomial
 
     Args:
-        coeff (np.ndarray):  Polynomial coefficients
-        x (np.ndarray):  x array
-        nrm (np.ndarray): Normalization terms
+        coeff (`numpy.ndarray`_):  Polynomial coefficients
+        x (`numpy.ndarray`_):  x array
+        nrm (`numpy.ndarray`_): Normalization terms
 
     Returns:
-        np.ndarray:  Same shape as x
+        `numpy.ndarray`_:  Same shape as x
 
     """
     #
@@ -400,15 +414,23 @@ def read_ascii(tbl_file, in_vac=True):
 
 def xidl_arcspec(xidl_file, slit):
     """
-    Read an XIDL format solution
+    Read an XIDL format solution for a Longslit
 
     Note:  These are in air
 
-    Args:
-        xidl_file (str):
-        slit (int):
+    Parameters
+    ----------
+    xidl_file : str
+       XIDL file
+    slit : int
+       The slit in question.
 
-    Returns:
+    Returns
+    -------
+    wave : np.ndarray
+        Wavelengths in vacuum for that slit
+    spec : np.ndarray
+        Arc spectrum for that slit
 
     """
     xidl_dict = readsav(xidl_file)
@@ -438,13 +460,83 @@ def xidl_arcspec(xidl_file, slit):
     # Return
     return wv_vac.value, spec
 
+def read_reid(reid_file:str):
+    # Read
+    tbl = Table.read(reid_file)
+    # Return
+    return tbl['wave'].data, tbl['flux'].data
+
+def xidl_esihires(xidl_file, specbin=1, order_vec=None,
+                  log10=True):
+    """
+    Read an XIDL format solution for Keck/HIRES
+    or Keck/ESI
+
+    Note:  They used air
+
+    Args:
+        xidl_file (str):
+            Keck/HIRES save file
+
+    Returns:
+        tuple: np.ndarray, np.ndarray, np.ndarray  of orders, wavelength, flux
+
+    """
+    xidl_dict = readsav(xidl_file)
+    if order_vec is None:
+        order_vec = xidl_dict['guess_ordr']
+    norders = order_vec.size
+    nspec = xidl_dict['sv_aspec'].shape[1]
+
+    # Wavelengths
+    wave = np.zeros((norders, specbin*nspec))
+    spec = np.zeros((norders, specbin*nspec))
+
+    calib = xidl_dict['all_arcfit']
+    order_mask = np.ones(norders, dtype=bool)
+
+    # Here we go on the fits
+    for kk in range(norders):
+        # Generate the wavelengths
+        if calib['FUNC'][kk] == b'CHEBY':
+            log10_wv_air = cheby_val(calib['FFIT'][kk], 
+                               np.arange(nspec),
+                        calib['NRM'][kk], calib['NORD'][kk])
+        elif calib['FUNC'][kk] == b'POLY':
+            log10_wv_air = poly_val(calib['FFIT'][kk], 
+                              np.arange(nspec),
+                              calib['NRM'][kk])
+        else:
+            order_mask[kk]=False
+            continue
+        if not log10:
+            log10_wv_air = np.log10(log10_wv_air)
+
+        wv_vac = airtovac(10**log10_wv_air * units.AA).value
+        ispec = xidl_dict['sv_aspec'][kk,:]
+        # Flip to blue to red?
+        if wv_vac[1] < wv_vac[0]:
+            wv_vac = wv_vac[::-1]
+            ispec = ispec[::-1]
+        # Fill
+        if specbin != 1:
+            wave[kk,:] = arc.resize_spec(wv_vac, nspec*specbin)
+            spec[kk,:] = arc.resize_spec(ispec, nspec*specbin)
+        else:
+            wave[kk,:] = wv_vac
+            spec[kk,:] = ispec
+    # Return
+
+    return order_vec[order_mask], wave[order_mask,:], spec[order_mask,:]
+
+
 def main(flg):
 
     # Keck LRISb
     if flg & (2**0): # B300, all lamps
         binspec = 1
         slits = [15]
-        xidl_file = os.path.join(template_path, 'Keck_LRIS', 'B300', 'lris_blue_300.sav')
+        xidl_file = template_path / 'Keck_LRIS' / 'B300' / 'lris_blue_300.sav'
         outroot = 'keck_lris_blue_300_d680.fits'
         build_template(xidl_file, slits, None, binspec, outroot, lowredux=True)
 
@@ -453,7 +545,7 @@ def main(flg):
         outroot='keck_lris_blue_400_d560.fits'
         slits = [19,14]
         lcut = [5500.]
-        xidl_file = os.path.join(template_path, 'Keck_LRIS', 'B400', 'lris_blue_400_d560.sav')
+        xidl_file = template_path / 'Keck_LRIS' / 'B400' / 'lris_blue_400_d560.sav'
         build_template(xidl_file, slits, lcut, binspec, outroot, lowredux=True)
 
     if flg & (2**2): # B600, all lamps
@@ -461,7 +553,7 @@ def main(flg):
         outroot='keck_lris_blue_600_d560.fits'
         slits = [0,7]
         lcut = [4500.]
-        wfile = os.path.join(template_path, 'Keck_LRIS', 'B600', 'MasterWaveCalib_A_1_01.json')
+        wfile = template_path / 'Keck_LRIS' / 'B600' / 'MasterWaveCalib_A_1_01.json'
         build_template(wfile, slits, lcut, binspec, outroot, lowredux=False)
 
     if flg & (2**3): # B1200, all lamps?
@@ -469,7 +561,7 @@ def main(flg):
         outroot='keck_lris_blue_1200_d460.fits'
         slits = [19,44]
         lcut = [3700.]
-        xidl_file = os.path.join(template_path, 'Keck_LRIS', 'B1200', 'lris_blue_1200.sav')
+        xidl_file = template_path / 'Keck_LRIS' / 'B1200' /'lris_blue_1200.sav'
         build_template(xidl_file, slits, lcut, binspec, outroot, lowredux=True)
 
     # ###############################################3
@@ -479,7 +571,7 @@ def main(flg):
         outroot='keck_lris_red_400.fits'
         slits = [7]  # Quite blue, but not the bluest
         lcut = []
-        wfile = os.path.join(template_path, 'Keck_LRIS', 'R400', 'MasterWaveCalib_A_1_01.json')
+        wfile = template_path / 'Keck_LRIS' / 'R400' / 'MasterWaveCalib_A_1_01.json'
         build_template(wfile, slits, lcut, binspec, outroot, lowredux=False)
 
     if flg & (2**11):  # R1200
@@ -490,8 +582,8 @@ def main(flg):
         ifiles = [0, 1]
         slits = [3, 7]
         lcut = [9250.]
-        wfile1 = os.path.join(template_path, 'Keck_LRIS', 'R1200_9000', 'MasterWaveCalib_A_1_02.json')  # Original Dev
-        wfile2 = os.path.join(template_path, 'Keck_LRIS', 'R1200_9000', 'MasterWaveCalib_A_1_01.json')  # Dev suite 2x1
+        wfile1 = template_path / 'Keck_LRIS' / 'R1200_9000' / 'MasterWaveCalib_A_1_02.json'  # Original Dev
+        wfile2 = template_path / 'Keck_LRIS' / 'R1200_9000' / 'MasterWaveCalib_A_1_01.json'  # Dev suite 2x1
         build_template([wfile1,wfile2], slits, lcut, binspec, outroot, lowredux=False,
                        ifiles=ifiles)
 
@@ -502,7 +594,7 @@ def main(flg):
         outroot='keck_lris_red_600_5000.fits'
         slits = [4, 7]
         lcut = [7820.]
-        wfile = os.path.join(template_path, 'Keck_LRIS', 'R600_5000', 'MasterWaveCalib_B_1_01.json')
+        wfile = template_path / 'Keck_LRIS' / 'R600_5000' /'MasterWaveCalib_B_1_01.json'
         build_template(wfile, slits, lcut, binspec, outroot, lowredux=False)
 
     if flg & (2**27):  # R600/7500
@@ -512,7 +604,7 @@ def main(flg):
         outroot='keck_lris_red_600_7500.fits'
         slits = [10, 4]
         lcut = [7840.]
-        wfile = os.path.join(template_path, 'Keck_LRIS', 'R600_7500', 'MasterWaveCalib_I_1_01.json')
+        wfile = template_path / 'Keck_LRIS' / 'R600_7500' / 'MasterWaveCalib_I_1_01.json'
         build_template(wfile, slits, lcut, binspec, outroot, lowredux=False,
                        chk=True, normalize=True, subtract_conti=True)
 
@@ -520,8 +612,8 @@ def main(flg):
     # Magellan/MagE
     if flg & (2**13):
         # Load
-        mase_path = os.path.join(os.getenv('XIDL_DIR'), 'Magellan', 'MAGE', 'mase', 'Calib')
-        sav_file = os.path.join(mase_path, 'MagE_wvguess_jfh.idl')
+        mase_path = pathlib.Path(os.getenv('XIDL_DIR')) / 'Magellan' / 'MAGE' / 'mase' / 'Calib'
+        sav_file = mase_path / 'MagE_wvguess_jfh.idl'
         mase_dict = readsav(sav_file)
         mase_sol = Table(mase_dict['all_arcfit'])
         # Do it
@@ -541,14 +633,12 @@ def main(flg):
         tbl.meta['BINSPEC'] = 1
         # Write
         outroot='magellan_mage.fits'
-        outfile = os.path.join(template_path, outroot)
+        outfile = template_path / outroot
         tbl.write(outfile, overwrite=True)
         print("Wrote: {}".format(outfile))
 
     if flg & (2**14):  # Magellan/MagE Plots
-        outpath = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'plots')
-        new_mage_file = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv',
-                                     'magellan_mage.fits')
+        new_mage_file = dataPaths.reid_arxiv.path / 'magellan_mage.fits'
         # Load
         mage_wave = Table.read(new_mage_file)
         llist = waveio.load_line_lists(['ThAr_MagE'])
@@ -562,18 +652,20 @@ def main(flg):
             # Fit
             final_fit = wv_fitting.fit_slit(fx, patt_dict, detections, llist)
             # Output
-            outfile=os.path.join(outpath, 'MagE_order{:2d}_IDs.pdf'.format(order))
+            outfile=dataPaths.arc_plot.path / f'MagE_order{order:2d}_IDs.pdf'
             autoid.arc_fit_qa(final_fit, outfile=outfile, ids_only=True)
-            print("Wrote: {}".format(outfile))
-            autoid.arc_fit_qa(final_fit, outfile=os.path.join(outpath, 'MagE_order{:2d}_full.pdf'.format(order)))
+            print(f"Wrote: {outfile}")
+            autoid.arc_fit_qa(
+                final_fit,
+                outfile=dataPaths.arc_plot.path / f'MagE_order{order:2d}_full.pdf'
+            )
 
     if flg & (2**15):  # VLT/X-Shooter reid_arxiv
         # VIS
-        reid_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv')
         for iroot, iout in zip(['vlt_xshooter_vis1x1.json', 'vlt_xshooter_nir.json'],
             ['vlt_xshooter_vis1x1.fits', 'vlt_xshooter_nir.fits']):
             # Load
-            old_file = os.path.join(reid_path, iroot)
+            old_file = dataPaths.reid_arxiv.path / iroot
             odict, par = waveio.load_reid_arxiv(old_file)
 
             # Do it
@@ -593,13 +685,12 @@ def main(flg):
             tbl['order'] = orders
             tbl.meta['BINSPEC'] = 1
             # Write
-            outfile = os.path.join(reid_path, iout)
+            outfile = dataPaths.reid_arxiv.path / iout
             tbl.write(outfile, overwrite=True)
             print("Wrote: {}".format(outfile))
 
     if flg & (2**16):  # VLT/X-Shooter line list
-        line_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'lists')
-        old_file = os.path.join(line_path, 'ThAr_XSHOOTER_VIS_air_lines.dat')
+        old_file = dataPaths.linelist.path / 'ThAr_XSHOOTER_VIS_air_lines.dat'
         # Load
         air_list = waveio.load_line_list(old_file)
         # Vacuum
@@ -607,16 +698,15 @@ def main(flg):
         vac_list = air_list.copy()
         vac_list['wave'] = vac_wv
         # Write
-        new_file = os.path.join(line_path, 'ThAr_XSHOOTER_VIS_lines.dat')
+        new_file = dataPaths.linelist.path / 'ThAr_XSHOOTER_VIS_lines.dat'
         vac_list.write(new_file, format='ascii.fixed_width', overwrite=True)
         print("Wrote: {}".format(new_file))
 
     if flg & (2**17):  # NIRES
-        reid_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv')
         iroot = 'keck_nires.json'
         iout = 'keck_nires.fits'
         # Load
-        old_file = os.path.join(reid_path, iroot)
+        old_file = dataPaths.reid_arxiv.path / iroot
         odict, par = waveio.load_reid_arxiv(old_file)
 
         # Do it
@@ -636,17 +726,16 @@ def main(flg):
         tbl['order'] = orders
         tbl.meta['BINSPEC'] = 1
         # Write
-        outfile = os.path.join(reid_path, iout)
+        outfile = dataPaths.reid_arxiv.path / iout
         tbl.write(outfile, overwrite=True)
         print("Wrote: {}".format(outfile))
 
 
     if flg & (2**18):  # Gemini/GNIRS
-        reid_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv')
         iroot = 'gemini_gnirs.json'
         iout = 'gemini_gnirs.fits'
         # Load
-        old_file = os.path.join(reid_path, iroot)
+        old_file = dataPaths.reid_arxiv.path / iroot
         odict, par = waveio.load_reid_arxiv(old_file)
 
         # Do it
@@ -666,7 +755,7 @@ def main(flg):
         tbl['order'] = orders
         tbl.meta['BINSPEC'] = 1
         # Write
-        outfile = os.path.join(reid_path, iout)
+        outfile = dataPaths.reid_arxiv.path / iout
         tbl.write(outfile, overwrite=True)
         print("Wrote: {}".format(outfile))
 
@@ -674,18 +763,17 @@ def main(flg):
     if flg & (2**23):  # WHT/ISIS
         iroot = 'wht_isis_blue_1200_4800.json'
         outroot = 'wht_isis_blue_1200_4800.fits'
-        wfile = os.path.join(template_path, 'WHT_ISIS', '1200B', iroot)
+        wfile = template_path / 'WHT_ISIS' / '1200B' / iroot
         binspec = 2
         slits = [0]
         lcut = [3200.]
         build_template(wfile, slits, lcut, binspec, outroot, lowredux=False)
 
     if flg & (2**24):  # Magellan/FIRE
-        reid_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv')
         iroot = 'magellan_fire_echelle.json'
         iout = 'magellan_fire_echelle.fits'
         # Load
-        old_file = os.path.join(reid_path, iroot)
+        old_file = dataPaths.reid_arxiv.path / iroot
         odict, par = waveio.load_reid_arxiv(old_file)
 
         # Do it
@@ -706,40 +794,38 @@ def main(flg):
         tbl['order'] = orders
         tbl.meta['BINSPEC'] = 1
         # Write
-        outfile = os.path.join(reid_path, iout)
+        outfile = dataPaths.reid_arxiv.path / iout
         tbl.write(outfile, overwrite=True)
         print("Wrote: {}".format(outfile))
 
     if flg & (2**25): # FIRE longslit
         binspec = 1
-        reid_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv')
         outroot = 'magellan_fire_long.fits'
-        xidl_file = os.path.join(os.getenv('FIRE_DIR'), 'LowDispersion', 'NeNeAr_archive_fit.fits')
-        spec_file = os.path.join(os.getenv('FIRE_DIR'), 'LowDispersion', 'NeNeAr2.sav')
+        xidl_file = pathlib.Path(os.getenv('FIRE_DIR')) / 'LowDispersion' / 'NeNeAr_archive_fit.fits'
+        spec_file = pathlib.Path(os.getenv('FIRE_DIR')) / 'LowDispersion' / 'NeNeAr2.sav'
         fire_sol = Table.read(xidl_file)
         wave = cheby_val(fire_sol['FFIT'].data[0], np.arange(2048), fire_sol['NRM'].data[0], fire_sol['NORD'].data[0])
         wv_vac = airtovac(wave * units.AA)
         xidl_dict = readsav(spec_file)
         flux = xidl_dict['arc1d']
-        wvutils.write_template(wv_vac.value, flux, binspec, reid_path, outroot, det_cut=None)
+        wvutils.write_template(wv_vac.value, flux, binspec, dataPaths.reid_arxiv.path, outroot, det_cut=None)
 
     # Gemini/Flamingos2
     if flg & (2**26):
-        reid_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv')
         iroot = ['Flamingos2_JH_JH.json','Flamingos2_HK_HK.json']
         outroot=['Flamingos2_JH_JH.fits','Flamingos2_HK_HK.fits']
         binspec = 1
         slits = [0]
         lcut = []
         for ii in range(len(iroot)):
-            wfile = os.path.join(reid_path, iroot[ii])
+            wfile = dataPaths.reid_arxiv.path / iroot[ii]
             build_template(wfile, slits, lcut, binspec, outroot[ii], lowredux=False)
 
 
     # MDM/OSMOS -- MDM4K
     if flg & (2 ** 28):
         # ArI 4159 -- 6800
-        wfile = os.path.join(template_path, 'MDM_OSMOS', 'MasterWaveCalib_MDM4K_01.json')
+        wfile = template_path / 'MDM_OSMOS' / 'MasterWaveCalib_MDM4K_01.json'
         outroot = 'mdm_osmos_mdm4k.fits'
         binspec = 1
         slits = [0]
@@ -747,73 +833,32 @@ def main(flg):
         build_template(wfile, slits, lcut, binspec, outroot, lowredux=False,
                        chk=True, subtract_conti=True)
 
-    # Keck KCWI
-    if flg & (2 ** 29):
-        # FeAr BH2
-        wfile1 = os.path.join(template_path, 'KCWI', 'BH2', 'Keck_KCWI_BH2_4200.json')
-        outroot = 'keck_kcwi_BH2_4200.fits'
-        binspec = 1
-        slits = [1015]
-        lcut = [4350.0, 8000.0]
-        build_template([wfile1], slits, lcut, binspec, outroot, lowredux=False, normalize=True)
-        # FeAr BM
-        wfile1 = os.path.join(template_path, 'KCWI', 'BM', 'Keck_KCWI_BM_4060.json')
-        wfile2 = os.path.join(template_path, 'KCWI', 'BM', 'Keck_KCWI_BM_4670.json')
-        outroot = 'keck_kcwi_BM.fits'
-        binspec = 1
-        slits = [1026, 1021]
-        lcut = [4350.0, 8000.0]
-        build_template([wfile1, wfile2], slits, lcut, binspec, outroot, lowredux=False, normalize=True)
-
-    # P200 DBSP r
-    if flg & (2 ** 30):
-        # HeNeAr
-        wfile = os.path.join(template_path, 'P200_DBSP', 'R316_7500_D55', 'P200_DBSP_Red.json')
-        outroot = 'p200_dbsp_red_316_7500_d55.fits'
-        binspec = 1
-        slits = [221]
-        lcut = None # only matters if >1 slit
-        build_template([wfile], slits, lcut, binspec, outroot, lowredux=False, normalize=True)
-
-    # P200 DBSP b
-    if flg & (2 ** 31):
-        # FeAr
-        wfile = os.path.join(template_path, 'P200_DBSP', 'B600_4000_D55', 'P200_DBSP_Blue.json')
-        outroot = 'p200_dbsp_blue_600_4000_d55.fits'
-        binspec = 1
-        slits = [231]
-        lcut = None
-        build_template([wfile], slits, lcut, binspec, outroot, lowredux=False, normalize=True)
-
     # MMT/MMIRS
     if flg & (2**32):
-        reid_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv')
         iroot = ['mmt_mmirs_HK_zJ.json','mmt_mmirs_J_zJ.json','mmt_mmirs_K3000_Kspec.json']
         outroot=['mmt_mmirs_HK_zJ.fits','mmt_mmirs_J_zJ.fits','mmt_mmirs_K3000_Kspec.fits']
         binspec = 1
         slits = [1020,1020,1020]
         lcut = []
         for ii in range(len(iroot)):
-            wfile = os.path.join(reid_path, iroot[ii])
+            wfile = dataPaths.reid_arxiv.path / iroot[ii]
             build_template(wfile, slits, lcut, binspec, outroot[ii], lowredux=False)
     # LBT/MODS
     if flg & (2**33):
-        reid_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv')
         iroot = ['lbt_mods1r_red.json','lbt_mods2r_red.json']
         outroot=['lbt_mods1r_red.fits','lbt_mods2r_red.fits']
         binspec = 1
         slits = [[1557],[1573]]
         lcut = []
         for ii in range(len(iroot)):
-            wfile = os.path.join(reid_path, iroot[ii])
+            wfile = dataPaths.reid_arxiv.path / iroot[ii]
             build_template(wfile, slits[ii], lcut, binspec, outroot[ii], lowredux=False)
     # P200 Triplespec
     if flg & (2**34):
-        reid_path = os.path.join(resource_filename('pypeit', 'data'), 'arc_lines', 'reid_arxiv')
         iroot = 'p200_triplespec_MasterWaveCalib.fits'
         iout = 'p200_triplespec.fits'
         # Load
-        old_file = os.path.join(reid_path, iroot)
+        old_file = dataPaths.reid_arxiv.path / iroot
         par = io.fits_open(old_file)
         pyp_spec = par[0].header['PYP_SPEC']
         spectrograph  = load_spectrograph(pyp_spec)
@@ -832,7 +877,7 @@ def main(flg):
         tbl['order'] = orders
         tbl.meta['BINSPEC'] = 1
         # Write
-        outfile = os.path.join(reid_path, iout)
+        outfile = dataPaths.reid_arxiv.path / iout
         tbl.write(outfile, overwrite=True)
         print("Wrote: {}".format(outfile))
 
@@ -888,15 +933,6 @@ if __name__ == '__main__':
     # MDM/OSMMOS
     #flg += 2**28
 
-    # Keck KCWI
-    #flg += 2**29
-
-    # P200 DBSP r
-    #flg += 2**30
-
-    # P200 DBSP b
-    #flg += 2**31
-
     # MMT MMIRS
     #flg += 2**32
 
