@@ -38,35 +38,40 @@ from pypeit import inputfiles
 
 def get_report_metadata(object_header_keys, spec_obj_keys, file_info):
     """
-    Gets the metadata from a SourceObject instance used building a report
-    on the results of collation.  It is intended to be wrapped in by functools
-    partial object that passes in object_header_keys and spec_obj_keys. file_info
-    is then passed as in by the :obj:`pypeit.archive.ArchiveMetadata` object.
-    Unlike the other get_*_metadata functions, this is not used for archiving; it is
-    used for reporting on the results of collating.
+    Gets the metadata from a :class:`~pypeit.core.collate.SourceObject` instance
+    used building a report on the results of collation.  It is intended to be
+    wrapped in by functools partial object that passes in object_header_keys and
+    spec_obj_keys. file_info is then passed as in by the
+    :class:`~pypeit.archive.ArchiveMetadata` object.  Unlike the other
+    get_*_metadata functions, this is not used for archiving; it is used for
+    reporting on the results of collating.
 
-    If another type of file is added to the ArchiveMetadata object, the file_info
-    argument will not be a SourceObject, In this case, a list of ``None`` values are 
-    returned.
+    If another type of file is added to the ArchiveMetadata object, the
+    file_info argument will not be a :class:`~pypeit.core.collate.SourceObject`,
+    In this case, a list of ``None`` values are returned.
 
     Args:
         object_header_keys (list of str):
             The keys to read fom the spec1d headers from the SourceObject.
 
         spec_obj_keys (list of str):
-            The keys to read from the (:obj:`pypeit.specobj.SpecObj`) objects in the SourceObject.
+            The keys to read from the (:class:`~pypeit.specobj.SpecObj`) objects in
+            the SourceObject.
 
-        file_info (:obj:`pypeit.scripts.collate_1d.SourceObject`)): 
-            The source object containing the headers, filenames and SpecObj information for a coadd output file.
+        file_info (:class:`~pypeit.core.collate.SourceObject`): 
+            The source object containing the headers, filenames and SpecObj
+            information for a coadd output file.
 
     Returns:
         tuple: A tuple of two lists:.
 
-               **data_rows** (:obj:`list` of :obj:`list`): The metadata rows built from the source object.
+            - **data_rows** (:obj:`list` of :obj:`list`): The metadata rows
+              built from the source object.
+
+            - **files_to_copy** (iterable): An list of tuples of files to copy.
+              Because this function is not used for archving data, this is
+              always ``None``.
 
-               **files_to_copy** (iterable):
-               An list of tuples of files to copy. Because this function is not used for
-               archving data, this is always ``None``.
     """
 
     if not isinstance(file_info, SourceObject):
@@ -81,7 +86,7 @@ def get_report_metadata(object_header_keys, spec_obj_keys, file_info):
         header = file_info.spec1d_header_list[i]
 
         # Get the spec1d header metadata needed for the report
-        # Use getattr for the spec_obj data because one of the attributes is actually a property (med_s2n)
+        # Use getattr for the spec_obj data
         spec_obj_data = [getattr(spec_obj, x) for x in spec_obj_keys]
         spec1d_filename =  os.path.basename(file_info.spec1d_file_list[i])
         header_data = [header[x] if x in header else None for x in object_header_keys]
@@ -100,7 +105,7 @@ def find_slits_to_exclude(spec2d_files, par):
     Args:
         spec2d_files (:obj:`list`): 
             List of spec2d files to build the map from.
-        par (:class:`~pypeit.par.pypeitpar.Collate1DPar`):
+        par (:class:`~pypeit.par.pypeitpar.PypeItPar`):
             Parameters from a ``.collate1d`` file
 
     Returns:
@@ -119,11 +124,11 @@ def find_slits_to_exclude(spec2d_files, par):
     exclude_map = dict()
     for spec2d_file in spec2d_files:
 
-        allspec2d = AllSpec2DObj.from_fits(spec2d_file)
+        allspec2d = AllSpec2DObj.from_fits(spec2d_file, chk_version=par['rdx']['chk_version'])
         for sobj2d in [allspec2d[det] for det in allspec2d.detectors]:
             for (slit_id, mask, slit_mask_id) in sobj2d['slits'].slit_info:
                 for flag in exclude_flags:
-                    if bit_mask.flagged(mask, flag):
+                    if bit_mask.flagged(mask, flag=flag):
                         if slit_mask_id not in exclude_map:
                             exclude_map[slit_mask_id] = {flag}
                         else:
@@ -151,11 +156,11 @@ def exclude_source_objects(source_objects, exclude_map, par):
     Returns:
         tuple: Tuple containing two lists:
 
-               **filtered_objects** (:obj:`list`): A list of :class:`~pypeit.core.collate.SourceObject` 
-               with any excluded ones removed.
+            - **filtered_objects** (:obj:`list`): A list of
+              :class:`~pypeit.core.collate.SourceObject` with any excluded ones
+              removed.
 
-               **missing_archive_msgs** (:obj:`list`): A list of messages explaining why some source 
-               objects were excluded.
+            - **excluded_messages** (:obj:`list`): A list of messages explaining why some source objects were excluded.
     """
     filtered_objects = []
     excluded_messages= []
@@ -188,29 +193,80 @@ def exclude_source_objects(source_objects, exclude_map, par):
             excluded_messages.append(msg)
             continue
 
-        if par['coadd1d']['ex_value'] == 'OPT' and sobj.OPT_COUNTS is None:
-            msg = f'Excluding {sobj.NAME} in {spec1d_file} because of missing OPT_COUNTS. Consider changing ex_value to "BOX".'
-            msgs.warn(msg)
-            excluded_messages.append(msg)
-            continue
+        if par['coadd1d']['ex_value'] == 'OPT':
+            msg = None
+            if sobj.OPT_COUNTS is None:
+                msg = f'Excluding {sobj.NAME} in {spec1d_file} because of missing OPT_COUNTS. Consider changing ex_value to "BOX".'
+            elif sobj.OPT_MASK is None:
+                msg = f'Excluding {sobj.NAME} in {spec1d_file} because of missing OPT_MASK. Consider changing ex_value to "BOX".'
+            else:
+                if len(sobj.OPT_COUNTS[sobj.OPT_MASK]) == 0:
+                    msg = f'Excluding {sobj.NAME} in {spec1d_file} because all of OPT_COUNTS was masked out. Consider changing ex_value to "BOX".'
+            
+            if msg is not None:
+                msgs.warn(msg)
+                excluded_messages.append(msg)
+                continue
 
-        if par['coadd1d']['ex_value'] == 'BOX' and sobj.BOX_COUNTS is None:
-            msg = f'Excluding {sobj.NAME} in {spec1d_file} because of missing BOX_COUNTS. Consider changing ex_value to "OPT".'
-            msgs.warn(msg)
-            excluded_messages.append(msg)
-            continue
+        if par['coadd1d']['ex_value'] == 'BOX':
+            msg = None
+            if sobj.BOX_COUNTS is None:
+                msg = f'Excluding {sobj.NAME} in {spec1d_file} because of missing BOX_COUNTS. Consider changing ex_value to "OPT".'
+            elif sobj.BOX_MASK is None:
+                msg = f'Excluding {sobj.NAME} in {spec1d_file} because of missing BOX_MASK. Consider changing ex_value to "OPT".'
+            else:
+                if len(sobj.BOX_COUNTS[sobj.BOX_MASK]) == 0:
+                    msg = f'Excluding {sobj.NAME} in {spec1d_file} because all of BOX_COUNTS was masked out. Consider changing ex_value to "OPT".'
+
+            if msg is not None:
+                msgs.warn(msg)
+                excluded_messages.append(msg)
+                continue
 
         filtered_objects.append(source_object)
     return (filtered_objects, excluded_messages)
 
+def read_spec1d_files(par, spec1d_files, failure_msgs):
+    """
+    Read spec1d files.
+
+    Args:
+        par (`obj`:pypeit.par.pypeitpar.PypeItPar): 
+            Parameters for collating, fluxing, and coadding.
+        spec1d_files (list of str):
+            List of spec1d files to read.
+        failure_msgs(list of str):
+            Return parameter describing any failures that occurred when reading.
+
+    Returns:
+        list of str: The SpecObjs objects that were successfully read.
+        list of str: The spec1d files that were successfully read.
+    """
+    
+    specobjs_list = []
+    good_spec1d_files = []
+    for spec1d_file in spec1d_files:
+        try:
+            sobjs = SpecObjs.from_fitsfile(spec1d_file, chk_version=par['rdx']['chk_version'])
+            specobjs_list.append(sobjs)
+            good_spec1d_files.append(spec1d_file)
+        except Exception as e:
+            formatted_exception = traceback.format_exc()
+            msgs.warn(formatted_exception)
+            msgs.warn(f"Failed to read {spec1d_file}, skipping it.")
+            failure_msgs.append(f"Failed to read {spec1d_file}, skipping it.")
+            failure_msgs.append(formatted_exception)
+
+    return specobjs_list, good_spec1d_files
+
 def flux(par, spectrograph, spec1d_files, failed_fluxing_msgs):
     """
     Flux calibrate spec1d files using archived sens func files.
 
     Args:
-        par (`obj`:pypeit.par.pypeitpar.PypeItPar): 
+        par (:class:`~pypeit.par.pypeitpar.PypeItPar`): 
             Parameters for collating, fluxing, and coadding.
-        spectrograph (`obj`:pypeit.spectrographs.spectrograph):
+        spectrograph (:class:`~pypeit.spectrographs.spectrograph.Spectrograph`):
             Spectrograph for the files to flux.
         spec1d_files (list of str):
             List of spec1d files to flux calibrate.
@@ -240,12 +296,13 @@ def flux(par, spectrograph, spec1d_files, failed_fluxing_msgs):
             msgs.warn(f"Could not find archived sensfunc to flux {spec1d_file}, skipping it.")
             failed_fluxing_msgs.append(f"Could not find archived sensfunc to flux {spec1d_file}, skipping it.")
             failed_fluxing_msgs.append(formatted_exception)
+            continue
             
         # Flux calibrate the spec1d file
         try:
             msgs.info(f"Running flux calibrate on {spec1d_file}")
-            FxCalib = fluxcalibrate.FluxCalibrate.get_instance([spec1d_file], [sens_file],
-                                                                par=par['fluxcalib'])
+            FxCalib = fluxcalibrate.flux_calibrate([spec1d_file], [sens_file], par=par['fluxcalib'],
+                                                   chk_version=par['rdx']['chk_version'])
             flux_calibrated_files.append(spec1d_file)
 
         except Exception:
@@ -254,6 +311,7 @@ def flux(par, spectrograph, spec1d_files, failed_fluxing_msgs):
             msgs.warn(f"Failed to flux calibrate {spec1d_file}, skipping it.")
             failed_fluxing_msgs.append(f"Failed to flux calibrate {spec1d_file}, skipping it.")
             failed_fluxing_msgs.append(formatted_exception)
+            continue
 
     # Return the succesfully fluxed files
     return flux_calibrated_files
@@ -295,7 +353,7 @@ def refframe_correction(par, spectrograph, spec1d_files, spec1d_failure_msgs):
     for spec1d in spec1d_files:
         # Get values from the fits header needed to calculate the correction
         try:
-            sobjs = SpecObjs.from_fitsfile(spec1d)
+            sobjs = SpecObjs.from_fitsfile(spec1d, chk_version=par['rdx']['chk_version'])
             hdr_ra = sobjs.header['RA']
             hdr_dec = sobjs.header['DEC']
             hdr_radec = ltu.radec_to_coord((hdr_ra, hdr_dec))
@@ -359,9 +417,11 @@ def coadd(par, coaddfile, source):
     """coadd the spectra for a given source.
 
     Args:
-        par (`obj`:Collate1DPar): Paramters for the coadding
-        source (`obj`:SourceObject): The SourceObject with information on
-            which files and spectra to coadd.
+        par (:class:`~pypeit.par.pypeitpar.Collate1DPar`):
+            Parameters for the coadding
+        source (:class:`~pypeit.core.collate.SourceObject`):
+            The SourceObject with information on which files and spectra to
+            coadd.
     """
     # Set destination file for coadding
     par['coadd1d']['coaddfile'] = coaddfile
@@ -434,7 +494,7 @@ def write_warnings(par, excluded_obj_msgs, failed_source_msgs, spec1d_failure_ms
         failed_source_msgs (:obj:`list` of :obj:`str`): 
             Messages about which objects failed coadding and why.
 
-        spec1d_failure_msgs (:obj:)`list` of :obj:`str`): 
+        spec1d_failure_msgs (:obj:`list` of :obj:`str`): 
             Messages about failures with spec1d files and why.
 
     """
@@ -520,8 +580,8 @@ def build_parameters(args):
     if args.match_using is not None:
         params['collate1d']['match_using'] = args.match_using
 
-    if args.exclude_slit_bm is not None and len(args.exclude_slit_bm) > 0:
-        params['collate1d']['exclude_slit_trace_bm'] = args.exclude_slit_bm
+    if args.exclude_slit_trace_bm is not None and len(args.exclude_slit_trace_bm) > 0:
+        params['collate1d']['exclude_slit_trace_bm'] = args.exclude_slit_trace_bm.split(',')
 
     if args.exclude_serendip:
         params['collate1d']['exclude_serendip'] = True
@@ -547,6 +607,9 @@ def build_parameters(args):
     if args.refframe is not None:
         params['collate1d']['refframe'] = args.refframe
 
+    if args.chk_version is True:
+        params['rdx']['chk_version'] = True
+
     return params, spectrograph, spec1d_files
 
 def create_report_archive(par):
@@ -570,8 +633,8 @@ def create_report_archive(par):
     COADDED_SPEC1D_HEADER_KEYS  = ['DISPNAME', 'DECKER',   'BINNING', 'MJD', 'AIRMASS', 'EXPTIME','GUIDFWHM', 'PROGPI', 'SEMESTER', 'PROGID']
     COADDED_SPEC1D_COLUMN_NAMES = ['dispname', 'slmsknam', 'binning', 'mjd', 'airmass', 'exptime','guidfwhm', 'progpi', 'semester', 'progid']
 
-    COADDED_SOBJ_KEYS  =        ['MASKDEF_OBJNAME', 'MASKDEF_ID', 'NAME',        'DET', 'RA',    'DEC',    'med_s2n', 'MASKDEF_EXTRACT', 'WAVE_RMS']
-    COADDED_SOBJ_COLUMN_NAMES = ['maskdef_objname', 'maskdef_id', 'pypeit_name', 'det', 'objra', 'objdec', 's2n',     'maskdef_extract', 'wave_rms']
+    COADDED_SOBJ_KEYS  =        ['MASKDEF_OBJNAME', 'MASKDEF_ID', 'NAME',        'DET', 'RA',    'DEC',    'MASKDEF_OBJMAG', 'MASKDEF_OBJMAG_BAND', 'S2N', 'MASKDEF_EXTRACT', 'WAVE_RMS']
+    COADDED_SOBJ_COLUMN_NAMES = ['maskdef_objname', 'maskdef_id', 'pypeit_name', 'det', 'objra', 'objdec', 'maskdef_objmag', 'maskdef_objmag_band', 's2n', 'maskdef_extract', 'wave_rms']
 
     report_names = ['filename'] + \
                    COADDED_SOBJ_COLUMN_NAMES + \
@@ -599,7 +662,8 @@ class Collate1D(scriptbase.ScriptBase):
     @classmethod
     def get_parser(cls, width=None):
         # A blank Colate1DPar to avoid duplicating the help text.
-        blank_par = pypeitpar.Collate1DPar()
+        blank_pypar = pypeitpar.PypeItPar()
+        blank_par = blank_pypar['collate1d']
 
         parser = super().get_parser(description='Flux/Coadd multiple 1d spectra from multiple '
                                                 'nights and prepare a directory for the KOA.',
@@ -648,13 +712,14 @@ class Collate1D(scriptbase.ScriptBase):
         parser.add_argument('--dry_run', action='store_true', help=blank_par.descr['dry_run'])
         parser.add_argument('--ignore_flux', default=False, action='store_true', help=blank_par.descr['ignore_flux'])
         parser.add_argument('--flux', default=False, action = 'store_true', help=blank_par.descr['flux'])
-        parser.add_argument('--exclude_slit_bm', type=str, nargs='*',
-                            help=blank_par.descr['exclude_slit_trace_bm'])
+        parser.add_argument('--exclude_slit_trace_bm', type=str, # nargs='*',
+                            help=blank_par.descr['exclude_slit_trace_bm']+' Comma separated.')
         parser.add_argument('--exclude_serendip', action='store_true',
                             help=blank_par.descr['exclude_serendip'])
         parser.add_argument("--wv_rms_thresh", type=float, default = None, help=blank_par.descr['wv_rms_thresh'])
         parser.add_argument("--refframe", type=str, default = None, choices = pypeitpar.WavelengthSolutionPar.valid_reference_frames(),
                             help=blank_par.descr['refframe'])
+        parser.add_argument('--chk_version', action = 'store_true', help=blank_pypar['rdx'].descr['chk_version'])
         parser.add_argument('-v', '--verbosity', type=int, default=1,
                             help='Verbosity level between 0 [none] and 2 [all]. Default: 1. '
                                  'Level 2 writes a log with filename collate_1d_YYYYMMDD-HHMM.log')
@@ -708,16 +773,17 @@ class Collate1D(scriptbase.ScriptBase):
         # Flux the spec1ds based on a archived sensfunc
         spec1d_failure_msgs = []
         copied_spec1d = False
-        if par['collate1d']['flux']:
+        if par['collate1d']['flux'] and not args.dry_run:
             if par['collate1d']['spec1d_outdir'] is not None:
                 # Fluxing modifies the spec1d files, copy them to a new output directory
                 # if requested
                 spec1d_files = copy_spec1d_to_outdir(spec1d_files, par['collate1d']['spec1d_outdir'])
                 copied_spec1d = True                
             spec1d_files = flux(par, spectrograph, spec1d_files, spec1d_failure_msgs)
+        
 
         # Perform reference frame correction
-        if par['collate1d']['refframe'] in  ['heliocentric', 'barycentric']:
+        if par['collate1d']['refframe'] in  ['heliocentric', 'barycentric'] and not args.dry_run:
             if not copied_spec1d and par['collate1d']['spec1d_outdir'] is not None:
                 # Refframe correction modifies the spec1d files, copy them to a new output directory
                 # if requested and fluxing hasn't already done so
@@ -725,12 +791,14 @@ class Collate1D(scriptbase.ScriptBase):
 
             refframe_correction(par, spectrograph, spec1d_files, spec1d_failure_msgs)
 
+        # Read in the spec1d files        
+        specobjs_to_coadd, spec1d_files = read_spec1d_files(par, spec1d_files, spec1d_failure_msgs)
+
         # Build source objects from spec1d file, this list is not collated 
-        source_objects = SourceObject.build_source_objects(spec1d_files,
+        source_objects = SourceObject.build_source_objects(specobjs_to_coadd, spec1d_files,
                                                            par['collate1d']['match_using'])
 
-        # Filter based on the coadding ex_value, and the exclude_serendip 
-        # boolean
+        # Filter out unwanted SpecObj objects based on parameters 
         (objects_to_coadd, excluded_obj_msgs) = exclude_source_objects(source_objects, exclude_map, par)
 
         # Collate the spectra
@@ -740,12 +808,18 @@ class Collate1D(scriptbase.ScriptBase):
         successful_source_list = []
         failed_source_msgs = []
         for source in source_list:
+
             coaddfile = os.path.join(par['collate1d']['outdir'], build_coadd_file_name(source))
             msgs.info(f'Creating {coaddfile} from the following sources:')
             for i in range(len(source.spec_obj_list)):
                 msgs.info(f'    {source.spec1d_file_list[i]}: {source.spec_obj_list[i].NAME} '
                           f'({source.spec_obj_list[i].MASKDEF_OBJNAME})')
 
+            # Exclude sources with a single object to coadd
+            if len(source.spec_obj_list) == 1:
+                excluded_obj_msgs.append(f"Excluding {source.spec_obj_list[0].NAME} in {source.spec1d_file_list[0]} because there's no other SpecObj to coadd with.")
+                continue
+
             if not args.dry_run:
                 try:
                     coadd(par, coaddfile, source)
