@@ -12,17 +12,15 @@ from IPython import embed
 import numpy as np
 
 from astropy.table import Table
-from astropy.coordinates import SkyCoord, Angle
+from astropy.coordinates import SkyCoord
 from astropy import units
 from astropy.stats import sigma_clipped_stats
-from scipy.interpolate import RegularGridInterpolator, interp1d
-try:
-    from skimage import transform as skimageTransform
-except ImportError:
-    skimageTransform = None
+from astropy.io import fits
 
+from pypeit.pypmsgs import PypeItBitMaskError
 from pypeit import msgs
 from pypeit import datamodel
+from pypeit import calibframe
 from pypeit import specobj
 from pypeit.bitmask import BitMask
 from pypeit.core import parse
@@ -32,9 +30,19 @@ class SlitTraceBitMask(BitMask):
     """
     Mask bits used during slit tracing.
     """
-    version = '1.0.0'
+    version = '1.0.1'
+
+    # TODO: Consider using a unique bit prefix for when the bits are written to
+    # the header, like so:
+    #
+    # prefix = 'SLITB'
+    #
+    # It's not necessary, though.
 
     def __init__(self):
+        # !!!!!!!!!!
+        # IMPORTANT: Only ever *append* new bits, and don't remove old ones
+        # !!!!!!!!!!
         mask = dict([
             ('SHORTSLIT', 'Slit formed by left and right edge is too short. Not ignored for flexure'),
             ('BOXSLIT', 'Slit formed by left and right edge is valid (large enough to be a valid '
@@ -42,11 +50,14 @@ class SlitTraceBitMask(BitMask):
             ('USERIGNORE', 'User has specified to ignore this slit. Not ignored for flexure.'),
             ('BADWVCALIB', 'Wavelength calibration failed for this slit'),
             ('BADTILTCALIB', 'Tilts analysis failed for this slit'),
+            ('BADALIGNCALIB', 'Alignment analysis failed for this slit'),
             ('SKIPFLATCALIB', 'Flat field generation failed for this slit. Skip flat fielding'),
             ('BADFLATCALIB', 'Flat field generation failed for this slit. Ignore it fully.'),
-            ('BADREDUCE', 'Skysub/extraction failed for this slit'),
+            ('BADREDUCE', 'Reduction failed for this slit'), # THIS IS DEPRECATED (we may remove in v1.13) BUT STAYS HERE TO ALLOW FOR BACKWARDS COMPATIBILITY
+            ('BADSKYSUB', 'Skysub failed for this slit'),
+            ('BADEXTRACT', 'Extraction failed for this slit'),
         ])
-        super(SlitTraceBitMask, self).__init__(list(mask.keys()), descr=list(mask.values()))
+        super().__init__(list(mask.keys()), descr=list(mask.values()))
 
     @property
     def exclude_for_reducing(self):
@@ -57,59 +68,44 @@ class SlitTraceBitMask(BitMask):
     def exclude_for_flexure(self):
         # Ignore these flags when performing a flexure calculation
         #  Currently they are *all* of the flags..
-        return ['SHORTSLIT', 'USERIGNORE', 'BADWVCALIB', 'BADTILTCALIB',
-                'SKIPFLATCALIB', 'BADFLATCALIB', 'BADREDUCE']
+        return ['SHORTSLIT', 'BOXSLIT', 'USERIGNORE', 'BADWVCALIB', 'BADTILTCALIB', 'BADALIGNCALIB',
+                'SKIPFLATCALIB', 'BADFLATCALIB', 'BADSKYSUB', 'BADEXTRACT']
 
 
 
-class SlitTraceSet(datamodel.DataContainer):
+class SlitTraceSet(calibframe.CalibFrame):
     """
     Defines a generic class for holding and manipulating image traces
     organized into left-right slit pairs.
 
-    Instantiation arguments map directly to the object
-    :attr:`datamodel`. The only additional argument is ``load``,
-    described below.
-
-    :class:`SlitTraceSet` objects can be instantiated with only the
-    master-frame arguments. If this is done, it's expected that
-    you'll attempt to load an existing master frame, either using
-    ``load=True`` on instantiation or by a call to :func:`load`.
-    Otherwise, all the elements of the data model will be empty.
-
     The datamodel attributes are:
 
     .. include:: ../include/class_datamodel_slittraceset.rst
 
-    Args:
-        load (:obj:`bool`, optional):
-            Attempt to load an existing master frame with the slit
-            trace data. WARNING: If ``load`` is True, all of the slit
-            information provided to the initialization is ignored!
-            Only the arguments relevant to the
-            :class:`pypeit.masterframe.MasterFrame` components of
-            :class:`SlitTraceSet` are used.
-
     Attributes:
-        left_flexure (`numpy.ndarray`_):  Convenient spot to hold flexure corrected left
-        right_flexure (`numpy.ndarray`_):  Convenient spot to hold flexure corrected right
-        master_key (:obj:`str`):
-        master_dir (:obj:`str`):
-
+        left_flexure (`numpy.ndarray`_):
+            Convenient spot to hold flexure corrected left
+        right_flexure (`numpy.ndarray`_):
+            Convenient spot to hold flexure corrected right
     """
-    frametype = 'slits'
-    master_type = 'Slits'
-    """Name for type of master frame."""
-    master_file_format = 'fits.gz'
-    """File format for the master frame file."""
-    minimum_version = '1.1.0'
-    version = '1.1.4'
+    calib_type = 'Slits'
+    """Name for type of calibration frame."""
+
+    calib_file_format = 'fits.gz'
+    """File format for the calibration frame file."""
+
+    version = '1.1.5'
     """SlitTraceSet data model version."""
 
-    hdu_prefix = None
-    output_to_disk = None
-
     bitmask = SlitTraceBitMask()
+    """
+    Bit interpreter for slit masks.
+    """
+
+    internals = calibframe.CalibFrame.internals + ['left_flexure', 'right_flexure']
+    """
+    Attributes kept separate from the datamodel.
+    """
 
     # Define the data model
     datamodel = {'PYP_SPEC': dict(otype=str, descr='PypeIt spectrograph name'),
@@ -129,13 +125,18 @@ class SlitTraceSet(datamodel.DataContainer):
                                  descr='Slit ID number from SPAT measured at half way point.'),
                  'maskdef_id': dict(otype=np.ndarray, atype=(int,np.integer),
                                     descr='Slit ID number slitmask'),
-                 'maskdef_designtab': dict(otype=Table, descr='Table with slitmask design and object info'),
+                 'maskdef_designtab': dict(otype=Table,
+                                           descr='Table with slitmask design and object info'),
                  'maskfile': dict(otype=str, descr='Data file that yielded the slitmask info'),
-                 'maskdef_posx_pa': dict(otype=float, descr='PA that aligns with spatial dimension of the detector'),
-                 'maskdef_offset': dict(otype=float, descr='Slitmask offset (pixels) from position expected '
-                                                           'by the slitmask design'),
+                 'maskdef_posx_pa': dict(otype=float,
+                                         descr='PA that aligns with spatial dimension of the '
+                                               'detector'),
+                 'maskdef_offset': dict(otype=float,
+                                        descr='Slitmask offset (pixels) from position expected '
+                                              'by the slitmask design'),
                  'maskdef_objpos': dict(otype=np.ndarray, atype=np.floating,
-                                         descr='Object positions expected by the slitmask design [relative pixels]'),
+                                        descr='Object positions expected by the slitmask design '
+                                              '[relative pixels]'),
                  'maskdef_slitcen': dict(otype=np.ndarray, atype=np.floating,
                                          descr='Slit centers expected by the slitmask design'),
                  'ech_order': dict(otype=np.ndarray, atype=(int,np.integer),
@@ -166,13 +167,12 @@ class SlitTraceSet(datamodel.DataContainer):
                  'mask': dict(otype=np.ndarray, atype=np.integer,
                               descr='Bit mask for slits (fully good slits have 0 value).  Shape '
                                     'is Nslits.'),
-                'slitbitm': dict(otype=str, descr='List of BITMASK keys from SlitTraceBitMask'),
                 'specmin': dict(otype=np.ndarray, atype=np.floating,
-                                descr='Minimum spectral position allowed for each slit/order.  '
-                                      'Shape is Nslits.'),
+                                descr='Minimum spectral position (pixel units) allowed for each '
+                                      'slit/order.  Shape is Nslits.'),
                 'specmax': dict(otype=np.ndarray, atype=np.floating,
-                                descr='Maximum spectral position allowed for each slit/order.  '
-                                      'Shape is Nslits.')}
+                                descr='Maximum spectral position (pixel units) allowed for each '
+                                      'slit/order.  Shape is Nslits.')}
     """Provides the class data model."""
 
     # TODO: Allow tweaked edges to be arguments?
@@ -183,15 +183,15 @@ class SlitTraceSet(datamodel.DataContainer):
                  pad=0, spat_id=None, maskdef_id=None, maskdef_designtab=None, maskfile=None,
                  maskdef_posx_pa=None, maskdef_offset=None, maskdef_objpos=None,
                  maskdef_slitcen=None, ech_order=None, nslits=None, left_tweak=None,
-                 right_tweak=None, center=None, mask=None, slitbitm=None):
+                 right_tweak=None, center=None, mask=None):
 
         # Instantiate the DataContainer
         args, _, _, values = inspect.getargvalues(inspect.currentframe())
         _d = dict([(k,values[k]) for k in args[1:]])
         # The dictionary passed to DataContainer.__init__ does not
-        # contain self or the MasterFrame arguments.
+        # contain self.
         # TODO: Does it matter if the calling function passes the
-        # keyword arguments in a different order?
+        # keyword arguments in a different order? No.
         datamodel.DataContainer.__init__(self, d=_d)
 
     def _validate(self):
@@ -255,22 +255,28 @@ class SlitTraceSet(datamodel.DataContainer):
         self.mask_init = np.atleast_1d(self.mask_init)
         self.specmin = np.atleast_1d(self.specmin)
         self.specmax = np.atleast_1d(self.specmax)
-        if self.slitbitm is None:
-            self.slitbitm = ','.join(list(self.bitmask.keys()))
-        else:
-            # Validate
-            if self.slitbitm != ','.join(list(self.bitmask.keys())):
-                msgs.error("Input BITMASK keys differ from current data model!")
         # Mask
         if self.mask is None:
             self.mask = self.mask_init.copy()
 
-    def _init_internals(self):
-        self.left_flexure = None
-        self.right_flexure = None
-        # Master stuff
-        self.master_key = None
-        self.master_dir = None
+    def _base_header(self, hdr=None):
+        """
+        Construct the baseline header for all HDU extensions.
+
+        This appends the :class:`SlitTraceBitMask` data to the supplied header.
+
+        Args:
+            hdr (`astropy.io.fits.Header`_, optional):
+                Baseline header for additional data. If None, set by
+                :func:`pypeit.io.initialize_header()`.
+
+        Returns:
+            `astropy.io.fits.Header`_: Header object to include in
+            all HDU extensions.
+        """
+        _hdr = super()._base_header(hdr=hdr)
+        self.bitmask.to_header(_hdr)
+        return _hdr
 
     def _bundle(self):
         """
@@ -279,7 +285,7 @@ class SlitTraceSet(datamodel.DataContainer):
         See :func:`pypeit.datamodel.DataContainer._bundle`. Data is
         always written to a 'SLITS' extension.
         """
-        bndl = super(SlitTraceSet, self)._bundle(ext='SLITS', transpose_arrays=True)
+        bndl = super()._bundle(ext='SLITS', transpose_arrays=True)
         if self.maskdef_designtab is not None:
             # save the table
             tab_detached = bndl[0]['SLITS']['maskdef_designtab']
@@ -288,8 +294,7 @@ class SlitTraceSet(datamodel.DataContainer):
             # create a dict for the `tab_detached`
             tab_dict = {'maskdef_designtab': tab_detached}
             return [bndl[0], tab_dict]
-        else:
-            return bndl
+        return bndl
 
     # TODO: Although I don't like doing it, kwargs is here to catch the
     # extraneous keywords that can be passed to _parse from the base class but
@@ -303,7 +308,7 @@ class SlitTraceSet(datamodel.DataContainer):
         always read from the 'SLITS' extension.
         """
         if not hasattr(hdu, '__len__'):
-            return super(SlitTraceSet, cls)._parse(hdu, transpose_table_arrays=True)
+            return super()._parse(hdu, transpose_table_arrays=True)
 
         # TODO: My edit to the code causes the code below to fault in some cases
         # because of a consistency limitation that I put on the values that ext
@@ -311,11 +316,52 @@ class SlitTraceSet(datamodel.DataContainer):
         # code in the except block will always fault now, and I don't remember
         # why we needed this try/except block in the first place.
         try:
-            return super(SlitTraceSet, cls)._parse(hdu, ext=['SLITS', 'MASKDEF_DESIGNTAB'],
-                                                   transpose_table_arrays=True)
+            return super()._parse(hdu, ext=['SLITS', 'MASKDEF_DESIGNTAB'],
+                                  transpose_table_arrays=True)
         except KeyError:
-            return super(SlitTraceSet, cls)._parse(hdu, ext='SLITS',
-                                                   transpose_table_arrays=True)
+            return super()._parse(hdu, ext='SLITS', transpose_table_arrays=True)
+
+    @classmethod
+    def from_hdu(cls, hdu, chk_version=True, **kwargs):
+        """
+        Instantiate the object from an HDU extension.
+
+        This overrides the base-class method, only to add checks (or not) for
+        the bitmask.
+
+        Args:
+            hdu (`astropy.io.fits.HDUList`_, `astropy.io.fits.ImageHDU`_, `astropy.io.fits.BinTableHDU`_):
+                The HDU(s) with the data to use for instantiation.
+            chk_version (:obj:`bool`, optional):
+                If True, raise an error if the datamodel version or
+                type check failed. If False, throw a warning only.
+            **kwargs:
+                Passed directly to :func:`_parse`.
+        """
+        # Run the default parser
+        d, version_passed, type_passed, parsed_hdus = cls._parse(hdu, **kwargs)
+        # Check
+        cls._check_parsed(version_passed, type_passed, chk_version=chk_version)
+
+        # Instantiate
+        self = super().from_dict(d=d)
+
+        # Calibration frame attributes
+        # NOTE: If multiple HDUs are parsed, this assumes that the information
+        # necessary to set all the calib internals is always in *every* header.
+        # BEWARE!
+        self.calib_keys_from_header(hdu[parsed_hdus[0]].header)
+
+        # Check the bitmasks. Bits should have been written to *any* header
+        # associated with the object
+        hdr = hdu[parsed_hdus[0]].header if isinstance(hdu, fits.HDUList) else hdu.header
+        hdr_bitmask = BitMask.from_header(hdr)
+        if chk_version and hdr_bitmask.bits != self.bitmask.bits:
+            msgs.error('The bitmask in this fits file appear to be out of date!  Recreate this '
+                       'file by re-running the relevant script or set chk_version=False.',
+                       cls='PypeItBitMaskError')
+
+        return self
 
     def init_tweaked(self):
         """
@@ -349,18 +395,33 @@ class SlitTraceSet(datamodel.DataContainer):
     @property
     def slitord_id(self):
         """
-        Return array of slit_spatId (MultiSlit, IFU) or ech_order (Echelle) values
+        Return array of slit_spatId (MultiSlit, SlicerIFU) or ech_order (Echelle) values
 
         Returns:
             `numpy.ndarray`_:
 
         """
-        if self.pypeline in ['MultiSlit', 'IFU']:
+        if self.pypeline in ['MultiSlit', 'SlicerIFU']:
             return self.spat_id
-        elif self.pypeline == 'Echelle':
+        if self.pypeline == 'Echelle':
             return self.ech_order
-        else:
-            msgs.error('Unrecognized Pypeline {:}'.format(self.pypeline))
+        msgs.error(f'Unrecognized Pypeline {self.pypeline}')
+
+    @property
+    def slitord_txt(self):
+        """
+        Return string indicating if the logs/QA should use "slit" (MultiSlit,
+        SlicerIFU) or "order" (Echelle).
+
+        Returns:
+            str: Either 'slit' or 'order'
+
+        """
+        if self.pypeline in ['MultiSlit', 'SlicerIFU']:
+            return 'slit'
+        if self.pypeline == 'Echelle':
+            return 'order'
+        msgs.error(f'Unrecognized Pypeline {self.pypeline}')
 
     def spatid_to_zero(self, spat_id):
         """
@@ -386,12 +447,11 @@ class SlitTraceSet(datamodel.DataContainer):
             int: zero-based index of the input spat_id
 
         """
-        if self.pypeline in ['MultiSlit', 'IFU']:
+        if self.pypeline in ['MultiSlit', 'SlicerIFU']:
             return np.where(self.spat_id == slitord)[0][0]
-        elif self.pypeline in ['Echelle']:
+        if self.pypeline == 'Echelle':
             return np.where(self.ech_order == slitord)[0][0]
-        else:
-            msgs.error('Unrecognized Pypeline {:}'.format(self.pypeline))
+        msgs.error('Unrecognized Pypeline {:}'.format(self.pypeline))
 
     def get_slitlengths(self, initial=False, median=False):
         """
@@ -417,33 +477,31 @@ class SlitTraceSet(datamodel.DataContainer):
         """
         left, right, _ = self.select_edges(initial=initial)
         slitlen = right - left
-        if median is True:
-            slitlen = np.median(slitlen, axis=1)
-        return slitlen
+        return np.median(slitlen, axis=1) if median else slitlen
 
-    def get_radec_image(self, wcs, alignments, tilts, locations,
-                        astrometric=True, initial=True, flexure=None):
+    def get_radec_image(self, wcs, alignSplines, tilts, slit_compute=None, slice_offset=None, initial=False, flexure=None):
         """Generate an RA and DEC image for every pixel in the frame
-        NOTE: This function is currently only used for IFU reductions.
+        NOTE: This function is currently only used for SlicerIFU reductions.
 
         Parameters
         ----------
-        wcs : astropy.wcs
+        wcs : `astropy.wcs.WCS`_
             The World Coordinate system of a science frame
-        alignments : :class:`pypeit.alignframe.Alignments`
-            The alignments (traces) of the slits. This allows
-            different slits to be aligned correctly.
-        tilts : `numpy.ndarray`
+        alignSplines : :class:`pypeit.alignframe.AlignmentSplines`
+            An instance of the AlignmentSplines class that allows one to build and
+            transform between detector pixel coordinates and WCS pixel coordinates.
+        tilts : `numpy.ndarray`_
             Spectral tilts.
-        locations : `numpy.ndarray`_, list
-            locations along the slit of the alignment traces. Must
-            be a 1D array of the same length as alignments.traces.shape[1]
-        maxslitlen : int
-            This is the slit length in pixels, and it should be the same
-            value that was passed to get_wcs() to generate the WCS that
-            is passed into this function as an argument.
-        astrometric : bool
-            Perform astrometric correction using alignment frame?
+        slit_compute : int, list, `numpy.ndarray`_, optional
+            The slit indices to compute the RA and DEC image for. If None, all slits
+            are computed. If an integer, the RA and DEC image for the slit with this
+            index is computed. If a list or `numpy.ndarray`_, the RA and DEC image
+            for the slits with these indices are computed.
+        slice_offset : float, optional
+            Offset to apply to the slice positions. A value of 0.0 means that the
+            slice positions are the centre of the slits. A value of +/-0.5 means that
+            the slice positions are at the edges of the slits. If None, the slice_offset
+            is set to 0.0.
         initial : bool
             Select the initial slit edges?
         flexure : float, optional
@@ -451,35 +509,34 @@ class SlitTraceSet(datamodel.DataContainer):
 
         Returns
         -------
-        tuple : There are three elements in the tuple. The first two are 2D numpy arrays
-                of shape (nspec, nspat), where the first ndarray is the RA image, and the
-                second ndarray is the DEC image. RA and DEC are in units degrees. The third
-                element of the tuple stores the minimum and maximum difference (in pixels)
-                between the WCS reference (usually the centre of the slit) and the edge of
-                the slits. The third array has a shape of (nslits, 2).
+        raimg : `numpy.ndarray`_
+            Image with the RA coordinates of each pixel in degrees.  Shape is
+            (nspec, nspat).
+        decimg : `numpy.ndarray`_
+            Image with the DEC coordinates of each pixel in degrees.  Shape is
+            (nspec, nspat).
+        minmax : `numpy.ndarray`_
+            The minimum and maximum difference (in pixels) between the WCS
+            reference (usually the centre of the slit) and the edges of the
+            slits. Shape is (nslits, 2).
         """
-        msgs.work("Spatial flexure is not currently implemented for the astrometric alignment")
-        # Check if the user has skimage installed
-        if skimageTransform is None or alignments is None:
-            if skimageTransform is None:
-                    msgs.warn("scikit-image is not installed - astrometric correction not implemented")
-            else:
-                msgs.warn("Alignments were not provided - astrometric correction not implemented")
-            astrometric = False
-        # Prepare the parameters
-        if not astrometric:
-            left, right, _ = self.select_edges(initial=initial, flexure=flexure)
-            trace_cen = 0.5 * (left + right)
+        # Check the input
+        if slit_compute is None:
+            # Compute all of the slits
+            slit_compute = np.arange(self.nslits)
+        elif isinstance(slit_compute, (int, list)):
+            slit_compute = np.atleast_1d(slit_compute)
         else:
-            if type(locations) is list:
-                locations = np.array(locations)
-            elif type(locations) is not np.ndarray:
-                msgs.error("locations must be a 1D list or 1D numpy array")
-            nspec, nloc, nslit = alignments.traces.shape
-
-            # Generate a spline of the waveimg for interpolation
-            tilt_spl = RegularGridInterpolator((np.arange(tilts.shape[0]), np.arange(tilts.shape[1])), tilts*(nspec-1), method='linear')
+            msgs.error('Unrecognized type for slit_compute')
 
+        # Prepare the print out
+        substring = '' if slice_offset is None else f' with slice_offset={slice_offset:.3f}'
+        msgs.info("Generating an RA/DEC image"+substring)
+        # Check the input
+        if slice_offset is None:
+            slice_offset = 0.0
+        if slice_offset < -0.5 or slice_offset > 0.5:
+            msgs.error(f"Slice offset must be between -0.5 and 0.5. slice_offset={slice_offset}")
         # Initialise the output
         raimg = np.zeros((self.nspec, self.nspat))
         decimg = np.zeros((self.nspec, self.nspat))
@@ -487,49 +544,20 @@ class SlitTraceSet(datamodel.DataContainer):
         # Get the slit information
         slitid_img_init = self.slit_img(pad=0, initial=initial, flexure=flexure)
         for slit_idx, spatid in enumerate(self.spat_id):
+            if slit_idx not in slit_compute:
+                continue
             onslit = (slitid_img_init == spatid)
             onslit_init = np.where(onslit)
             if self.mask[slit_idx] != 0:
-                msgs.error("Slit {0:d} ({1:d}/{2:d}) is masked. Cannot generate RA/DEC image.".format(spatid,
-                                                                                                      slit_idx+1,
-                                                                                                      self.spat_id.size))
-            if astrometric:
-                # Calculate the typical pixel difference in the spatial direction
-                medpixdiff = np.median(np.diff(alignments.traces[:, :, slit_idx], axis=1))
-                nspecpix = int(np.ceil(nspec / medpixdiff))
-                specpix = np.round(np.linspace(0.0, nspec-1, nspecpix)).astype(int)
-                # Calculate the source locations (pixel space)
-                xsrc = alignments.traces[specpix, :, slit_idx].flatten()
-                ysrc = specpix.repeat(nloc).flatten()
-                src = np.column_stack((xsrc, ysrc))
-                # Calculate the destinations (slit space)
-                xdst = locations[np.newaxis, :].repeat(nspecpix, axis=0).flatten()
-                ydst = tilt_spl((ysrc, xsrc))
-                dst = np.column_stack((xdst, ydst))
-                msgs.info("Calculating astrometric transform of slit {0:d}/{1:d}".format(slit_idx+1, nslit))
-                tform = skimageTransform.estimate_transform("polynomial", src, dst, order=1)
-                # msgs.info("Calculating inverse transform of slit {0:d}/{1:d}".format(slit_idx+1, nslit))
-                tfinv = skimageTransform.estimate_transform("polynomial", dst, src, order=1)
-                # Calculate the slitlength at a given tilt value
-                xyll = tfinv(np.column_stack((np.zeros(nspec), np.linspace(0.0, 1.0, nspec))))
-                xyrr = tfinv(np.column_stack((np.ones(nspec), np.linspace(0.0, 1.0, nspec))))
-                slitlen = np.sqrt((xyll[:, 0]-xyrr[:, 0])**2 + (xyll[:, 1]-xyrr[:, 1])**2)
-                slen_spl = interp1d(np.linspace(0.0, 1.0, nspec), slitlen, kind='linear',
-                                    bounds_error=False, fill_value="extrapolate")
-                slitlength = slen_spl(tilts[onslit_init])
-                # Now perform the transform
-                pixsrc = np.column_stack((onslit_init[1], onslit_init[0]))
-                pixdst = tform(pixsrc)
-                evalpos = (pixdst[:, 0] - 0.5) * slitlength
-            else:
-                evalpos = onslit_init[1] - trace_cen[onslit_init[0], slit_idx]
-            minmax[:, 0] = np.min(evalpos)
-            minmax[:, 1] = np.max(evalpos)
-            slitID = np.ones(evalpos.size) * slit_idx - wcs.wcs.crpix[0]
-            if astrometric:
-                world_ra, world_dec, _ = wcs.wcs_pix2world(slitID, evalpos, tilts[onslit_init]*(nspec-1), 0)
-            else:
-                world_ra, world_dec, _ = wcs.wcs_pix2world(slitID, evalpos, onslit_init[0], 0)
+                msgs.error(f'Slit {spatid} ({slit_idx+1}/{self.spat_id.size}) is masked. Cannot '
+                           'generate RA/DEC image.')
+            # Retrieve the pixel offset from the central trace
+            evalpos = alignSplines.transform(slit_idx, onslit_init[1], onslit_init[0])
+            minmax[slit_idx, 0] = np.min(evalpos)
+            minmax[slit_idx, 1] = np.max(evalpos)
+            # Calculate the WCS from the pixel positions
+            slitID = np.ones(evalpos.size) * slit_idx + slice_offset
+            world_ra, world_dec, _ = wcs.wcs_pix2world(slitID, evalpos, tilts[onslit_init]*(self.nspec-1), 0)
             # Set the RA first and DEC next
             raimg[onslit] = world_ra.copy()
             decimg[onslit] = world_dec.copy()
@@ -573,9 +601,8 @@ class SlitTraceSet(datamodel.DataContainer):
         # Return
         return left.copy(), right.copy(), self.mask.copy()
 
-    def slit_img(self, pad=None, slitidx=None, initial=False, 
-                 flexure=None,
-                 exclude_flag=None, use_spatial=True):
+    def slit_img(self, pad=None, slitidx=None, initial=False, flexure=None, exclude_flag=None,
+                 use_spatial=True):
         r"""
         Construct an image identifying each pixel with its associated
         slit.
@@ -619,8 +646,8 @@ class SlitTraceSet(datamodel.DataContainer):
                 :attr:`right_init`) are used. To use the nominal edges
                 regardless of the presence of the tweaked edges, set
                 this to True. See :func:`select_edges`.
-            exclude_flag (:obj:`str`, optional):
-                Bitmask flag to ignore when masking
+            exclude_flag (:obj:`str`, :obj:`list`, optional):
+                One or more bitmask flag names to ignore when masking
                 Warning -- This could conflict with input slitids, i.e. avoid using both
             use_spatial (bool, optional):
                 If True, use self.spat_id value instead of 0-based indices
@@ -652,10 +679,11 @@ class SlitTraceSet(datamodel.DataContainer):
         if slitidx is not None:
             slitidx = np.atleast_1d(slitidx).ravel()
         else:
-            bpm = self.mask.astype(bool)
-            if exclude_flag:
-                bpm &= np.invert(self.bitmask.flagged(self.mask, flag=exclude_flag))
-            slitidx = np.where(np.invert(bpm))[0]
+            bpm = self.bitmask.flagged(self.mask, and_not=exclude_flag)
+#            bpm = self.mask.astype(bool)
+#            if exclude_flag:
+#                bpm &= np.invert(self.bitmask.flagged(self.mask, flag=exclude_flag))
+            slitidx = np.where(np.logical_not(bpm))[0]
 
         # TODO: When specific slits are chosen, need to check that the
         # padding doesn't lead to slit overlap.
@@ -813,10 +841,11 @@ class SlitTraceSet(datamodel.DataContainer):
 
     def mask_add_missing_obj(self, sobjs, spat_flexure, fwhm, boxcar_rad):
         """
-        Generate new SpecObj and add them into the SpecObjs object for any slits missing the targeted source.
+        Generate new SpecObj and add them into the SpecObjs object for any slits
+        missing the targeted source.
 
         Args:
-            sobjs (:class:`pypeit.specobjs.SpecObjs`):
+            sobjs (:class:`~pypeit.specobjs.SpecObjs`):
                 List of SpecObj that have been found and traced
             spat_flexure (:obj:`float`):
                 Shifts, in spatial pixels, between this image and SlitTrace
@@ -826,16 +855,19 @@ class SlitTraceSet(datamodel.DataContainer):
                 BOX_RADIUS in pixels to be used in the boxcar extraction
 
         Returns:
-            :class:`pypeit.specobjs.SpecObjs`: Updated list of SpecObj that have been found and traced
+            :class:`~pypeit.specobjs.SpecObjs`: Updated list of SpecObj that have
+            been found and traced
 
         """
         msgs.info('Add undetected objects at the expected location from slitmask design.')
 
         if fwhm is None:
-            msgs.error('A FWHM for the optimal extraction must be provided. See `find_fwhm` in `FindObjPar`.')
+            msgs.error('A FWHM for the optimal extraction must be provided. See `find_fwhm` in '
+                       '`FindObjPar`.')
 
         if self.maskdef_objpos is None:
-            msgs.error('An array with the object positions expected from slitmask design is missing.')
+            msgs.error('An array with the object positions expected from slitmask design is '
+                       'missing.')
 
         if self.maskdef_offset is None:
             msgs.error('A value for the slitmask offset must be provided.')
@@ -847,10 +879,10 @@ class SlitTraceSet(datamodel.DataContainer):
         else:
             cut_sobjs = sobjs
 
-        # get slits edges init
-        left_init, _, _ = self.select_edges(initial=True, flexure=spat_flexure)  # includes flexure
-        # get slits edges tweaked
-        left_tweak, right_tweak, _ = self.select_edges(initial=False, flexure=spat_flexure)  # includes flexure
+        # get slits edges init; includes flexure
+        left_init, _, _ = self.select_edges(initial=True, flexure=spat_flexure)
+        # get slits edges tweaked; includes flexure
+        left_tweak, right_tweak, _ = self.select_edges(initial=False, flexure=spat_flexure)
 
         # midpoint in the spectral direction
         specmid = left_init[:,0].size//2
@@ -891,6 +923,7 @@ class SlitTraceSet(datamodel.DataContainer):
                 xoff = SPAT_PIXPOS - self.center[specmid, islit]
                 thisobj.TRACE_SPAT = self.center[:, islit] + xoff
                 thisobj.SPAT_PIXPOS = SPAT_PIXPOS
+                thisobj.SPAT_PIXPOS_ID = int(np.rint(thisobj.SPAT_PIXPOS))
                 thisobj.SPAT_FRACPOS = (SPAT_PIXPOS - left_tweak[specmid, islit]) / \
                                        (right_tweak[specmid, islit]-left_tweak[specmid, islit])
                 thisobj.trace_spec = np.arange(left_tweak.shape[0])
@@ -903,6 +936,7 @@ class SlitTraceSet(datamodel.DataContainer):
                 xoff = SPAT_PIXPOS - cut_sobjs[idx_nearest].TRACE_SPAT[specmid]
                 thisobj.TRACE_SPAT = cut_sobjs[idx_nearest].TRACE_SPAT + xoff
                 thisobj.SPAT_PIXPOS = SPAT_PIXPOS
+                thisobj.SPAT_PIXPOS_ID = int(np.rint(thisobj.SPAT_PIXPOS))
                 thisobj.trace_spec = cut_sobjs[idx_nearest].trace_spec
                 thisobj.SPAT_FRACPOS = (SPAT_PIXPOS - left_tweak[specmid, islit]) / \
                                        (right_tweak[specmid, islit]-left_tweak[specmid, islit])
@@ -915,7 +949,7 @@ class SlitTraceSet(datamodel.DataContainer):
 
             # FWHM
             thisobj.FWHM = fwhm  # pixels
-            thisobj.BOX_RADIUS = boxcar_rad  # pixels
+            thisobj.BOX_R_PIX = boxcar_rad  # pixels
             thisobj.maskwidth = 4. * fwhm  # matches objfind() in extract.py
             thisobj.smash_snr = 0.
             thisobj.smash_peakflux = 0.
@@ -926,6 +960,8 @@ class SlitTraceSet(datamodel.DataContainer):
             thisobj.RA = self.maskdef_designtab['OBJRA'][oidx]
             thisobj.DEC = self.maskdef_designtab['OBJDEC'][oidx]
             thisobj.MASKDEF_OBJNAME = self.maskdef_designtab['OBJNAME'][oidx]
+            thisobj.MASKDEF_OBJMAG = self.maskdef_designtab['OBJMAG'][oidx]
+            thisobj.MASKDEF_OBJMAG_BAND = self.maskdef_designtab['OBJMAG_BAND'][oidx]
             thisobj.MASKDEF_ID = self.maskdef_designtab['MASKDEF_ID'][oidx]
             thisobj.MASKDEF_EXTRACT = True
             thisobj.hand_extract_flag = False
@@ -1066,6 +1102,8 @@ class SlitTraceSet(datamodel.DataContainer):
                 sobj.RA = self.maskdef_designtab['OBJRA'][oidx]
                 sobj.DEC = self.maskdef_designtab['OBJDEC'][oidx]
                 sobj.MASKDEF_OBJNAME = self.maskdef_designtab['OBJNAME'][oidx]
+                sobj.MASKDEF_OBJMAG = self.maskdef_designtab['OBJMAG'][oidx]
+                sobj.MASKDEF_OBJMAG_BAND = self.maskdef_designtab['OBJMAG_BAND'][oidx]
                 sobj.MASKDEF_EXTRACT = False
                 # Remove that idx value
                 idx = idx.tolist()
@@ -1096,6 +1134,35 @@ class SlitTraceSet(datamodel.DataContainer):
         # Return
         return sobjs
 
+    def det_of_slit(self, spat_id:int, det_img:np.ndarray,
+                    slit_img:np.ndarray=None):
+        """ Identify the 'best' detector for this slit/order
+        The metric is the detector where the slit appears
+        the most frequently.
+
+        Only sensibly used for mosaic images
+
+        Args:
+            spat_id (`int`): 
+                spat_id value for the slit of interest
+            det_img (`numpy.ndarray`_): 
+                :obj:`int` image specifying the detector number
+                (1-based) for each pixel in the mosaic
+            slit_img (`numpy.ndarray`_, optional): 
+                image identifying each pixel with its associated slit.
+
+        Returns:
+            :obj:`int`: Detector number for the slit (1-based)
+        """
+        # Grab slit image?
+        if slit_img is None:
+            slit_img = self.slit_img()
+        # Find the most common detector value
+        det, cnt = np.unique(
+            det_img[(slit_img == spat_id) & (det_img > 0)], 
+            return_counts=True)
+        return det[np.argmax(cnt)]
+
     def get_maskdef_objpos(self, plate_scale, det_buffer):
         """
         Determine the object positions expected by the slitmask design
@@ -1338,16 +1405,18 @@ class SlitTraceSet(datamodel.DataContainer):
         will be computed using the average fwhm of the detected objects.
 
         Args:
-            sobjs (:class:`pypeit.specobjs.SpecObjs`):
+            sobjs (:class:`~pypeit.specobjs.SpecObjs`):
                 List of SpecObj that have been found and traced.
             platescale (:obj:`float`):
                 Platescale.
             fwhm_parset (:obj:`float`, optional):
-                Parset that guides the determination of the fwhm of the maskdef_extract objects.
-                If None (default) the fwhm are computed as the averaged from the detected objects,
+                :class:`~pypeit.par.parset.Parset` that guides the determination
+                of the fwhm of the maskdef_extract objects.  If None (default)
+                the fwhm are computed as the averaged from the detected objects,
                 if it is a number it will be adopted as the fwhm.
             find_fwhm (:obj:`float`):
-            Initial guess of the objects fwhm in pixels (used in object finding)
+                Initial guess of the objects fwhm in pixels (used in object
+                finding)
 
         Returns:
             :obj:`float`: FWHM in pixels to be used in the optimal extraction
@@ -1405,13 +1474,13 @@ class SlitTraceSet(datamodel.DataContainer):
                 #TODO -- Consider putting in a tolerance which if not met causes a crash
                 idx = np.argmin(np.abs(self.spat_id - slit_spat))
                 msk[idx] = False
-            self.mask[msk] = self.bitmask.turn_on(self.mask[msk], 
+            self.mask[msk] = self.bitmask.turn_on(self.mask[msk],
                                                   'USERIGNORE')
         elif user_slits['method'] == 'maskIDs':
             # Mask only the good one
             msk = np.logical_not(np.isin(self.maskdef_id, user_slits['slit_info']))
             # Set
-            self.mask[msk] = self.bitmask.turn_on(self.mask[msk], 
+            self.mask[msk] = self.bitmask.turn_on(self.mask[msk],
                                                   'USERIGNORE')
         else:
             msgs.error('Not ready for this method: {:s}'.format(
@@ -1419,15 +1488,15 @@ class SlitTraceSet(datamodel.DataContainer):
 
     def mask_flats(self, flatImages):
         """
-        Mask from a :class:`pypeit.flatfield.Flats` object
+        Mask based on a :class:`~pypeit.flatfield.FlatImages` object.
 
         Args:
-            flatImages (:class:`pypeit.flatfield.FlatImages`):
+            flatImages (:class:`~pypeit.flatfield.FlatImages`):
 
         """
         # Loop on all the FLATFIELD BPM keys
         for flag in ['SKIPFLATCALIB', 'BADFLATCALIB']:
-            bad_flats = self.bitmask.flagged(flatImages.get_bpmflats(), flag)
+            bad_flats = self.bitmask.flagged(flatImages.get_bpmflats(), flag=flag)
             if np.any(bad_flats):
                 self.mask[bad_flats] = self.bitmask.turn_on(self.mask[bad_flats], flag)
 
