@@ -18,10 +18,12 @@ from linetools.spectra import xspectrum1d
 from pypeit import msgs
 from pypeit.core import flexure
 from pypeit.core import flux_calib
+from pypeit.core import parse
 from pypeit import utils
 from pypeit import datamodel
 from pypeit.images.detector_container import DetectorContainer
 from pypeit.images.mosaic import Mosaic
+from pypeit.spectrographs.util import load_spectrograph
 
 
 class SpecObj(datamodel.DataContainer):
@@ -39,14 +41,14 @@ class SpecObj(datamodel.DataContainer):
     Args:
         PYPELINE (:obj:`str`):
             Name of the ``PypeIt`` pipeline method.  Allowed options are
-            MultiSlit, Echelle, or IFU.
+            MultiSlit, Echelle, or SlicerIFU.
         DET (:obj:`str`):
             The name of the detector or mosaic from which the spectrum was
             extracted.  For example, DET01.
         OBJTYPE (:obj:`str`, optional):
             Object type.  For example: 'unknown', 'standard', 'science'.
         SLITID (:obj:`int`, optional):
-            For multislit and IFU reductions, this is an identifier for the slit
+            For multislit and SlicerIFU reductions, this is an identifier for the slit
             (max=9999).
         ECH_ORDER (:obj:`int`, optional):
             Physical order number.
@@ -54,7 +56,7 @@ class SpecObj(datamodel.DataContainer):
             Running index for the order.
     """
 
-    version = '1.1.6'
+    version = '1.1.13'
     """
     Current datamodel version number.
     """
@@ -62,8 +64,9 @@ class SpecObj(datamodel.DataContainer):
     datamodel = {'TRACE_SPAT': dict(otype=np.ndarray, atype=float,
                                     descr='Object trace along the spec (spatial pixel)'),
                  'FWHM': dict(otype=float, descr='Spatial FWHM of the object (pixels)'),
-                 'FWHMFIT': dict(otype=np.ndarray,
+                 'FWHMFIT': dict(otype=np.ndarray, atype=float,
                                  descr='Spatial FWHM across the detector (pixels)'),
+                 'SPAT_FWHM': dict(otype=float, descr='Spatial FWHM of the object (arcsec)'),
                  'smash_peakflux': dict(otype=float,
                                         descr='Peak value of the spectral direction collapsed spatial profile'),
                  'smash_snr': dict(otype=float,
@@ -85,8 +88,12 @@ class SpecObj(datamodel.DataContainer):
                  'OPT_COUNTS_NIVAR': dict(otype=np.ndarray, atype=float,
                                           descr='Optimally extracted noise variance, sky+read '
                                                 'noise only (counts^2)'),
+                 'OPT_FLAT': dict(otype=np.ndarray, atype=float,
+                                  descr='Optimally extracted flatfield spectrum, normalised to the peak value.'),
                  'OPT_MASK': dict(otype=np.ndarray, atype=np.bool_,
                                   descr='Mask for optimally extracted flux. True=good'),
+                 'OPT_FWHM': dict(otype=np.ndarray, atype=float,
+                                  descr='Spectral FWHM (in Angstroms) at every pixel of the optimally extracted flux.'),
                  'OPT_COUNTS_SKY': dict(otype=np.ndarray, atype=float,
                                         descr='Optimally extracted sky (counts)'),
                  'OPT_COUNTS_SIG_DET': dict(otype=np.ndarray, atype=float,
@@ -116,8 +123,12 @@ class SpecObj(datamodel.DataContainer):
                  'BOX_COUNTS_NIVAR': dict(otype=np.ndarray, atype=float,
                                           descr='Boxcar extracted noise variance, sky+read noise '
                                                 'only (counts^2)'),
+                 'BOX_FLAT': dict(otype=np.ndarray, atype=float,
+                                   descr='Boxcar extracted flatfield spectrum, normalized to the peak value.'),
                  'BOX_MASK': dict(otype=np.ndarray, atype=np.bool_,
                                   descr='Mask for boxcar extracted flux. True=good'),
+                 'BOX_FWHM': dict(otype=np.ndarray, atype=float,
+                                  descr='Spectral FWHM (in Angstroms) at every pixel of the boxcar extracted flux.'),
                  'BOX_COUNTS_SKY': dict(otype=np.ndarray, atype=float,
                                         descr='Boxcar extracted sky (counts)'),
                  'BOX_COUNTS_SIG_DET': dict(otype=np.ndarray, atype=float,
@@ -127,7 +138,10 @@ class SpecObj(datamodel.DataContainer):
                                             'used for this extraction'),
                  'BOX_CHI2': dict(otype=np.ndarray, atype=float,
                                   descr='Reduced chi2 of the model fit for this spectral pixel'),
-                 'BOX_RADIUS': dict(otype=float, descr='Size of boxcar radius (pixels)'),
+                 'BOX_R_PIX': dict(otype=float, descr='Size of boxcar radius (pixels)'),
+                 'BOX_R_ASEC': dict(otype=float, descr='Size of boxcar radius (arcsec)'),
+                 'S2N': dict(otype=float, descr='Median signal to noise ratio of the extracted spectrum'
+                                                '(OPT if available, otherwise BOX)'),
                  #
                  'FLEX_SHIFT_GLOBAL': dict(otype=float, descr='Global shift of the spectrum to correct for spectral'
                                                               'flexure (pixels). This is based on the sky spectrum at'
@@ -150,17 +164,21 @@ class SpecObj(datamodel.DataContainer):
                  'DETECTOR': dict(otype=(DetectorContainer, Mosaic),
                                   descr='Object with the detector or mosaic metadata'),
                  'PYPELINE': dict(otype=str, descr='Name of the PypeIt pipeline mode'),
+                 'PYP_SPEC': dict(otype=str, descr='PypeIt spectrograph name'),
                  # TODO: It's unclear if OBJTYPE has to be one among a set of
                  # specific keywords.
                  'OBJTYPE': dict(otype=str, descr='Object type (e.g., standard, science)'),
                  'SPAT_PIXPOS': dict(otype=(float, np.floating),
                                      descr='Spatial location of the trace on detector (pixel) at half-way'),
+                 'SPAT_PIXPOS_ID': dict(otype=(int, np.integer),
+                                     descr='Nearest integer spatial location of the trace on detector (pixel) at half-way '
+                                           'used as a unique identifier for the naming model'),                 
                  'SPAT_FRACPOS': dict(otype=(float, np.floating),
                                       descr='Fractional location of the object on the slit'),
                  'trace_spec': dict(otype=np.ndarray, atype=(int,np.integer),
                                       descr='Array of pixels along the spectral direction'),
                  'maskwidth': dict(otype=(float, np.floating),
-                                      descr='Size (in units of fwhm) of the region used for local sky subtraction'),
+                                      descr='Size (in units of spatial fwhm) of the region used for local sky subtraction'),
                  # Slit and Object
                  'WAVE_RMS': dict(otype=(float, np.floating),
                                      descr='RMS (pix) for the wavelength solution for this slit.'),
@@ -175,6 +193,9 @@ class SpecObj(datamodel.DataContainer):
                  'DEC': dict(otype=float, descr='Declination (J2000) decimal degree'),
                  'MASKDEF_ID': dict(otype=(int, np.integer), descr='Slitmask definition ID'),
                  'MASKDEF_OBJNAME': dict(otype=str, descr='Name of the object from the slitmask definition'),
+                 'MASKDEF_OBJMAG': dict(otype=float, descr='Magnitude of the object from the slitmask definition'),
+                 'MASKDEF_OBJMAG_BAND': dict(otype=str, descr='Magnitude band of the object from the slitmask '
+                                                              'definition'),
                  'MASKDEF_EXTRACT': dict(otype=bool, descr='Boolean indicating if this is a forced extraction '
                                                            'at the expected location from slitmask design. '),
                  'hand_extract_flag': dict(otype=bool, descr='Boolean indicating if this is a forced extraction '
@@ -184,12 +205,17 @@ class SpecObj(datamodel.DataContainer):
                                    descr='Object ID for echelle data. Each object is given an '
                                          'index in the order it appears increasing from from left '
                                          'to right. These are one based.'),
+                 # TODO ECH_ORDERINDX should be purged. It is not reliable for anything given masking. Instead
+                 # one needs to use SLITID or ECH_ORDER
                  'ECH_ORDERINDX': dict(otype=(int, np.integer),
                                        descr='Order indx, analogous to SLITID for echelle. '
                                              'Zero based.'),
                  'ECH_FRACPOS': dict(otype=(float, np.floating),
                                      descr='Synced echelle fractional location of the object on '
                                            'the slit'),
+                 'ECH_FRACPOS_ID': dict(otype=(int, np.integer),
+                                     descr='Echelle fractional location of the object on the slit multiplied '
+                                     'by 1000 used as a unique identifier for the naming model'), 
                  'ECH_ORDER': dict(otype=(int, np.integer), descr='Physical echelle order'),
                  'ECH_NAME': dict(otype=str,
                                   descr='Name of the object for echelle data. Same as NAME above '
@@ -199,13 +225,38 @@ class SpecObj(datamodel.DataContainer):
     Defines the current datmodel.
     """
 
+    internals = [# Object finding
+                 'smash_peakflux',
+                 'smash_snr',
+                 # Hand
+                 'hand_extract_flag',
+                 'hand_extract_spec',
+                 'hand_extract_spat',
+                 'hand_extract_det',
+                 'hand_extract_fwhm',
+                 # Object profile
+                 'prof_nsigma',
+                 'sign',
+                 'min_spat',
+                 'max_spat',
+                 # Echelle
+                 'ech_frac_was_fit',
+                 'ech_snr',
+                 # spectrograph
+                'spectrograph',
+                ]
+
     def __init__(self, PYPELINE, DET, OBJTYPE='unknown',
-                 SLITID=None, ECH_ORDER=None, ECH_ORDERINDX=None):
+                 SLITID=None, ECH_ORDER=None, ECH_ORDERINDX=None, PYP_SPEC=None):
 
         args, _, _, values = inspect.getargvalues(inspect.currentframe())
         _d = dict([(k,values[k]) for k in args[1:]])
         super().__init__(d=_d)
 
+        # Initialize internal values that are not None
+        self.hand_extract_flag = False
+        self.sign = 1.
+
         self.FLEX_SHIFT_GLOBAL = 0.
         self.FLEX_SHIFT_LOCAL = 0.
         self.FLEX_SHIFT_TOTAL = 0.
@@ -215,43 +266,30 @@ class SpecObj(datamodel.DataContainer):
 
     @classmethod
     def from_arrays(cls, PYPELINE:str, wave:np.ndarray, counts:np.ndarray, ivar:np.ndarray,
-                    mode='OPT', DET='DET01', SLITID=0, **kwargs):
+                    flat=None, mode='OPT', DET='DET01', SLITID=0, **kwargs):
         # Instantiate
         slf = cls(PYPELINE, DET, SLITID=SLITID)
+        # Check the type of the flat field if it's not None
+        if flat is not None:
+            if not isinstance(flat, np.ndarray):
+                msgs.error('Flat must be a numpy array')
+            if flat.shape != counts.shape:
+                msgs.error('Flat and counts must have the same shape')
         # Add in arrays
-        for item, attr in zip([wave, counts, ivar], ['_WAVE', '_COUNTS', '_COUNTS_IVAR']):
+        for item, attr in zip([wave, counts, ivar, flat], ['_WAVE', '_COUNTS', '_COUNTS_IVAR', '_FLAT']):
+            # Check if any of the arrays are None. If so, skip
+            if item is None:
+                continue
             setattr(slf, mode+attr, item.astype(float))
         # Mask. Watch out for places where ivar is infinite due to a divide by 0
         slf[mode+'_MASK'] = (slf[mode+'_COUNTS_IVAR'] > 0.) & np.isfinite(slf[mode+'_COUNTS_IVAR'])
         return slf
 
-    def _init_internals(self):
-        # Object finding
-        self.smash_peakflux = None
-        self.smash_snr = None
-
-        # Hand
-        self.hand_extract_flag = False
-        self.hand_extract_spec = None
-        self.hand_extract_spat = None
-        self.hand_extract_det = None
-        self.hand_extract_fwhm = None
-
-        # Object profile
-        self.prof_nsigma = None
-        self.sign = 1.0
-        self.min_spat = None
-        self.max_spat = None
-
-        # Echelle
-        self.ech_frac_was_fit = None #
-        self.ech_snr = None #
-
     def _validate(self):
         """
         Validate the object.
         """
-        pypelines = ['MultiSlit', 'IFU', 'Echelle']
+        pypelines = ['MultiSlit', 'SlicerIFU', 'Echelle']
         if self.PYPELINE not in pypelines:
             msgs.error(f'{self.PYPELINE} is not a known pipeline procedure.  Options are: '
                        f"{', '.join(pypelines)}")
@@ -296,7 +334,7 @@ class SpecObj(datamodel.DataContainer):
             return self.ECH_ORDER
         elif self.PYPELINE == 'MultiSlit':
             return self.SLITID
-        elif self.PYPELINE == 'IFU':
+        elif self.PYPELINE == 'SlicerIFU':
             return self.SLITID
         else:
             msgs.error("Bad PYPELINE")
@@ -308,7 +346,7 @@ class SpecObj(datamodel.DataContainer):
             return self.ECH_ORDERINDX
         elif self.PYPELINE == 'MultiSlit':
             return self.SLITID
-        elif self.PYPELINE == 'IFU':
+        elif self.PYPELINE == 'SlicerIFU':
             return self.SLITID
         else:
             msgs.error("Bad PYPELINE")
@@ -329,7 +367,6 @@ class SpecObj(datamodel.DataContainer):
                 break
         return mnx
 
-    @property
     def med_s2n(self):
         """Return median S/N of the spectrum
         Uses OPT_COUNTS if present and then BOX_COUNTS
@@ -339,19 +376,72 @@ class SpecObj(datamodel.DataContainer):
         """
         SN = 0.
         for pref in ['OPT', 'BOX']:
-            if self[pref+'_COUNTS'] is not None:
-                SN = np.median(self[pref+'_COUNTS'] * np.sqrt(self[pref+'_COUNTS_IVAR']))
+            if self[pref+'_COUNTS'] is not None and np.any(self[pref+'_MASK']):
+                SN = np.median(self[pref+'_COUNTS'][self[pref+'_MASK']] *
+                               np.sqrt(self[pref+'_COUNTS_IVAR'][self[pref+'_MASK']]))
             if SN != 0.:
                 break
         return SN
 
+    def med_fwhm(self):
+        """Return median spatial FWHM of the spectrum
+
+        Returns:
+            float: Median spatial FWHM in arcsec
+        """
+        FWHM = 0.
+        if self['FWHMFIT'] is not None and self['OPT_COUNTS'] is not None:
+            FWHM = np.median(self['FWHMFIT']) * self.platescale
+        return FWHM
+
+    def boxcar_arcsec(self):
+        """Return the boxcar radius in arcsec
+
+        Returns:
+            float: Boxcar radius in arcsec
+        """
+
+        boxcar_arcsec = 0.
+        if self['BOX_R_PIX'] is not None and self['BOX_COUNTS'] is not None:
+            boxcar_arcsec = self['BOX_R_PIX'] * self.platescale
+        return boxcar_arcsec
+
+    @property
+    def platescale(self):
+        """Return the platescale in arcsec/pixel and includes the binning factor
+
+        Returns:
+            float: Platescale in arcsec/pixel
+        """
+        # make sure we have the spectrograph
+        if self.spectrograph is None:
+            self.get_spectrograph()
+
+        if self.PYPELINE == 'Echelle' and self.spectrograph.orders is not None:
+            idx = np.where(self.spectrograph.orders==self['ECH_ORDER'])[0][0]
+            return self.spectrograph.order_platescale(self.spectrograph.orders, self['DETECTOR']['binning'])[idx]
+
+        _, binspatial = parse.parse_binning(self['DETECTOR']['binning'])
+        return self['DETECTOR']['platescale'] * binspatial
+
+    def get_spectrograph(self):
+        """Set the spectrograph attribute from the PYP_SPEC attribute.
+
+        """
+        # some checks first
+        if self.spectrograph is None and self.PYP_SPEC is None:
+            msgs.error("PYP_SPEC must be set to access the spectrograph")
+        # get it
+        if self.spectrograph is None:
+            self.spectrograph = load_spectrograph(self.PYP_SPEC)
+
     def set_name(self):
         """
         Construct the ``PypeIt`` name for this object.
 
         The ``PypeIt`` name depends on the type of data being processed:
 
-            - For multislit and IFU data, the name is
+            - For multislit and SlicerIFU data, the name is
               ``SPATnnnn-SLITmmmm-{DET}``, where ``nnnn`` is the nearest integer
               pixel in the spatial direction (at the spectral midpoint) where
               the object was extracted, ``mmmm`` is the slit identification
@@ -375,12 +465,12 @@ class SpecObj(datamodel.DataContainer):
             # ObjID
             name = naming_model['obj']
             ech_name = naming_model['obj']
-            if self['ECH_FRACPOS'] is None:
+            if self['ECH_FRACPOS_ID'] is None:
                 name += '----'
             else:
                 # JFH TODO Why not just write it out with the decimal place. That is clearer than this??
-                name += '{:04d}'.format(int(np.rint(1000*self.ECH_FRACPOS)))
-                ech_name += '{:04d}'.format(int(np.rint(1000*self.ECH_FRACPOS)))
+                name += '{:04d}'.format(self.ECH_FRACPOS_ID)
+                ech_name += '{:04d}'.format(self.ECH_FRACPOS_ID)
             name += f'-{self.DET}'
             ech_name += f'-{self.DET}'
             # Order number
@@ -388,13 +478,15 @@ class SpecObj(datamodel.DataContainer):
             name += '{:04d}'.format(self.ECH_ORDER)
             self.ECH_NAME = ech_name
             self.NAME = name
-        elif self.PYPELINE in ['MultiSlit', 'IFU']:
+        elif self.PYPELINE in ['MultiSlit', 'SlicerIFU']:
             # Spat
             name = naming_model['spat']
-            if self['SPAT_PIXPOS'] is None:
+            if self['SPAT_PIXPOS_ID'] is None:
                 name += '----'
             else:
-                name += '{:04d}'.format(int(np.rint(self.SPAT_PIXPOS)))
+                name += '{:04d}'.format(self.SPAT_PIXPOS_ID)
+                #name += '{:04d}'.format(int(np.rint(self.SPAT_PIXPOS)))
+
             # Slit
             name += '-'+naming_model['slit']
             name += '{:04d}'.format(self.SLITID)
@@ -459,38 +551,39 @@ class SpecObj(datamodel.DataContainer):
         # Now update the total flexure
         self.FLEX_SHIFT_TOTAL += shift
 
-    # TODO This should be a wrapper calling a core algorithm.
     def apply_flux_calib(self, wave_zp, zeropoint, exptime, tellmodel=None, extinct_correct=False,
-                         airmass=None, longitude=None, latitude=None, extinctfilepar=None, extrap_sens=False):
+                         airmass=None, longitude=None, latitude=None, extinctfilepar=None,
+                         extrap_sens=False):
         """
         Apply a sensitivity function to our spectrum
 
         FLAM, FLAM_SIG, and FLAM_IVAR are generated
 
         Args:
-            wave_zp (float array)
+            wave_zp (`numpy.ndarray`_):
                 Zeropoint wavelength array
-            zeropoint (float array):
+            zeropoint (`numpy.ndarray`_):
                 zeropoint array
             exptime (float):
                 Exposure time
-            tellmodel:
-                Telluric correction
-            extinct_correct:
+            tellmodel (`numpy.ndarray`_, optional):
+                Telluric model. To be applied to the sensitivity function. Only used to
+                generate the std fluxed QA plot. It should be None otherwise. To telluric
+                correct the data, use the telluric correct method.
+            extinct_correct (bool, optional):
                 If True, extinction correct
             airmass (float, optional):
                 Airmass
             longitude (float, optional):
                 longitude in degree for observatory
-            latitude:
+            latitude (float, optional):
                 latitude in degree for observatory
                 Used for extinction correction
-            extinctfilepar (str):
+            extinctfilepar (str, optional):
                 [sensfunc][UVIS][extinct_file] parameter
                 Used for extinction correction
             extrap_sens (bool, optional):
                 Extrapolate the sensitivity function (instead of crashing out)
-
         """
         # Loop on extraction modes
         for attr in ['BOX', 'OPT']:
@@ -551,8 +644,10 @@ class SpecObj(datamodel.DataContainer):
         Convert spectrum into np.ndarray arrays
 
         Args:
-            extraction (str): Extraction method to convert
+            extraction (str):
+               Extraction method to convert
             fluxed:
+               Use the fluxed tags
 
         Returns:
             tuple: wave, flux, ivar, mask arrays
@@ -572,17 +667,30 @@ class SpecObj(datamodel.DataContainer):
         # Return
         return self[swave], self[sflux], self[sivar], self[smask]
 
-    def to_xspec1d(self, **kwargs):
+    def to_xspec1d(self, masked=True, extraction='OPT', fluxed=True):
         """
-        Push the data in :class:`SpecObj` into an XSpectrum1D object
+        Create an `XSpectrum1D <linetools.spectra.xspectrum1d.XSpectrum1D>`_
+        using this spectrum.
 
+        Args:
+            masked (:obj:`bool`, optional):
+                If True, only unmasked data are included.
+            extraction (str):
+                Extraction method to convert
+            fluxed:
+                Use the fluxed tags
 
         Returns:
-            linetools.spectra.xspectrum1d.XSpectrum1D:  Spectrum object
-
+            `linetools.spectra.xspectrum1d.XSpectrum1D`_: Spectrum object
         """
-        wave, flux, ivar, _ = self.to_arrays(**kwargs)
+        wave, flux, ivar, gpm = self.to_arrays(extraction=extraction, fluxed=fluxed)
         sig = np.sqrt(utils.inverse(ivar))
+        wave_gpm = wave > 1.0
+        wave, flux, sig, gpm = wave[wave_gpm], flux[wave_gpm], sig[wave_gpm], gpm[wave_gpm]
+        if masked:
+            flux = flux*gpm
+            sig = sig*gpm
+
         # Create
         return xspectrum1d.XSpectrum1D.from_tuple((wave, flux, sig))
 
@@ -593,7 +701,7 @@ class SpecObj(datamodel.DataContainer):
         Returns:
             bool: True if all checks have passed
         """
-        required = ['TRACE_SPAT', 'SPAT_PIXPOS', 'SPAT_FRACPOS',
+        required = ['TRACE_SPAT', 'SPAT_PIXPOS', 'SPAT_PIXPOS_ID', 'SPAT_FRACPOS',
             'trace_spec', 'OBJID', 'FWHM', 'maskwidth', 'NAME',
             'smash_peakflux', 'smash_snr',
             'SLITID', 'DET', 'PYPELINE', 'OBJTYPE']
@@ -621,8 +729,9 @@ class SpecObj(datamodel.DataContainer):
         for attr in self.datamodel.keys():
             if hasattr(self, attr) and getattr(self, attr) is not None:
                 # Special ones
-                if attr in ['DET', 'SLITID', 'SPAT_PIXPOS', 'NAME', 'RA', 
-                            'DEC', 'MASKDEF_ID', 'MASKDEF_OBJNAME', 'MASKDEF_EXTRACT']:
+                if attr in ['DET', 'SLITID', 'SPAT_PIXPOS', 'SPAT_PIXPOS_ID', 'NAME', 'RA', 
+                            'DEC', 'MASKDEF_ID', 'MASKDEF_OBJNAME', 'MASKDEF_EXTRACT',
+                            'MASKDEF_OBJMAG', 'MASKDEF_OBJMAG_BAND']:
                     rdict[attr] = getattr(self,attr)
                 else:
                     rdict[attr] = True
@@ -635,48 +744,112 @@ class SpecObj(datamodel.DataContainer):
                 repr += '{}: {}\n'.format(key, rdict[key])
         return repr + '>'
 
-    def has_opt_ext(self):
+    def has_opt_ext(self, fluxed=False):
         """
-        Cehck that all the values of the optimal extraction exist
+        Check that all the values of the optimal extraction exist
+
+        Args:
+            fluxed (:obj:`bool`, optional):
+                Check that the flux-calibrated data exist.
 
         Returns:
-            :obj:bool: True if all OPT values are available
+            :obj:`bool`: True if all OPT values are available
         """
-        keys_to_check = ['OPT_WAVE', 'OPT_COUNTS', 'OPT_COUNTS_IVAR', 'OPT_MASK']
+        flx = 'FLAM' if fluxed else 'COUNTS'
+        return self.check_populated(['OPT_WAVE', f'OPT_{flx}', f'OPT_{flx}_IVAR', 'OPT_MASK'])
 
-        return np.all([self[key] is not None for key in keys_to_check])
-
-    def get_opt_ext(self):
+    def get_opt_ext(self, fluxed=False):
         """
         Return the optimal extraction values
 
-        Returns:
-            :obj:tuple: OPT_WAVE, OPT_COUNTS, OPT_COUNTS_IVAR, OPT_MASK attributes of SpecObj
+        Args:
+            fluxed (:obj:`bool`, optional):
+                Return the flux-calibrated data.
 
+        Returns:
+            :obj:`tuple`: OPT_WAVE, OPT_COUNTS, OPT_COUNTS_IVAR, OPT_MASK
+            attributes of SpecObj
         """
+        if fluxed:
+            # TODO: This does not check if the fluxed data exists!
+            return self.OPT_WAVE, self.OPT_FLAM, self.OPT_FLAM_IVAR, self.OPT_MASK
         return self.OPT_WAVE, self.OPT_COUNTS, self.OPT_COUNTS_IVAR, self.OPT_MASK
 
-    def has_box_ext(self):
+    def has_box_ext(self, fluxed=False):
         """
-        Cehck that all the values of the boxcar extraction exist
+        Check that all the values of the boxcar extraction exist
+
+        Args:
+            fluxed (:obj:`bool`, optional):
+                Check that the flux-calibrated data exist.
 
         Returns:
-            :obj:bool: True if all BOX values are available
+            :obj:`bool`: True if all BOX values are available
         """
-        keys_to_check = ['BOX_WAVE', 'BOX_COUNTS', 'BOX_COUNTS_IVAR', 'BOX_MASK']
+        flx = 'FLAM' if fluxed else 'COUNTS'
+        return self.check_populated(['BOX_WAVE', f'BOX_{flx}', f'BOX_{flx}_IVAR', 'BOX_MASK'])
 
-        return np.all([self[key] is not None for key in keys_to_check])
-
-    def get_box_ext(self):
+    def get_box_ext(self, fluxed=False):
         """
         Return the boxcar extraction values
 
-        Returns:
-            :obj:tuple: BOX_WAVE, BOX_COUNTS, BOX_COUNTS_IVAR, BOX_MASK attributes of SpecObj
+        Args:
+            fluxed (:obj:`bool`, optional):
+                Return the flux-calibrated data.
 
+        Returns:
+            :obj:`tuple`: BOX_WAVE, BOX_COUNTS, BOX_COUNTS_IVAR, BOX_MASK
+            attributes of SpecObj
         """
+        if fluxed:
+            # TODO: This does not check if the fluxed data exists!
+            return self.BOX_WAVE, self.BOX_FLAM, self.BOX_FLAM_IVAR, self.BOX_MASK
         return self.BOX_WAVE, self.BOX_COUNTS, self.BOX_COUNTS_IVAR, self.BOX_MASK
 
+    def best_ext_match(self, extract=None, fluxed=True):
+        """
+        Determine the extraction and calibration type that best matches a user
+        request.
 
+        Precedence is given to the requested extraction and calibration types.
+        Beyond that, optimal extraction takes precedence over boxcar extraction,
+        and flux-calibrated data take precedence over uncalibrated counts.
 
+        Args:
+            extract (:obj:`str`, optional):
+                The extraction used to produce the spectrum.  Must be either
+                None, ``'BOX'`` (for a boxcar extraction), or ``'OPT'`` for
+                optimal extraction.  If None, the optimal extraction will be
+                returned, if it exists, otherwise the boxcar extraction will be
+                returned.
+            fluxed (:obj:`bool`, optional):
+                If True, return the flux calibrated spectrum, if it exists.  If
+                the flux calibration hasn't been performed or ``fluxed=False``,
+                the spectrum is returned in counts.
+
+        Returns:
+            :obj:`tuple`: The adjusted extraction type (``'BOX'`` or ``'OPT'``)
+            and the adjusted calibration type (True for flux-calibrated, False
+            for uncalibrated counts).
+        """
+        # If not set, prefer the optimal extraction over the boxcar one.
+        _extract = 'OPT' if extract is None else extract
+        if _extract not in ['OPT', 'BOX']:
+            msgs.error(f'Extraction type ({_extract}) not understood; must be OPT or BOX.')
+        if _extract == 'OPT':
+            if self.has_opt_ext(fluxed=fluxed):
+                return 'OPT', fluxed
+            # If we make it here, expect that fluxed was True.  Try flipping it.
+            if self.has_opt_ext(fluxed=False):
+                return 'OPT', False
+
+        # If we make it here, either extract was BOX from the start, or none of
+        # the optimal extraction options were available
+        if self.has_box_ext(fluxed=fluxed):
+            return 'BOX', fluxed
+        # If we make it here, assume fluxed was True.  Try flipping it.
+        if self.has_box_ext(fluxed=False):
+            return 'BOX', False
+        # If we make it here, we've got a problem!
+        msgs.error('Unable to find a relevant set of data!')
 
