@@ -5,6 +5,7 @@ Module for the SpecObjs and SpecObj classes
 .. include:: ../include/links.rst
 """
 import os
+from pathlib import Path
 import re
 from typing import List
 
@@ -23,10 +24,13 @@ from pypeit import io
 from pypeit.spectrographs.util import load_spectrograph
 from pypeit.core import parse
 from pypeit.images.detector_container import DetectorContainer
+# NOTE: Mosaic cannot be found in this module explicitly, but it is used in
+# statements like: dmodcls = eval(hdu.header['DMODCLS'])
 from pypeit.images.mosaic import Mosaic
-from pypeit import slittrace
 from pypeit import utils
 
+
+# TODO: Make this a DataContainer
 class SpecObjs:
     """
     Object to hold a set of :class:`~pypeit.specobj.SpecObj` objects
@@ -42,9 +46,12 @@ class SpecObjs:
     Args:
         specobjs (`numpy.ndarray`_, list, optional):
             One or more :class:`~pypeit.specobj.SpecObj`  objects
+        header (`astropy.io.fits.Header`_, optional):
+            Baseline header to use
 
     Attributes:
-        summary (astropy.table.Table):
+        summary (`astropy.table.Table`_):
+            Summary table (?)
     """
     version = '1.0.0'
 
@@ -72,45 +79,57 @@ class SpecObjs:
             provided fits file.
         """
         # HDUList
-        hdul = io.fits_open(fits_file)
-        # Init
-        slf = cls()
-        # Add on the header
-        slf.header = hdul[0].header
-        # Keep track of HDUList for closing later
+        with io.fits_open(fits_file) as hdul:
+            # Init
+            slf = cls()
+            # Add on the header
+            slf.header = hdul[0].header
 
-        detector_hdus = {}
-        # Loop for Detectors first as we need to add these to the objects
-        for hdu in hdul[1:]:
-            if 'DETECTOR' not in hdu.name:
-                continue
-            if 'DMODCLS' not in hdu.header:
-                msgs.error('HDUs with DETECTOR in the name must have DMODCLS in their header.')
-            try:
-                dmodcls = eval(hdu.header['DMODCLS'])
-            except:
-                msgs.error(f"Unknown detector type datamodel class: {hdu.header['DMODCLS']}")
-            # NOTE: This requires that any "detector" datamodel class has a
-            # from_hdu method, and the name of the HDU must have a known format
-            # (e.g., 'DET01-DETECTOR').
-            _det = hdu.name.split('-')[0]
-            detector_hdus[_det] = dmodcls.from_hdu(hdu)
+            # Catch common error of trying to read a OneSpec file
+            if 'DMODCLS' in hdul[1].header and hdul[1].header['DMODCLS'] == 'OneSpec':
+                msgs.error('This is a OneSpec file.  You are treating it like a SpecObjs file.')
+
+            # Load the calibration association into the instance attribute `calibs`
+            if 'CLBS_DIR' in slf.header:
+                slf.calibs = {}
+                slf.calibs['DIR'] = slf.header['CLBS_DIR']
+                for key in slf.header.keys():
+                    if key.startswith('CLBS_') \
+                            and (Path(slf.calibs['DIR']).absolute() / slf.header[key]).exists():
+                        slf.calibs['_'.join(key.split('_')[1:])] = slf.header[key]
+
+            detector_hdus = {}
+            # Loop for Detectors first as we need to add these to the objects
+            for hdu in hdul[1:]:
+                if 'DETECTOR' not in hdu.name:
+                    continue
+                if 'DMODCLS' not in hdu.header:
+                    msgs.error('HDUs with DETECTOR in the name must have DMODCLS in their header.')
+                try:
+                    dmodcls = eval(hdu.header['DMODCLS'])
+                except:
+                    msgs.error(f"Unknown detector type datamodel class: {hdu.header['DMODCLS']}")
+                # NOTE: This requires that any "detector" datamodel class has a
+                # from_hdu method, and the name of the HDU must have a known format
+                # (e.g., 'DET01-DETECTOR').
+                _det = hdu.name.split('-')[0]
+                detector_hdus[_det] = dmodcls.from_hdu(hdu, chk_version=chk_version)
+
+            # Now the objects
+            for hdu in hdul[1:]:
+                if 'DETECTOR' in hdu.name:
+                    continue
+                sobj = specobj.SpecObj.from_hdu(hdu, chk_version=chk_version)
+                # Restrict on det?
+                if det is not None and sobj.DET != det:
+                    continue
+                # Check for detector
+                if sobj.DET in detector_hdus.keys():
+                    sobj.DETECTOR = detector_hdus[sobj.DET]
+                # Append
+                slf.add_sobj(sobj)
 
-        # Now the objects
-        for hdu in hdul[1:]:
-            if 'DETECTOR' in hdu.name:
-                continue
-            sobj = specobj.SpecObj.from_hdu(hdu, chk_version=chk_version)
-            # Restrict on det?
-            if det is not None and sobj.DET != det:
-                continue
-            # Check for detector
-            if sobj.DET in detector_hdus.keys():
-                sobj.DETECTOR = detector_hdus[sobj.DET]
-            # Append
-            slf.add_sobj(sobj)
         # Return
-        hdul.close()
         return slf
 
 
@@ -126,6 +145,7 @@ class SpecObjs:
 
         self.header = header
         self.hdul = None
+        self.calibs = None
 
         # Turn off attributes from here
         #   Anything else set will be on the individual specobj objects in the specobjs array
@@ -168,16 +188,28 @@ class SpecObjs:
         """
         return len(self.specobjs)
 
-    def unpack_object(self, ret_flam=False, extract_type='OPT'):
+    def unpack_object(self, ret_flam=False, log10blaze=False, min_blaze_value=1e-3, extract_type='OPT',
+                      extract_blaze=False, remove_missing=False):
         """
         Utility function to unpack the sobjs for one object and
         return various numpy arrays describing the spectrum and meta
-        data. The user needs to already have trimmed the Specobjs to
+        data. The user needs to already have trimmed the :class:`SpecObjs` to
         the relevant indices for the object.
 
         Args:
-           ret_flam (:obj:`bool`, optional):
-              If True return the FLAM, otherwise return COUNTS.
+            ret_flam (:obj:`bool`, optional):
+               If True return the FLAM, otherwise return COUNTS.
+            log10blaze (:obj:`bool`, optional):
+                If True return the log10 of the blaze function.
+            min_blaze_value (:obj:`float`, optional):
+                Minimum value of the blaze function to consider as good.
+            extract_type (:obj:`str`, optional):
+                Extraction type to use.  Default is 'OPT'.
+            extract_blaze (:obj:`bool`, optional):
+                If True, extract the blaze function.  Default is False.
+            remove_missing (:obj:`bool`, optional):
+                If True, remove any missing data (i.e. where the flux is None).
+                Default is False.
 
         Returns:
             tuple: Returns the following where all numpy arrays
@@ -190,31 +222,58 @@ class SpecObjs:
                   Flambda or counts)
                 - flux_gpm (`numpy.ndarray`_): Good pixel mask.
                   True=Good
+                - blaze (`numpy.ndarray`_, None): Blaze function
                 - meta_spec (dict:) Dictionary containing meta data.
                   The keys are defined by
-                  spectrograph.header_cards_from_spec()
+                  spectrograph.parse_spec_header()
                 - header (astropy.io.header object): header from
                   spec1d file
         """
         # Prep
-        norddet = self.nobj
         flux_attr = 'FLAM' if ret_flam else 'COUNTS'
         flux_key = '{}_{}'.format(extract_type, flux_attr)
         wave_key = '{}_WAVE'.format(extract_type)
-        # Test
-        if getattr(self, flux_key)[0] is None:
-            msgs.error("Flux not available for {}.  Try the other ".format(flux_key))
+        blaze_key = '{}_FLAT'.format(extract_type)
+
+        # Check for missing data
+        none_flux = [f is None for f in getattr(self, flux_key)]
+        other = 'OPT' if extract_type == 'BOX' else 'BOX'
+        if np.any(none_flux):
+            if flux_attr == 'FLAM':
+                msg = f"{flux_key} is not available for all slits/orders." \
+                      "Your data may not have been fluxed, check your input files, or use unfluxed data. "
+            else:
+                msg = f"{extract_type} extracted flux is not available for all slits/orders. " \
+                      f"Consider trying the {other} extraction."
+            if not remove_missing:
+                msgs.error(msg)
+            else:
+                msg += f"{msgs.newline()}-- The missing data will be removed --"
+                msgs.warn(msg)
+                # Remove missing data
+                r_indx = np.where(none_flux)[0]
+                self.remove_sobj(r_indx)
+        # check for missing blaze
+        if extract_blaze:
+            none_blaze = [f is None for f in getattr(self, blaze_key)]
+            if np.any(none_blaze):
+                msgs.error(f"{extract_type} extracted blaze is not available for all slits/orders. "
+                           f"Consider trying the {other} extraction, or NOT using the flat.")
+
         #
+        norddet = self.nobj
         nspec = getattr(self, flux_key)[0].size
         # Allocate arrays and unpack spectrum
         wave = np.zeros((nspec, norddet))
         flux = np.zeros((nspec, norddet))
         flux_ivar = np.zeros((nspec, norddet))
         flux_gpm = np.zeros((nspec, norddet), dtype=bool)
+        if extract_blaze:
+            blaze = np.zeros((nspec, norddet), dtype=float)
+
         detector = [None]*norddet
         ech_orders = np.zeros(norddet, dtype=int)
 
-        # TODO make the extraction that is desired OPT vs BOX an optional input variable.
         for iorddet in range(norddet):
             wave[:, iorddet] = getattr(self, wave_key)[iorddet]
             flux_gpm[:, iorddet] = getattr(self, '{}_MASK'.format(extract_type))[iorddet]
@@ -222,7 +281,20 @@ class SpecObjs:
             if self[0].PYPELINE == 'Echelle':
                 ech_orders[iorddet] = self[iorddet].ECH_ORDER
             flux[:, iorddet] = getattr(self, flux_key)[iorddet]
-            flux_ivar[:, iorddet] = getattr(self, flux_key+'_IVAR')[iorddet] #OPT_FLAM_IVAR
+            flux_ivar[:, iorddet] = getattr(self, flux_key+'_IVAR')[iorddet]
+            if extract_blaze:
+                blaze[:, iorddet] = getattr(self, blaze_key)[iorddet]
+
+        # Log10 blaze
+        if extract_blaze:
+            blaze_function = np.copy(blaze)
+            if log10blaze:
+                for iorddet in range(norddet):
+                    blaze_function_smooth = utils.fast_running_median(blaze[:, iorddet], 5)
+                    blaze_function_norm = blaze_function_smooth / blaze_function_smooth.max()
+                    blaze_function[:, iorddet] = np.log10(np.clip(blaze_function_norm, min_blaze_value, None))
+        else:
+            blaze_function = None
 
         # Populate meta data
         spectrograph = load_spectrograph(self.header['PYP_SPEC'])
@@ -235,17 +307,18 @@ class SpecObjs:
         meta_spec['DET'] = np.array(detector)
         meta_spec['DISPNAME'] = self.header['DISPNAME']
         # Return
-        if self[0].PYPELINE in ['MultiSlit', 'IFU'] and self.nobj == 1:
+        if self[0].PYPELINE in ['MultiSlit', 'SlicerIFU'] and self.nobj == 1:
             meta_spec['ECH_ORDERS'] = None
+            blaze_ret = blaze_function.reshape(nspec) if blaze_function is not None else None
             return wave.reshape(nspec), flux.reshape(nspec), flux_ivar.reshape(nspec), \
-                   flux_gpm.reshape(nspec), meta_spec, self.header
+                   flux_gpm.reshape(nspec), blaze_ret, meta_spec, self.header
         else:
             meta_spec['ECH_ORDERS'] = ech_orders
-            return wave, flux, flux_ivar, flux_gpm, meta_spec, self.header
+            return wave, flux, flux_ivar, flux_gpm, blaze_function, meta_spec, self.header
 
     def get_std(self, multi_spec_det=None):
         """
-        Return the standard star from this Specobjs. For MultiSlit this
+        Return the standard star from this :class:`SpecObjs`. For MultiSlit this
         will be a single specobj in SpecObjs container, for Echelle it
         will be the standard for all the orders.
 
@@ -260,7 +333,7 @@ class SpecObjs:
         """
         # Is this MultiSlit or Echelle
         pypeline = (self.PYPELINE)[0]
-        if 'MultiSlit' in pypeline or 'IFU' in pypeline:
+        if 'MultiSlit' in pypeline or 'SlicerIFU' in pypeline:
             # Have to do a loop to extract the counts for all objects
             if self.OPT_COUNTS[0] is not None:
                 SNR = np.median(self.OPT_COUNTS * np.sqrt(self.OPT_COUNTS_IVAR), axis=1)
@@ -342,7 +415,7 @@ class SpecObjs:
             sobjs_neg.OBJID = -sobjs_neg.OBJID
         elif sobjs_neg[0].PYPELINE == 'MultiSlit':
             sobjs_neg.OBJID = -sobjs_neg.OBJID
-        elif sobjs_neg[0].PYPELINE == 'IFU':
+        elif sobjs_neg[0].PYPELINE == 'SlicerIFU':
             sobjs_neg.OBJID = -sobjs_neg.OBJID
         else:
             msgs.error("The '{0:s}' PYPELINE is not defined".format(self[0].PYPELINE))
@@ -363,7 +436,7 @@ class SpecObjs:
                 index = self.ECH_OBJID < 0
             elif self[0].PYPELINE == 'MultiSlit':
                 index = self.OBJID < 0
-            elif self[0].PYPELINE == 'IFU':
+            elif self[0].PYPELINE == 'SlicerIFU':
                 index = self.OBJID < 0
             else:
                 msgs.error("The '{0:s}' PYPELINE is not defined".format(self[0].PYPELINE))
@@ -381,7 +454,7 @@ class SpecObjs:
                 index = self.ECH_OBJID < 0
             elif self[0].PYPELINE == 'MultiSlit':
                 index = self.OBJID < 0
-            elif self[0].PYPELINE == 'IFU':
+            elif self[0].PYPELINE == 'SlicerIFU':
                 index = self.OBJID < 0
             else:
                 msgs.error("Should not get here")
@@ -407,13 +480,15 @@ class SpecObjs:
             indx = self.ECH_ORDER == slitorder
         elif self[0].PYPELINE == 'MultiSlit':
             indx = self.SLITID == slitorder
-        elif self[0].PYPELINE == 'IFU':
+        elif self[0].PYPELINE == 'SlicerIFU':
             indx = self.SLITID == slitorder
         else:
             msgs.error("The '{0:s}' PYPELINE is not defined".format(self[0].PYPELINE))
-        #
+
         return indx
 
+        
+
     def name_indices(self, name):
         """
         Return the set of indices matching the input slit/order
@@ -429,41 +504,41 @@ class SpecObjs:
             indx = self.ECH_NAME == name
         elif self[0].PYPELINE == 'MultiSlit':
             indx = self.NAME == name
-        elif self[0].PYPELINE == 'IFU':
+        elif self[0].PYPELINE == 'SlicerIFU':
             indx = self.NAME == name
         else:
             msgs.error("The '{0:s}' PYPELINE is not defined".format(self[0].PYPELINE))
         return indx
 
-    def slitorder_objid_indices(self, slitorder, objid, toler=5):
+
+    def slitorder_uniq_id_indices(self, uniq_id, order=None):
         """
-        Return the set of indices matching the input slit/order and the input
-        objid
+        Return the set of indices matching the unique object identifier. 
+        For MultiSlit this is the SPAT_PIXPOS_ID, for Echelle it is the ECH_FRACPOS_ID
+        but the order must also be specified.
+        
+        Parameters
+        ----------
+        object_id : int
+            The unique object identifier for the slit/order of interest.
+        order : int, optional
+            The order for Echelle data. Required for Echelle data. 
+            
+        Returns
+        -------
+        `numpy.ndarray`_
+            Array of indices with the corresponding object ID. Shape is (nobj,).
         
-        Args:
-            slitorder (int):
-                Order/Spatial pixel value for slit of interest.
-            objid (int):
-                ID value for object of interest.
-            toler (int, optional):
-                Tolerance for slit spatial pixel values used for slit
-                identification. Default = 5
-
-        Returns:
-            :obj:`int`: Index value for input slit/order and object ID values
-            for specobjs object.
-
         """
-
         if self[0].PYPELINE == 'Echelle':
-            indx = (self.ECH_ORDER == slitorder) & (self.ECH_OBJID == objid)
+            indx = (self.ECH_ORDER == order) & (self.ECH_FRACPOS_ID == uniq_id)
         elif self[0].PYPELINE == 'MultiSlit':
-            indx = (np.abs(self.SLITID - slitorder) <= toler) & (self.OBJID == objid)
-        elif self[0].PYPELINE == 'IFU':
-            indx = (self.SLITID == slitorder) & (self.OBJID == objid)
-        else:
+            indx = self.SPAT_PIXPOS_ID == uniq_id
+        elif self[0].PYPELINE == 'SlicerIFU':
+            indx = self.SPAT_PIXPOS_ID == uniq_id
+        else: 
             msgs.error("The '{0:s}' PYPELINE is not defined".format(self[0].PYPELINE))
-        #
+        
         return indx
 
     def set_names(self):
@@ -533,6 +608,90 @@ class SpecObjs:
                 chk &= (sub_box or sub_opt)
         return chk
 
+    def apply_flux_calib(self, par, spectrograph, sens, tell=False):
+        """
+        Flux calibrate the  object spectra (``sobjs``) using the provided
+        sensitivity function (``sens``).
+
+        Args:
+            par (:class:`~pypeit.par.pypeitpar.FluxCalibratePar`):
+                Parset object containing parameters governing the flux calibration.
+            spectrograph (:class:`~pypeit.spectrographs.spectrograph.Spectrograph`):
+                PypeIt Spectrograph class
+            sens (:class:`~pypeit.sensfunc.SensFunc`):
+                PypeIt Sensitivity function class
+            tell (:obj:`bool`, optional):
+                If True, apply telluric correction as well. The telluric model
+                comes from the sensitivity function. This is generally only
+                used for std fluxed QA plots.
+        """
+
+        _extinct_correct = (True if sens.algorithm == 'UVIS' else False) \
+            if par['extinct_correct'] is None else par['extinct_correct']
+
+        # TODO enbaling this for now in case someone wants to treat the IFU as a slit spectrograph
+        #  (not recommnneded but useful for quick reductions where you don't want to construct cubes and don't care about DAR).
+        if spectrograph.pypeline in ['MultiSlit','SlicerIFU']:
+            for ii, sci_obj in enumerate(self.specobjs):
+                if sens.wave.shape[1] == 1:
+                    tellmodel = sens.telluric.model['TELLURIC'][0, :] if tell else None
+                    sci_obj.apply_flux_calib(sens.wave[:, 0], sens.zeropoint[:, 0],
+                                             self.header['EXPTIME'],
+                                             extinct_correct=_extinct_correct,
+                                             tellmodel=tellmodel,
+                                             longitude=spectrograph.telescope['longitude'],
+                                             latitude=spectrograph.telescope['latitude'],
+                                             extinctfilepar=par['extinct_file'],
+                                             extrap_sens=par['extrap_sens'],
+                                             airmass=float(self.header['AIRMASS']))
+                elif sens.wave.shape[1] > 1 and sens.splice_multi_det:
+                    # This deals with the multi detector case where the sensitivity function is spliced. Note that
+                    # the final sensitivity function written to disk is  the spliced one. This functionality is only
+                    # used internal to sensfunc.py for fluxing the standard for the QA plot.
+                    tellmodel = sens.telluric.model['TELLURIC'][ii, :] if tell else None
+                    sci_obj.apply_flux_calib(sens.wave[:, ii], sens.zeropoint[:, ii],
+                                             self.header['EXPTIME'],
+                                             extinct_correct=_extinct_correct,
+                                             tellmodel=tellmodel,
+                                             longitude=spectrograph.telescope['longitude'],
+                                             latitude=spectrograph.telescope['latitude'],
+                                             extinctfilepar=par['extinct_file'],
+                                             extrap_sens=par['extrap_sens'],
+                                             airmass=float(self.header['AIRMASS']))
+                else:
+                    msgs.error('This should not happen, there is a problem with your sensitivity function.')
+
+
+        elif spectrograph.pypeline == 'Echelle':
+            # Flux calibrate the orders that are mutually in the meta_table and in
+            # the sobjs. This allows flexibility for applying to data for cases
+            # where not all orders are present in the data as in the sensfunc, etc.,
+            # i.e. X-shooter with the K-band blocking filter.
+            ech_orders = np.array(sens.sens['ECH_ORDERS']).flatten()
+            for sci_obj in self.specobjs:
+                # JFH Is there a more elegant pythonic way to do this without looping over both orders and sci_obj?
+                indx = np.where(ech_orders == sci_obj.ECH_ORDER)[0]
+                if indx.size == 1:
+                    tellmodel = sens.telluric.model['TELLURIC'][indx[0], :] if tell else None
+                    sci_obj.apply_flux_calib(sens.wave[:, indx[0]], sens.zeropoint[:, indx[0]],
+                                             self.header['EXPTIME'],
+                                             extinct_correct=_extinct_correct,
+                                             tellmodel=tellmodel,
+                                             extrap_sens=par['extrap_sens'],
+                                             longitude=spectrograph.telescope['longitude'],
+                                             latitude=spectrograph.telescope['latitude'],
+                                             extinctfilepar=par['extinct_file'],
+                                             airmass=float(self.header['AIRMASS']))
+                elif indx.size == 0:
+                    msgs.info('Unable to flux calibrate order = {:} as it is not in your sensitivity function. '
+                              'Something is probably wrong with your sensitivity function.'.format(sci_obj.ECH_ORDER))
+                else:
+                    msgs.error('This should not happen')
+
+        else:
+            msgs.error('Unrecognized pypeline: {0}'.format(spectrograph.pypeline))
+
+
     def copy(self):
         """
         Generate a copy of self
@@ -621,16 +780,23 @@ class SpecObjs:
 
         Args:
             subheader (:obj:`dict`):
+                Dictionary with header keywords and values to be added to the
+                primary header of the output file.
             outfile (str):
+                Name of the output file
             overwrite (bool, optional):
+                Overwrite the output file if it exists?
+            update_det (int or list, optional):
+              If provided, do not clobber the existing file but only update
+              the indicated detectors.  Useful for re-running on a subset of detectors
             slitspatnum (:obj:`str` or :obj:`list`, optional):
               Restricted set of slits for reduction.
               If provided, do not clobber the existing file but only update
               the indicated slits.  Useful for re-running on a subset of slits
-            update_det (int or list, optional):
-              If provided, do not clobber the existing file but only update
-              the indicated detectors.  Useful for re-running on a subset of detectors
-
+            history (:obj:`str`, optional):
+                String to be added to the header HISTORY keyword.
+            debug (:obj:`bool`, optional):
+                If True, run in debug mode.
         """
         if os.path.isfile(outfile) and not overwrite:
             msgs.warn(f'{outfile} exits. Set overwrite=True to overwrite it.')
@@ -660,7 +826,7 @@ class SpecObjs:
             _specobjs = self.specobjs
 
         # Build up the Header
-        header = io.initialize_header(primary=True)
+        header = io.initialize_header()
         for key in subheader.keys():
             if key.upper() == 'HISTORY':
                 if history is None:
@@ -668,11 +834,23 @@ class SpecObjs:
                         header[key.upper()] = line
             else:
                 header[key.upper()] = subheader[key]
+                # Also store the datetime in ISOT format
+                if key.upper() == 'MJD':
+                    if isinstance(subheader[key], (list, tuple)):
+                        mjdval = subheader[key][0]
+                    elif isinstance(subheader[key], float):
+                        mjdval = subheader[key]
+                    else:
+                        raise ValueError('Header card must be a float or a FITS header tuple')
+                    header['DATETIME'] = (Time(mjdval, format='mjd').isot, "Date and time of the observation in ISOT format")
+        # Add calibration associations to Header
+        if self.calibs is not None:
+            for key, val in self.calibs.items():
+                header[f'CLBS_{key}'] = val
 
         # Init
-        prihdu = fits.PrimaryHDU()
+        prihdu = fits.PrimaryHDU(header=header)
         hdus = [prihdu]
-        prihdu.header = header
 
         # Add class info
         prihdu.header['DMODCLS'] = (self.__class__.__name__, 'Datamodel class')
@@ -728,9 +906,6 @@ class SpecObjs:
         # A few more for the header
         prihdu.header['NSPEC'] = nspec
 
-        # Code versions
-        io.initialize_header(hdr=prihdu.header)
-
         # Finish
         hdulist = fits.HDUList(hdus)
         if debug:
@@ -738,8 +913,7 @@ class SpecObjs:
              #embed()
              #exit()
         hdulist.writeto(outfile, overwrite=overwrite)
-        msgs.info("Wrote 1D spectra to {:s}".format(outfile))
-        return
+        msgs.info(f'Wrote 1D spectra to {outfile}')
 
     def write_info(self, outfile, pypeline):
         """
@@ -751,65 +925,42 @@ class SpecObjs:
         """
         # TODO -- Deal with update_det
         # Lists for a Table
-        slits, names, maskdef_id, objname, objra, objdec, spat_pixpos, spat_fracpos, boxsize, opt_fwhm, s2n = \
-            [], [], [], [], [], [], [], [], [], [], []
+        slits, names, obj_ids, maskdef_id, objname, objra, objdec, spat_pixpos, spat_fracpos, boxsize, opt_fwhm, s2n = \
+            [], [], [], [], [], [], [], [], [], [], [], []
         wave_rms = []
         maskdef_extract = []
         manual_extract = []
-        # binspectral, binspatial = parse.parse_binning(binning)
         for specobj in self.specobjs:
-            det = specobj.DET
             if specobj is None:
                 continue
-            # Detector items
-            binspectral, binspatial = parse.parse_binning(specobj.DETECTOR.binning)
-            platescale = specobj.DETECTOR.platescale
             # Append
             spat_pixpos.append(specobj.SPAT_PIXPOS)
             if pypeline == 'MultiSlit':
                 spat_fracpos.append(specobj.SPAT_FRACPOS)
                 slits.append(specobj.SLITID)
                 names.append(specobj.NAME)
-            elif pypeline == 'IFU':
+                obj_ids.append(specobj.SPAT_PIXPOS_ID)
+            elif pypeline == 'SlicerIFU':
                 spat_fracpos.append(specobj.SPAT_FRACPOS)
                 slits.append(specobj.SLITID)
                 names.append(specobj.NAME)
+                obj_ids.append(specobj.SPAT_PIXPOS_ID)
             elif pypeline == 'Echelle':
                 spat_fracpos.append(specobj.ECH_FRACPOS)
                 slits.append(specobj.ECH_ORDER)
                 names.append(specobj.ECH_NAME)
+                obj_ids.append(specobj.ECH_FRACPOS_ID)
             # Wave RMS
             wave_rms.append(specobj.WAVE_RMS)
             # Boxcar width
-            if specobj.BOX_RADIUS is not None:
-                slit_pix = 2.0 * specobj.BOX_RADIUS
-                # Convert to arcsec
-                binspectral, binspatial = parse.parse_binning(specobj.DETECTOR.binning)
-                #binspectral, binspatial = parse.parse_binning(binning)
-                # JFH TODO This should be using the order_platescale for each order. Furthermore, not all detectors
-                # have the same platescale, i.e. with GNIRS it is the same detector but a different camera hence a
-                # different attribute. platescale should be a spectrograph attribute determined on the fly.
-                # boxsize.append(slit_pix*binspatial*spectrograph.detector[specobj.DET-1]['platescale'])
-                boxsize.append(slit_pix * binspatial * platescale)
-            else:
-                boxsize.append(0.)
-
+            boxsize.append(2*specobj.BOX_R_ASEC)
             # Optimal profile (FWHM)
+            opt_fwhm.append(specobj.SPAT_FWHM)
             # S2N -- default to boxcar
-            if specobj.FWHMFIT is not None and specobj.OPT_COUNTS is not None:
-                opt_fwhm.append(np.median(specobj.FWHMFIT) * binspatial * platescale)
-                # S2N -- optimal
-                ivar = specobj.OPT_COUNTS_IVAR
-                is2n = np.median(specobj.OPT_COUNTS * np.sqrt(ivar))
-                s2n.append(is2n)
-            else:  # Optimal is not required to occur
-                opt_fwhm.append(0.)
-                if specobj.BOX_COUNTS is None:
-                    is2n = 0.
-                else:
-                    ivar = specobj.BOX_COUNTS_IVAR
-                    is2n = np.median(specobj.BOX_COUNTS * np.sqrt(ivar))
-                s2n.append(is2n)
+            # NOTE: Below requires that S2N not be None, otherwise the code will
+            # fault.  If the code gets here and S2N is None, check that 1D
+            # extractions have been performed.
+            s2n.append(specobj.S2N)
             # Manual extraction?
             manual_extract.append(specobj.hand_extract_flag)
             # Slitmask info
@@ -825,13 +976,14 @@ class SpecObjs:
             if pypeline == 'MultiSlit':
                 obj_tbl['slit'] = slits
                 obj_tbl['slit'].format = 'd'
-            elif pypeline == 'IFU':
+            elif pypeline == 'SlicerIFU':
                 obj_tbl['slit'] = slits
                 obj_tbl['slit'].format = 'd'
             elif pypeline == 'Echelle':
                 obj_tbl['order'] = slits
                 obj_tbl['order'].format = 'd'
             obj_tbl['name'] = names
+            obj_tbl['obj_id'] = obj_ids
             if not np.all(np.array(maskdef_id) == None):
                 obj_tbl['maskdef_id'] = maskdef_id
             if not np.all(np.array(objname) == None):
@@ -914,6 +1066,30 @@ class SpecObjs:
 
         return groups
 
+    def flexure_diagnostics(self):
+        """
+        Print and return the spectral flexure of a spec1d file.
+
+        Returns:
+            :obj:`astropy.table.Table`: Table with the spectral flexure.
+        """    
+        spec_flex = Table()
+        spec_flex['NAME'] = self.NAME
+        spec_flex['global_spec_shift'] = self.FLEX_SHIFT_GLOBAL
+        if np.all(spec_flex['global_spec_shift'] != None):
+            spec_flex['global_spec_shift'].format = '0.3f'
+        spec_flex['local_spec_shift'] = self.FLEX_SHIFT_LOCAL
+        if np.all(spec_flex['local_spec_shift'] != None):
+            spec_flex['local_spec_shift'].format = '0.3f'
+        spec_flex['total_spec_shift'] = self.FLEX_SHIFT_TOTAL
+        if np.all(spec_flex['total_spec_shift'] != None):
+            spec_flex['total_spec_shift'].format = '0.3f'
+        # print the table
+        spec_flex.pprint_all()
+        # return the table
+        return spec_flex
+
+
 #TODO Should this be a classmethod on specobjs??
 def get_std_trace(detname, std_outfile, chk_version=True):
     """
@@ -924,10 +1100,13 @@ def get_std_trace(detname, std_outfile, chk_version=True):
              1-indexed detector(s) to process.
          std_outfile (:obj:`str`):
              Filename with the standard star spec1d file.  Can be None.
+
      Returns:
-         `numpy.ndarray`_: Trace of the standard star on input detector.
-         Will be None if ``std_outfile`` is None, or if the selected detector/mosaic is not available
-         in the provided spec1d file.
+         `astropy.table.Table`_: Table with the trace of the standard star on the input detector.
+         If this is a MultiSlit reduction, the table will have a single column: `TRACE_SPAT`.
+         If this is an Echelle reduction, the table will have two columns: `ECH_ORDER` and `TRACE_SPAT`.
+         Will be None if ``std_outfile`` is None, or if the selected detector/mosaic
+         is not available in the provided spec1d file, or for SlicerIFU reductions.
      """
 
     sobjs = SpecObjs.from_fitsfile(std_outfile, chk_version=chk_version)
@@ -944,38 +1123,52 @@ def get_std_trace(detname, std_outfile, chk_version=True):
         # No standard extracted on this detector??
         if sobjs_std is None:
             return None
-        std_trace = sobjs_std.TRACE_SPAT
+
+        # create table that contains the trace of the standard
+        std_tab = Table()
         # flatten the array if this multislit
         if 'MultiSlit' in pypeline:
-            std_trace = std_trace.flatten()
+            std_tab['TRACE_SPAT'] = sobjs_std.TRACE_SPAT
         elif 'Echelle' in pypeline:
-            std_trace = std_trace.T
+            std_tab['ECH_ORDER'] = sobjs_std.ECH_ORDER
+            std_tab['TRACE_SPAT'] = sobjs_std.TRACE_SPAT
+        elif 'SlicerIFU' in pypeline:
+            std_tab = None
         else:
             msgs.error('Unrecognized pypeline')
     else:
-        std_trace = None
+        std_tab = None
 
-    return std_trace
+    return std_tab
 
-def lst_to_array(lst, mask=None):
+
+def lst_to_array(lst):
     """
     Simple method to convert a list to an array
 
     Allows for a list of Quantity objects
 
     Args:
-        lst : list
-          Should be number or Quantities
-        mask (ndarray of bool, optional):  Limit to a subset of the list.  True=good
+        lst (:obj:`list`):
+            Should be number or Quantities
 
     Returns:
-        ndarray or Quantity array:  Converted list
-
+        `numpy.ndarray`_, `astropy.units.Quantity`_:  Converted list
     """
-    if mask is None:
-        mask = np.array([True]*len(lst))
+    # Return a Quantity array
     if isinstance(lst[0], units.Quantity):
-        return units.Quantity(lst)[mask]
-    else:
-        return np.array(lst)[mask]
+        return units.Quantity(lst)
+    try:
+        return np.array(lst)
+    except ValueError as e:
+        pass
+    return np.array(lst, dtype=object)
+
+    # NOTE: The dtype="object" is needed for the case where one element of lst
+    # is not a list but None. For example, if trying to unpack SpecObjs OPT fluxes
+    # and for one slit/order the OPT extraction failed (but not the BOX extraction),
+    # OPT_COUNTS is None for that slit/order, and lst would be something like
+    # [array, array, array, None, array], which makes np.array to fail and give the error
+    # "ValueError: setting an array element with a sequence. The requested array has an
+    # inhomogeneous shape after 1 dimensions..."
 
