@@ -4,8 +4,8 @@ files.
 
 .. include:: ../include/links.rst
 """
-import os
-import glob
+import datetime
+import pathlib
 import re
 import warnings
 
@@ -20,8 +20,6 @@ from astropy.coordinates import SkyCoord, Angle
 from astropy.table import Table
 from astropy import units, time
 
-import datetime
-
 import linetools
 
 from pypeit import msgs
@@ -32,13 +30,11 @@ from pypeit.core import framematch
 from pypeit.core import wave
 from pypeit import specobj, specobjs
 from pypeit.spectrographs import spectrograph
-from pypeit.images import detector_container
-from pypeit import data
+from pypeit.images.detector_container import DetectorContainer
+from pypeit import dataPaths
 from pypeit.images.mosaic import Mosaic
 from pypeit.core.mosaic import build_image_mosaic_transform
 
-from pypeit.utils import index_of_x_eq_y
-
 from pypeit.spectrographs import slitmask 
 from pypeit.spectrographs.opticalmodel import ReflectionGrating, OpticalModel, DetectorMap
 
@@ -141,7 +137,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
             specflip        = False,
             spatflip        = False,
             platescale      = 0.1185,
-            darkcurr        = 3.30, # changed by DP. Taken from WMKO measurements on 2022-Apr-22
+            darkcurr        = 3.30, #  units are e-/pixel/hour... NOTE : changed by DP. Taken from WMKO measurements on 2022-Apr-22
             saturation      = 65535., # ADU
             nonlinear       = 0.95,   # Changed by JFH from 0.86 to 0.95
             mincounts       = -1e10,
@@ -154,7 +150,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         detector_dict2.update(dict(
             det=2,
             dataext=2,
-            darkcurr=3.60,
+            darkcurr=3.60,  # e-/pixel/hour
             gain=np.atleast_1d(1.188),
             ronoise=np.atleast_1d(2.491),
         ))
@@ -163,7 +159,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         detector_dict3.update(dict(
             det=3,
             dataext=3,
-            darkcurr=3.50,
+            darkcurr=3.50,  # e-/pixel/hour
             gain=np.atleast_1d(1.248),
             ronoise=np.atleast_1d(2.618),
         ))
@@ -172,7 +168,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         detector_dict4.update(dict(
             det=4,
             dataext=4,
-            darkcurr=3.70,
+            darkcurr=3.70,  # e-/pixel/hour
             gain=np.atleast_1d(1.220),
             ronoise=np.atleast_1d(2.557),
         ))
@@ -181,7 +177,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         detector_dict5.update(dict(
             det=5,
             dataext=5,
-            darkcurr=2.70,
+            darkcurr=2.70,  # e-/pixel/hour
             gain=np.atleast_1d(1.184),
             ronoise=np.atleast_1d(2.482),
         ))
@@ -190,7 +186,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         detector_dict6.update(dict(
             det=6,
             dataext=6,
-            darkcurr=3.80,
+            darkcurr=3.80,  # e-/pixel/hour
             gain=np.atleast_1d(1.177),
             ronoise=np.atleast_1d(2.469),
         ))
@@ -199,7 +195,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         detector_dict7.update(dict(
             det=7,
             dataext=7,
-            darkcurr=3.30,
+            darkcurr=3.30,  # e-/pixel/hour
             gain=np.atleast_1d(1.201),
             ronoise=np.atleast_1d(2.518),
         ))
@@ -208,18 +204,23 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         detector_dict8.update(dict(
             det=8,
             dataext=8,
-            darkcurr=3.70,
+            darkcurr=3.70,  # e-/pixel/hour
             gain=np.atleast_1d(1.230),
             ronoise=np.atleast_1d(2.580),
         ))
 
         if hdu is not None:
+            amp = self.get_meta_value(self.get_headarr(hdu), 'amp')
+            if amp == 'DUAL:A+B':
+                msgs.error('PypeIt can only reduce images with AMPMODE == SINGLE:B or AMPMODE == SINGLE:A.')
+            amp_folder = "ampA" if amp == 'SINGLE:A' else "ampB"
             # raw frame date in mjd
             date = time.Time(self.get_meta_value(self.get_headarr(hdu), 'mjd'), format='mjd').value
             # get the measurements files
-            measure_files = np.array(glob.glob(os.path.join(data.Paths.spectrographs, "keck_deimos/gain_ronoise", "*")))
+            # NOTE: The use of ``glob`` here *requires* that the files be on disk
+            measure_files = sorted((dataPaths.spectrographs / "keck_deimos" / "gain_ronoise" / amp_folder).glob("*"))
             # Parse the dates recorded in the name of the files
-            measure_dates = np.array([os.path.basename(f).split('.')[2] for f in measure_files])
+            measure_dates = np.array([f.name.split('.')[2] for f in measure_files])
             # convert into datetime format
             dtime = np.array([datetime.datetime.strptime(mm, '%Y-%b-%d') for mm in measure_dates])
             # convert to mjd
@@ -229,33 +230,48 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
             # get measurements
             tab_measure = Table.read(measure_files[close_idx], format='ascii')
             measured_det = tab_measure['col3']
+            measured_amptype = tab_measure['col4']
             measured_gain = tab_measure['col5']  # [e-/DN]
             measured_ronoise = tab_measure['col7']   # [e-]
-            msgs.info(f"We are using DEIMOS gain/RN values based on WMKO estimates on {measure_dates[close_idx]}.")
+            msgs.info(f"We are using DEIMOS gain/RN values for AMPMODE = {amp} "
+                      f"based on WMKO estimates on {measure_dates[close_idx]}.")
+            # find values for this amp and each detector
+            this_amp = measured_amptype == 'A' if amp == 'SINGLE:A' else measured_amptype == 'B'
+            det_1 = this_amp & (measured_det == 1)
+            det_2 = this_amp & (measured_det == 2)
+            det_3 = this_amp & (measured_det == 3)
+            det_4 = this_amp & (measured_det == 4)
+            # we don't have measurements for the red detectors when amp = 'SINGLE:A',
+            # therefore we use the values for the blue detectors
+            det_5 = this_amp & (measured_det == 5 if amp == 'SINGLE:B' else measured_det == 1)
+            det_6 = this_amp & (measured_det == 6 if amp == 'SINGLE:B' else measured_det == 2)
+            det_7 = this_amp & (measured_det == 7 if amp == 'SINGLE:B' else measured_det == 3)
+            det_8 = this_amp & (measured_det == 8 if amp == 'SINGLE:B' else measured_det == 4)
+
             # get gain
-            detector_dict1['gain'] = measured_gain[measured_det == 1]
-            detector_dict2['gain'] = measured_gain[measured_det == 2]
-            detector_dict3['gain'] = measured_gain[measured_det == 3]
-            detector_dict4['gain'] = measured_gain[measured_det == 4]
-            detector_dict5['gain'] = measured_gain[measured_det == 5]
-            detector_dict6['gain'] = measured_gain[measured_det == 6]
-            detector_dict7['gain'] = measured_gain[measured_det == 7]
-            detector_dict8['gain'] = measured_gain[measured_det == 8]
+            detector_dict1['gain'] = measured_gain[det_1].data
+            detector_dict2['gain'] = measured_gain[det_2].data
+            detector_dict3['gain'] = measured_gain[det_3].data
+            detector_dict4['gain'] = measured_gain[det_4].data
+            detector_dict5['gain'] = measured_gain[det_5].data
+            detector_dict6['gain'] = measured_gain[det_6].data
+            detector_dict7['gain'] = measured_gain[det_7].data
+            detector_dict8['gain'] = measured_gain[det_8].data
 
             # get ronoise
-            detector_dict1['ronoise'] = measured_ronoise[measured_det == 1]
-            detector_dict2['ronoise'] = measured_ronoise[measured_det == 2]
-            detector_dict3['ronoise'] = measured_ronoise[measured_det == 3]
-            detector_dict4['ronoise'] = measured_ronoise[measured_det == 4]
-            detector_dict5['ronoise'] = measured_ronoise[measured_det == 5]
-            detector_dict6['ronoise'] = measured_ronoise[measured_det == 6]
-            detector_dict7['ronoise'] = measured_ronoise[measured_det == 7]
-            detector_dict8['ronoise'] = measured_ronoise[measured_det == 8]
+            detector_dict1['ronoise'] = measured_ronoise[det_1].data
+            detector_dict2['ronoise'] = measured_ronoise[det_2].data
+            detector_dict3['ronoise'] = measured_ronoise[det_3].data
+            detector_dict4['ronoise'] = measured_ronoise[det_4].data
+            detector_dict5['ronoise'] = measured_ronoise[det_5].data
+            detector_dict6['ronoise'] = measured_ronoise[det_6].data
+            detector_dict7['ronoise'] = measured_ronoise[det_7].data
+            detector_dict8['ronoise'] = measured_ronoise[det_8].data
 
         detectors = [detector_dict1, detector_dict2, detector_dict3, detector_dict4,
                      detector_dict5, detector_dict6, detector_dict7, detector_dict8]
         # Return
-        return detector_container.DetectorContainer(**detectors[det-1])
+        return DetectorContainer(**detectors[det-1])
 
     @classmethod
     def default_pypeit_par(cls):
@@ -264,7 +280,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         
         Returns:
             :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
-            all of ``PypeIt`` methods.
+            all of PypeIt methods.
         """
         par = super().default_pypeit_par()
 
@@ -305,12 +321,12 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         par['scienceframe']['process']['objlim'] = 1.5
 
         # If telluric is triggered
-        par['sensfunc']['IR']['telgridfile'] = 'TelFit_MaunaKea_3100_26100_R20000.fits'
+        par['sensfunc']['IR']['telgridfile'] = 'TellPCA_3000_26000_R15000.fits'
         return par
 
     def config_specific_par(self, scifile, inp_par=None):
         """
-        Modify the ``PypeIt`` parameters to hard-wired values used for
+        Modify the PypeIt parameters to hard-wired values used for
         specific instrument configurations.
 
         Args:
@@ -329,9 +345,14 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
 
         headarr = self.get_headarr(scifile)
 
-        # When using LVM mask reduce only detectors 3,7
-        if 'LVMslit' in self.get_meta_value(headarr, 'decker'):
-            par['rdx']['detnum'] = [(3,7)]
+        # When using LVM mask or AMPMODE = SINGLE:A reduce only detectors 3,7
+        if ('LVMslit' in self.get_meta_value(headarr, 'decker') or
+                self.get_meta_value(headarr, 'amp') == 'SINGLE:A'):
+            # give an info message if AMPMODE = SINGLE:A
+            if self.get_meta_value(headarr, 'amp') == 'SINGLE:A':
+                msgs.info('Data taken with AMPMODE = SINGLE:A. Only detectors 3,7 will be reduced. To change this,'
+                          ' modify the detnum parameter in the pypeit file.')
+            par['rdx']['detnum'] = [(3, 7)]
 
         # Turn PCA off for long slits
         # TODO: I'm a bit worried that this won't catch all
@@ -365,6 +386,9 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
             # pca
             par['calibrations']['slitedges']['sync_predict'] = 'auto'
 
+            # set offsets for coadd2d
+            par['coadd2d']['offsets'] = 'maskdef_offsets'
+
 
         # Templates
         if self.get_meta_value(headarr, 'dispname') == '600ZD':
@@ -395,13 +419,12 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
 
         # Wavelength FWHM
         binning = parse.parse_binning(self.get_meta_value(headarr, 'binning'))
-        par['calibrations']['wavelengths']['fwhm'] = 6.0 / binning[1]
-        par['calibrations']['wavelengths']['fwhm_fromlines'] = True
+        par['calibrations']['wavelengths']['fwhm'] = 6.0 / binning[0]
 
         # Objects FWHM
         # Find objects
         #  The following corresponds to 0.8"
-        par['reduce']['findobj']['find_fwhm'] = 7.0 / binning[0]
+        par['reduce']['findobj']['find_fwhm'] = 7.0 / binning[1]
 
         # Return
         return par
@@ -429,7 +452,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         """
         Define how metadata are derived from the spectrograph files.
 
-        That is, this associates the ``PypeIt``-specific metadata keywords
+        That is, this associates the PypeIt-specific metadata keywords
         with the instrument-specific header cards using :attr:`meta`.
         """
         self.meta = {}
@@ -514,6 +537,26 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         # removed from the list of valid frames in PypeItMetaData.
         return ['dispname', 'decker', 'binning', 'dispangle', 'amp', 'filter1']
 
+    def raw_header_cards(self):
+        """
+        Return additional raw header cards to be propagated in
+        downstream output files for configuration identification.
+
+        The list of raw data FITS keywords should be those used to populate
+        the :meth:`~pypeit.spectrographs.spectrograph.Spectrograph.configuration_keys`
+        or are used in :meth:`~pypeit.spectrographs.spectrograph.Spectrograph.config_specific_par`
+        for a particular spectrograph, if different from the name of the
+        PypeIt metadata keyword.
+
+        This list is used by :meth:`~pypeit.spectrographs.spectrograph.Spectrograph.subheader_for_spec`
+        to include additional FITS keywords in downstream output files.
+
+        Returns:
+            :obj:`list`: List of keywords from the raw data files that should
+            be propagated in output files.
+        """
+        return ['GRATENAM', 'SLMSKNAM', 'G3TLTWAV', 'G4TLTWAV', 'AMPMODE', 'DWFILNAM']
+
     def valid_configuration_values(self):
         """
         Return a fixed set of valid values for any/all of the configuration
@@ -524,7 +567,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
             and their associated discrete set of valid values. If there are
             no restrictions on configuration values, None is returned.
         """
-        return {'amp': ['SINGLE:B'], 'mode':['Spectral']}
+        return {'amp': ['SINGLE:B', 'SINGLE:A'], 'mode':['Spectral']}
 
     def config_independent_frames(self):
         """
@@ -542,11 +585,11 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
             keywords that can be used to assign the frames to a configuration
             group.
         """
-        return {'bias': 'dateobs', 'dark': 'dateobs'}
+        return {'bias': ['dateobs', 'binning', 'amp'], 'dark': ['dateobs', 'binning', 'amp']}
 
     def pypeit_file_keys(self):
         """
-        Define the list of keys to be output into a standard ``PypeIt`` file.
+        Define the list of keys to be output into a standard PypeIt file.
 
         Returns:
             :obj:`list`: The list of keywords in the relevant
@@ -559,7 +602,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
                            allow_missing=False):
         """
         Generate a dict that will be added to the Header of spectra files
-        generated by ``PypeIt`` (e.g. :class:`~pypeit.specobjs.SpecObjs`).
+        generated by PypeIt (e.g. :class:`~pypeit.specobjs.SpecObjs`).
         This version overrides the parent version to include KOA specific header cards.
 
         Args:
@@ -638,34 +681,6 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         msgs.warn('Cannot determine if frames are of type {0}.'.format(ftype))
         return np.zeros(len(fitstbl), dtype=bool)
 
-    # TODO: We should aim to get rid of this... I'm not sure it's ever used...
-    def idname(self, ftype):
-        """
-        Return the ``idname`` for the selected frame type for this
-        instrument.
-
-        Args:
-            ftype (:obj:`str`):
-                Frame type, which should be one of the keys in
-                :class:`~pypeit.core.framematch.FrameTypeBitMask`.
-
-        Returns:
-            :obj:`str`: The value of ``idname`` that should be available in
-            the :class:`~pypeit.metadata.PypeItMetaData` instance that
-            identifies frames of this type.
-        """
-        # TODO: Fill in the rest of these.
-        name = { 'arc': 'Line',
-                 'tilt': None,
-                 'bias': None,
-                 'dark': None,
-                 'pinhole': None,
-                 'pixelflat': 'IntFlat',
-                 'science': 'Object',
-                 'standard': None,
-                 'trace': 'IntFlat' }
-        return name[ftype]
-
     def get_rawimage(self, raw_file, det):
         """
         Read raw images and generate a few other bits and pieces
@@ -677,10 +692,10 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
 
         .. warning::
 
-            ``PypeIt`` currently *cannot* reduce images produced by
-            reading the DEIMOS CCDs with the A amplifier or those
+            PypeIt currently *cannot* reduce images produced by
+            reading the DEIMOS CCDs with the A+B amplifier or those
             taken in imaging mode. All image handling assumes DEIMOS
-            images have been read with the B amplifier in the
+            images have been read with the B or A amplifier in the
             "Spectral" observing mode. This method will fault if this
             is not true based on the header keywords MOSMODE and
             AMPMODE.
@@ -723,8 +738,8 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         mosaic = None if nimg == 1 else self.get_mosaic_par(det, hdu=hdu)
         detectors = [self.get_detector_par(det, hdu=hdu)] if nimg == 1 else mosaic.detectors
 
-        if hdu[0].header['AMPMODE'] != 'SINGLE:B':
-            msgs.error('PypeIt can only reduce images with AMPMODE == SINGLE:B.')
+        if hdu[0].header['AMPMODE'] not in ['SINGLE:B', 'SINGLE:A']:
+            msgs.error('PypeIt can only reduce images with AMPMODE == SINGLE:B or AMPMODE == SINGLE:A.')
         if hdu[0].header['MOSMODE'] != 'Spectral':
             msgs.error('PypeIt can only reduce images with MOSMODE == Spectral.')
 
@@ -781,13 +796,13 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
             return detectors[0], image[0], hdu, exptime, rawdatasec_img[0], oscansec_img[0]
         return mosaic, image, hdu, exptime, rawdatasec_img, oscansec_img
 
-    def get_mosaic_par(self, mosaic, hdu=None, msc_order=5):
+    def get_mosaic_par(self, mosaic, hdu=None, msc_ord=5):
         """
         Return the hard-coded parameters needed to construct detector mosaics
         from unbinned images.
 
         The parameters expect the images to be trimmed and oriented to follow
-        the ``PypeIt`` shape convention of ``(nspec,nspat)``.  For returned
+        the PypeIt shape convention of ``(nspec,nspat)``.  For returned
         lists, the length of the list is the same as the number of detectors in
         the mosaic, and they are ordered by the detector number.
 
@@ -802,7 +817,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
                 default.  BEWARE: If ``hdu`` is not provided, the binning is
                 assumed to be `1,1`, which will cause faults if applied to
                 binned images!
-            msc_order (:obj:`int`, optional):
+            msc_ord (:obj:`int`, optional):
                 Order of the interpolation used to construct the mosaic.
 
         Returns:
@@ -849,7 +864,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
             msc_tfm[i] = build_image_mosaic_transform(shape, msc_sft[i], msc_rot[i], binning)
 
         return Mosaic(mosaic_id, detectors, shape, np.array(msc_sft), np.array(msc_rot),
-                      np.array(msc_tfm), msc_order)
+                      np.array(msc_tfm), msc_ord)
 
     @property
     def allowed_mosaics(self):
@@ -859,7 +874,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         Returns:
             :obj:`list`: List of tuples, where each tuple provides the 1-indexed
             detector numbers that can be combined into a mosaic and processed by
-            ``PypeIt``.
+            PypeIt.
         """
         return [(1,5),(2,6),(3,7),(4,8)]
 
@@ -887,7 +902,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
                 Required if filename is None
                 Ignored if filename is not None
             msbias (`numpy.ndarray`_, optional):
-                Master bias frame used to identify bad pixels
+                Processed bias frame used to identify bad pixels
 
         Returns:
             `numpy.ndarray`_: An integer array with a masked value set
@@ -1187,13 +1202,13 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         # Grating slider
         slider = hdu[0].header['GRATEPOS']
 
-        mp_dir = os.path.join(data.Paths.static_calibs, 'keck_deimos')
-
         if slider in [3,4]:
-            self.amap = fits.getdata(os.path.join(mp_dir, f'amap.s{slider}.2003mar04.fits'))
-            self.bmap = fits.getdata(os.path.join(mp_dir, f'bmap.s{slider}.2003mar04.fits'))
+            self.amap = fits.getdata(dataPaths.static_calibs.get_file_path(
+                                        f'keck_deimos/amap.s{slider}.2003mar04.fits'))
+            self.bmap = fits.getdata(dataPaths.static_calibs.get_file_path(
+                                        f'keck_deimos/bmap.s{slider}.2003mar04.fits'))
         else:
-            msgs.error('No amap/bmap available for slider {0}. Set `use_maskdesign = False`'.format(slider))
+            msgs.error(f'No amap/bmap available for slider {slider}. Set `use_maskdesign = False`')
         #TODO: Figure out which amap and bmap to use for slider 2
 
         return self.amap, self.bmap
@@ -1320,7 +1335,8 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         # Use the detector map to convert to the detector coordinates
         return (x_img, y_img) + self.detector_map.ccd_coordinates(x_img, y_img, in_mm=False)
 
-    def get_maskdef_slitedges(self, ccdnum=None, filename=None, debug=None):
+    def get_maskdef_slitedges(self, ccdnum=None, filename=None, debug=None,
+                              trc_path=None, binning=None):
         """
         Provides the slit edges positions predicted by the slitmask design using
         the mask coordinates already converted from mm to pixels by the method
@@ -1504,11 +1520,8 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
             the array is 2D, there are detectors separated along the dispersion
             axis.
         """
-        if mosaic:
-            return np.array([self.get_det_name(_det) for _det in self.allowed_mosaics])
-        else:
-            return np.array([detector_container.DetectorContainer.get_name(i+1)
-                             for i in range(self.ndet)]).reshape(2,-1)
+        dets = super().list_detectors(mosaic=mosaic)
+        return dets if mosaic else dets.reshape(2,-1)
 
     def spec1d_match_spectra(self, sobjs):
         """
@@ -1529,7 +1542,7 @@ class KeckDEIMOSSpectrograph(spectrograph.Spectrograph):
         # MATCH RED TO BLUE VIA RA/DEC
 #        mb = sobjs['DET'] <=4
 #        mr = sobjs['DET'] >4
-        det = np.array([detector_container.DetectorContainer.parse_name(d) for d in sobjs.DET])
+        det = np.array([DetectorContainer.parse_name(d) for d in sobjs.DET])
         mb = det <= 4
         mr = det > 4
 
@@ -1884,7 +1897,7 @@ def deimos_read_1chip(hdu,chipno):
     return data, oscan
 
 
-def load_wmko_std_spectrum(fits_file:str, outfile=None, pad = False):
+def load_wmko_std_spectrum(fits_file:str, outfile=None, pad = False, split=True):
     """Load up a Standard spectrum generated by WMKO IDL scripts
     of the great Greg Wirth
 
@@ -1893,6 +1906,9 @@ def load_wmko_std_spectrum(fits_file:str, outfile=None, pad = False):
     Args:
         fits_file (str): filename
         outfile ([type], optional): Write the SpecObjs object to a FITS file. Defaults to None.
+        pad (bool,optional): True if the resulting SpecObjs should be padded to an even length. Defaults to False
+        split (bool,optional): True if the resulting SpecObjs should be split into two detectors, False if it
+                               should be treated as a mosaic. Defaults to True.
 
     Returns:
         specobjs.SpecObjs: object holding the spectra
@@ -1904,7 +1920,10 @@ def load_wmko_std_spectrum(fits_file:str, outfile=None, pad = False):
     idl_spec = Table(hdul[2].data)
 
     # Hope this always works..
-    npix = int(len(idl_spec)/2)
+    if split:
+        npix = int(len(idl_spec)/2)
+    else:
+        npix = len(idl_spec)
 
     if pad:
         if int(len(idl_spec)) % 2 != 0:
@@ -1915,27 +1934,40 @@ def load_wmko_std_spectrum(fits_file:str, outfile=None, pad = False):
     idl_vac = wave.airtovac(idl_spec['WAVELENGTH']*units.AA)
 
     # Generate SpecObj
-    sobj1 = specobj.SpecObj.from_arrays('MultiSlit', idl_vac.value[0:npix],
-                                  idl_spec['COUNTS'].data[0:npix], 
-                                   1./(idl_spec['COUNTS'].data[0:npix]),
-                                   DET='DET03')
-    sobj2 = specobj.SpecObj.from_arrays('MultiSlit', idl_vac.value[npix:],
-                                  idl_spec['COUNTS'].data[npix:], 
-                                   1./(idl_spec['COUNTS'].data[npix:]), 
-                                   DET='DET07')
+    if not split:
+        sobj1 = specobj.SpecObj.from_arrays('MultiSlit', idl_vac.value[0:npix],
+                                            idl_spec['COUNTS'].data[0:npix],
+                                            1./(idl_spec['COUNTS'].data[0:npix]),
+                                            np.ones(idl_spec['COUNTS'].data[0:npix].size),
+                                            DET='MSC03')
+    else:
+        sobj1 = specobj.SpecObj.from_arrays('MultiSlit', idl_vac.value[0:npix],
+                                            idl_spec['COUNTS'].data[0:npix],
+                                            1./(idl_spec['COUNTS'].data[0:npix]),
+                                            np.ones(idl_spec['COUNTS'].data[0:npix].size),
+                                            DET='DET03')
+        
+        sobj2 = specobj.SpecObj.from_arrays('MultiSlit', idl_vac.value[npix:],
+                                            idl_spec['COUNTS'].data[npix:],
+                                            1./(idl_spec['COUNTS'].data[npix:]),
+                                            np.ones(idl_spec['COUNTS'].data[npix:].size),
+                                            DET='DET07')
 
     # SpecObjs
     sobjs = specobjs.SpecObjs()
     sobjs.add_sobj(sobj1)
-    sobjs.add_sobj(sobj2)
+    if split:
+        sobjs.add_sobj(sobj2)
 
     # Fill in header
     coord = linetools.utils.radec_to_coord((meta['RA'][0], meta['DEC'][0]))
     sobjs.header = dict(EXPTIME=1., 
-                        AIRMASS=float(meta['AIRMASS']), 
+                        AIRMASS=float(meta['AIRMASS'][0]), 
                         DISPNAME=str(meta['GRATING'][0]), 
+                        SLMSKNAM='', # No slitmask is used for the throughput data
                         PYP_SPEC='keck_deimos', 
                         PYPELINE='MultiSlit',
+                        BINNING=f'{meta["SPEC_BIN"][0]},{meta["SPEC_BIN"][0]}',
                         RA=coord.ra.deg, 
                         DEC=coord.dec.deg
                    )
