@@ -3,8 +3,7 @@ Module for P200/DBSP specific methods.
 
 .. include:: ../include/links.rst
 """
-from typing import List
-from pkg_resources import resource_filename
+from typing import List, Optional
 
 import numpy as np
 
@@ -14,6 +13,7 @@ from astropy import units as u
 from astropy.time import Time
 
 from pypeit import msgs
+from pypeit import io
 from pypeit import telescopes
 from pypeit.core import framematch
 from pypeit.spectrographs import spectrograph
@@ -21,9 +21,6 @@ from pypeit.core import parse
 from pypeit.images import detector_container
 
 
-def flip_fits_slice(s: str) -> str:
-    return '[' + ','.join(s.strip('[]').split(',')[::-1]) + ']'
-
 
 class P200DBSPSpectrograph(spectrograph.Spectrograph):
     """
@@ -31,28 +28,13 @@ class P200DBSPSpectrograph(spectrograph.Spectrograph):
     """
     ndet = 1
     telescope = telescopes.P200TelescopePar()
-
-    def configuration_keys(self):
-        """
-        Return the metadata keys that define a unique instrument
-        configuration.
-
-        This list is used by :class:`~pypeit.metadata.PypeItMetaData` to
-        identify the unique configurations among the list of frames read
-        for a given reduction.
-
-        Returns:
-            :obj:`list`: List of keywords of data pulled from file headers
-            and used to constuct the :class:`~pypeit.metadata.PypeItMetaData`
-            object.
-        """
-        return ['dispname', 'binning', 'dispangle', 'dichroic']
+    url = 'https://sites.astro.caltech.edu/palomar/observer/200inchResources/dbspoverview.html'
 
     def init_meta(self):
         """
         Define how metadata are derived from the spectrograph files.
 
-        That is, this associates the ``PypeIt``-specific metadata keywords
+        That is, this associates the PypeIt-specific metadata keywords
         with the instrument-specific header cards using :attr:`meta`.
         """
         self.meta = {}
@@ -74,6 +56,7 @@ class P200DBSPSpectrograph(spectrograph.Spectrograph):
         self.meta['dispangle'] = dict(card=None, rtol=1e-2, compound=True)
         self.meta['slitwid'] = dict(ext=0, card='APERTURE')
         self.meta['idname'] = dict(ext=0, card='IMGTYPE')
+        self.meta['instrument'] = dict(ext=0, card='FPA')
         # Lamps
         self.meta['lampstat01'] = dict(ext=0, card='LAMPS')
 
@@ -102,9 +85,45 @@ class P200DBSPSpectrograph(spectrograph.Spectrograph):
         else:
             return None
 
+    def configuration_keys(self):
+        """
+        Return the metadata keys that define a unique instrument
+        configuration.
+
+        This list is used by :class:`~pypeit.metadata.PypeItMetaData` to
+        identify the unique configurations among the list of frames read
+        for a given reduction.
+
+        Returns:
+            :obj:`list`: List of keywords of data pulled from file headers
+            and used to constuct the :class:`~pypeit.metadata.PypeItMetaData`
+            object.
+        """
+        return ['dispname', 'binning', 'dispangle', 'dichroic']
+
+    def raw_header_cards(self):
+        """
+        Return additional raw header cards to be propagated in
+        downstream output files for configuration identification.
+
+        The list of raw data FITS keywords should be those used to populate
+        the :meth:`~pypeit.spectrographs.spectrograph.Spectrograph.configuration_keys`
+        or are used in :meth:`~pypeit.spectrographs.spectrograph.Spectrograph.config_specific_par`
+        for a particular spectrograph, if different from the name of the
+        PypeIt metadata keyword.
+
+        This list is used by :meth:`~pypeit.spectrographs.spectrograph.Spectrograph.subheader_for_spec`
+        to include additional FITS keywords in downstream output files.
+
+        Returns:
+            :obj:`list`: List of keywords from the raw data files that should
+            be propagated in output files.
+        """
+        return ['GRATING', 'ANGLE', 'APERTURE']
+
     def pypeit_file_keys(self):
         """
-        Define the list of keys to be output into a standard ``PypeIt`` file.
+        Define the list of keys to be output into a standard PypeIt file.
 
         Returns:
             :obj:`list`: The list of keywords in the relevant
@@ -147,6 +166,20 @@ class P200DBSPSpectrograph(spectrograph.Spectrograph):
         msgs.warn('Cannot determine if frames are of type {0}.'.format(ftype))
         return np.zeros(len(fitstbl), dtype=bool)
 
+    def get_rawimage(self, raw_file, det):
+        """
+        Read raw spectrograph image files and return data and relevant metadata
+        needed for image processing.
+
+        For P200/DBSP, the ``DATASEC`` and ``OSCANSEC`` regions are read
+        directly from the file header and are automatically adjusted to account
+        for the on-chip binning.  This is a simple wrapper for
+        :func:`pypeit.spectrographs.spectrograph.Spectrograph.get_rawimage` that
+        sets ``sec_includes_binning`` to True.  See the base-class function for
+        the detailed descriptions of the input parameters and returned objects.
+        """
+        return super().get_rawimage(raw_file, det, sec_includes_binning=True)
+
 
 class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
     """
@@ -154,6 +187,7 @@ class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
     """
     name = 'p200_dbsp_blue'
     camera = 'DBSPb'
+    header_name = 'DBSP_BLUE'
     supported = True
     comment = 'Blue camera'
     
@@ -180,24 +214,39 @@ class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
         if meta_key == 'binning':
             binspatial, binspec = headarr[0]['CCDSUM'].split(' ')
             return parse.binning2string(binspec, binspatial)
-        msgs.error("Not ready for this compound meta")
+        msgs.error(f"Not ready for this compound meta: {meta_key}")
 
-    def get_detector_par(self, hdu: fits.HDUList, det: int):
+    def get_detector_par(self, det: int, hdu: Optional[fits.HDUList] = None):
         """
         Return metadata for the selected detector.
 
+        .. warning::
+
+            Many of the necessary detector parameters are read from the file
+            header, meaning the ``hdu`` argument is effectively **required** for
+            P200/DBSPb.  The optional use of ``hdu`` is only viable for
+            automatically generated documentation.
+
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
             Object with the detector metadata.
         """
-        # Binning
-        binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
+        if hdu is None:
+            binning = '1,1'
+            datasec = None
+            oscansec = None
+        else:
+            # TODO: Could this be detector dependent??
+            binning = self.get_meta_value(self.get_headarr(hdu), 'binning')
+            datasec = np.atleast_1d(parse.flip_fits_slice(hdu[0].header['TSEC1']))
+            oscansec = np.atleast_1d(parse.flip_fits_slice(hdu[0].header['BSEC1']))
 
         # Detector 1
         detector_dict = dict(
@@ -208,22 +257,16 @@ class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
             specflip        = True,
             spatflip        = False, # check
             platescale      = 0.389,
-            darkcurr        = 0.0,
+            darkcurr        = 0.0,  # e-/pixel/hour
             saturation      = 65000.,
             nonlinear       = 62./65.,
             mincounts       = -1e10, # cross-check
             numamplifiers   = 1,
             gain            = np.atleast_1d(0.72),
-            ronoise         = np.atleast_1d(2.5)
+            ronoise         = np.atleast_1d(2.5),
+            datasec         = datasec,
+            oscansec        = oscansec
             )
-        
-        header = hdu[0].header
-        datasec = header['TSEC1']
-        oscansec = header['BSEC1']
-
-        detector_dict['datasec'] = np.atleast_1d(flip_fits_slice(datasec))
-        detector_dict['oscansec'] = np.atleast_1d(flip_fits_slice(oscansec))
-
         return detector_container.DetectorContainer(**detector_dict)
 
     @classmethod
@@ -233,10 +276,13 @@ class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
         
         Returns:
             :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
-            all of ``PypeIt`` methods.
+            all of PypeIt methods.
         """
         par = super().default_pypeit_par()
 
+        par['scienceframe']['process']['combine'] = 'median'
+        par['calibrations']['standardframe']['process']['combine'] = 'median'
+
         # Ignore PCA
         par['calibrations']['slitedges']['sync_predict'] = 'nearest'
         par['calibrations']['slitedges']['fit_min_spec_length'] = 0.55
@@ -249,7 +295,7 @@ class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
         par['calibrations']['pixelflatframe']['process']['combine'] = 'median'
         # Change the wavelength calibration method
         par['calibrations']['wavelengths']['method'] = 'full_template'
-        par['calibrations']['wavelengths']['lamps'] = ['FeI', 'FeII', 'ArI', 'ArII']
+        par['calibrations']['wavelengths']['lamps'] = ['FeI', 'ArI', 'ArII']
 
         #par['calibrations']['wavelengths']['nonlinear_counts'] = self.detector[0]['nonlinear'] * self.detector[0]['saturation']
         #par['calibrations']['wavelengths']['n_first'] = 3
@@ -258,7 +304,7 @@ class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
         # Do not flux calibrate
         par['fluxcalib'] = None
         # Set the default exposure time ranges for the frame typing
-        par['calibrations']['biasframe']['exprng'] = [None, 1]
+        par['calibrations']['biasframe']['exprng'] = [None, 0.001]
         par['calibrations']['darkframe']['exprng'] = [999999, None]     # No dark frames
         par['calibrations']['pinholeframe']['exprng'] = [999999, None]  # No pinhole frames
         par['calibrations']['arcframe']['exprng'] = [None, 120]
@@ -271,7 +317,7 @@ class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
 
     def config_specific_par(self, scifile, inp_par=None):
         """
-        Modify the ``PypeIt`` parameters to hard-wired values used for
+        Modify the PypeIt parameters to hard-wired values used for
         specific instrument configurations.
 
         Args:
@@ -289,15 +335,12 @@ class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
         # Start with instrument wide
         par = super().config_specific_par(scifile, inp_par=inp_par)
 
-        disp = self.get_meta_value(scifile, 'dispname')
-        if disp == '600/4000':
-            par['calibrations']['wavelengths']['reid_arxiv'] = 'p200_dbsp_blue_600_4000_d55.fits'
-        else:
-            msgs.error("Your grating " + disp + ' needs a template spectrum for the blue arm of DBSP.')
+        grating = self.get_meta_value(scifile, 'dispname')
+        dichroic = self.get_meta_value(scifile, 'dichroic')
 
         angle = Angle(self.get_meta_value(scifile, 'dispangle'), unit=u.deg).rad
         slitwidth = self.get_meta_value(scifile, 'slitwid') * u.arcsec
-        lines_mm = float(self.get_meta_value(scifile, 'dispname').split('/')[0]) / u.mm
+        lines_mm = float(grating.split('/')[0]) / u.mm
 
         theta_m = 38.5 * 2*np.pi / 360. - angle
         order = 2. if lines_mm == 158. * u.mm else 1.
@@ -311,6 +354,52 @@ class P200DBSPBlueSpectrograph(P200DBSPSpectrograph):
         resolving_power = cen_wv / dlam
 
         par['sensfunc']['UVIS']['resolution'] = resolving_power.decompose().value
+
+        cen_wv_AA = cen_wv.to(u.AA).value
+
+        high_res_reids = {
+            '1200/5000': {
+                'D55': {
+                    4700: 'p200_dbsp_blue_1200_5000_d55_4700.fits'
+                },
+                'D68': {
+                    6000: 'p200_dbsp_blue_1200_5000_d68_6000.fits'
+                }
+            }
+        }
+
+        if grating.split("/")[0] == "1200":
+            # high resolution grating!
+            # here we need to select the reid_arxiv that most closely matches the central wavelength
+            # and emit a warning if the difference is too great / the wavelength overlap is too small
+            try:
+                reids = high_res_reids[grating][dichroic]
+                cen_wvs = np.array(list(reids))
+                best_wv = cen_wvs[np.argmin(np.abs(cen_wvs - cen_wv_AA))]
+
+                # blue wavelength coverage with a 1200 lines/mm grating is about 1550 A
+                diff = np.abs(best_wv - cen_wv_AA)
+                if diff > 775:
+                    msgs.warn("Closest matching archived wavelength solutions"
+                        f"differs in central wavelength by {diff:4.0f} A. The"
+                        "wavelength solution may be unreliable. If wavelength"
+                        "calibration fails, try using the holy grail method by"
+                        "adding the following to your PypeIt file:\n"
+                        "[calibrations]\n"
+                        "\t[[wavelengths]]\n"
+                        "\t\tmethod = holy-grail")
+                par['calibrations']['wavelengths']['reid_arxiv'] = reids[best_wv]
+            except KeyError:
+                msgs.warn("Your grating " + grating + " doesn't have a template spectrum for the blue arm of DBSP.")
+        else:
+            if grating == '600/4000' and dichroic == 'D55':
+                par['calibrations']['wavelengths']['reid_arxiv'] = 'p200_dbsp_blue_600_4000_d55.fits'
+            elif grating == '600/4000' and dichroic == 'D68':
+                par['calibrations']['wavelengths']['reid_arxiv'] = 'p200_dbsp_blue_600_4000_d68.fits'
+            elif grating == '300/3990' and dichroic == 'D55':
+                par['calibrations']['wavelengths']['reid_arxiv'] = 'p200_dbsp_blue_300_3990_d55.fits'
+            else:
+                msgs.warn("Your grating " + grating + " doesn't have a template spectrum for the blue arm of DBSP.")
         
         return par
 
@@ -321,6 +410,7 @@ class P200DBSPRedSpectrograph(P200DBSPSpectrograph):
     """
     name = 'p200_dbsp_red'
     camera = 'DBSPr'
+    header_name = 'DBSP_RED2'
     supported = True
     comment = 'Red camera'
     
@@ -348,24 +438,39 @@ class P200DBSPRedSpectrograph(P200DBSPSpectrograph):
             binspec, binspatial = headarr[0]['CCDSUM'].split(' ')
             return parse.binning2string(binspec, binspatial)
         else:
-            msgs.error("Not ready for this compound meta")
+            msgs.error(f"Not ready for this compound meta: {meta_key}")
 
-    def get_detector_par(self, hdu: fits.HDUList, det: int):
+    def get_detector_par(self, det: int, hdu: Optional[fits.HDUList] = None):
         """
         Return metadata for the selected detector.
 
+        .. warning::
+
+            Many of the necessary detector parameters are read from the file
+            header, meaning the ``hdu`` argument is effectively **required** for
+            P200/DBSPr.  The optional use of ``hdu`` is only viable for
+            automatically generated documentation.
+
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
             Object with the detector metadata.
         """
-        # Binning
-        binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
+        if hdu is None:
+            binning = '1,1'
+            datasec = None
+            oscansec = None
+        else:
+            # TODO: Could this be detector dependent??
+            binning = self.get_meta_value(self.get_headarr(hdu), 'binning')
+            datasec = np.atleast_1d(parse.flip_fits_slice(hdu[0].header['TSEC1']))
+            oscansec = np.atleast_1d(parse.flip_fits_slice(hdu[0].header['BSEC1']))
 
         # Detector 1
         detector_dict = dict(
@@ -376,23 +481,16 @@ class P200DBSPRedSpectrograph(P200DBSPSpectrograph):
             specflip        = False,
             spatflip        = False, # check
             platescale      = 0.293,
-            darkcurr        = 0.0,
+            darkcurr        = 0.0,  # e-/pixel/hour
             saturation      = 45000.,
             nonlinear       = 40./45.,
             mincounts       = -1e10, # check
             numamplifiers   = 1,
             gain            = np.atleast_1d(2.8),
-            ronoise         = np.atleast_1d(8.5)
+            ronoise         = np.atleast_1d(8.5),
+            datasec         = datasec,
+            oscansec        = oscansec
         )
-
-        header = hdu[0].header
-
-        datasec = header['TSEC1']
-        oscansec = header['BSEC1']
-
-        detector_dict['datasec'] = np.atleast_1d(flip_fits_slice(datasec))
-        detector_dict['oscansec'] = np.atleast_1d(flip_fits_slice(oscansec))
-
         return detector_container.DetectorContainer(**detector_dict)
 
     @classmethod
@@ -402,13 +500,16 @@ class P200DBSPRedSpectrograph(P200DBSPSpectrograph):
         
         Returns:
             :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
-            all of ``PypeIt`` methods.
+            all of PypeIt methods.
         """
         par = super().default_pypeit_par()
 
         # Ignore PCA
         par['calibrations']['slitedges']['sync_predict'] = 'nearest'
 
+        par['scienceframe']['process']['combine'] = 'median'
+        par['calibrations']['standardframe']['process']['combine'] = 'median'
+
         par['scienceframe']['process']['use_overscan'] = True
         par['scienceframe']['process']['sigclip'] = 4.0 # Tweaked downward from 4.5. 
         par['scienceframe']['process']['objlim'] = 1.5 # Tweaked downward from 3.0. Same value as Keck KCWI and DEIMOS
@@ -426,7 +527,7 @@ class P200DBSPRedSpectrograph(P200DBSPSpectrograph):
         # Do not flux calibrate
         par['fluxcalib'] = None
         # Set the default exposure time ranges for the frame typing
-        par['calibrations']['biasframe']['exprng'] = [None, 1]
+        par['calibrations']['biasframe']['exprng'] = [None, 0.001]
         par['calibrations']['darkframe']['exprng'] = [999999, None]     # No dark frames
         par['calibrations']['pinholeframe']['exprng'] = [999999, None]  # No pinhole frames
         par['calibrations']['arcframe']['exprng'] = [None, 120]
@@ -435,12 +536,12 @@ class P200DBSPRedSpectrograph(P200DBSPSpectrograph):
 
         par['sensfunc']['algorithm'] = 'UVIS'
         par['sensfunc']['UVIS']['polycorrect'] = False
-        par['sensfunc']['IR']['telgridfile'] = resource_filename('pypeit', '/data/telluric/TelFit_Lick_3100_11100_R10000.fits')
+        par['sensfunc']['IR']['telgridfile'] = 'TellPCA_3000_26000_R10000.fits'
         return par
 
     def config_specific_par(self, scifile, inp_par=None):
         """
-        Modify the ``PypeIt`` parameters to hard-wired values used for
+        Modify the PypeIt parameters to hard-wired values used for
         specific instrument configurations.
 
         Args:
@@ -458,16 +559,12 @@ class P200DBSPRedSpectrograph(P200DBSPSpectrograph):
         # Start with instrument wide
         par = super().config_specific_par(scifile, inp_par=inp_par)
 
-        disp = self.get_meta_value(scifile, 'dispname')
-        if disp == '316/7500':
-            par['calibrations']['wavelengths']['reid_arxiv'] = 'p200_dbsp_red_316_7500_d55.fits'
-        else:
-            msgs.error("Your grating " + disp + ' needs a template spectrum for the red arm of DBSP.')
-
+        grating = self.get_meta_value(scifile, 'dispname')
+        dichroic = self.get_meta_value(scifile, 'dichroic')
 
         angle = Angle(self.get_meta_value(scifile, 'dispangle'), unit=u.deg).rad
         slitwidth = self.get_meta_value(scifile, 'slitwid') * u.arcsec
-        lines_mm = float(self.get_meta_value(scifile, 'dispname').split('/')[0]) / u.mm
+        lines_mm = float(grating.split('/')[0]) / u.mm
 
         theta_m = 35.0 * 2*np.pi / 360. - angle
         order = 1.
@@ -482,6 +579,91 @@ class P200DBSPRedSpectrograph(P200DBSPSpectrograph):
 
         par['sensfunc']['UVIS']['resolution'] = resolving_power.decompose().value
 
+        cen_wv_AA = cen_wv.to(u.AA).value
+
+        high_res_reids = {
+            '1200/7100': {
+                'D68': {
+                    7600: 'p200_dbsp_red_1200_7100_d68.fits',
+                    8200: 'p200_dbsp_red_1200_7100_d68.fits'
+                },
+                'D55': {
+                    6680: 'p200_dbsp_red_1200_7100_d55_6680.fits'
+                }
+            },
+            '1200/9400': {
+                'D55': {
+                    8800: 'p200_dbsp_red_1200_9400_d55_8800.fits'
+                }
+            }
+        }
+
+        if grating.split("/")[0] == "1200":
+            # high resolution grating!
+            # here we need to select the reid_arxiv that most closely matches the central wavelength
+            # and emit a warning if the difference is too great / the wavelength overlap is too small
+            try:
+                reids = high_res_reids[grating][dichroic]
+                cen_wvs = np.array(list(reids))
+                best_wv = cen_wvs[np.argmin(np.abs(cen_wvs - cen_wv_AA))]
+
+                # red wavelength coverage with a 1200 lines/mm grating is about 1600 A
+                diff = np.abs(best_wv - cen_wv_AA)
+                if diff > 800:
+                    msgs.warn("Closest matching archived wavelength solutions"
+                        f"differs in central wavelength by {diff:4.0f} A. The"
+                        "wavelength solution may be unreliable. If wavelength"
+                        "calibration fails, try using the holy grail method by"
+                        "adding the following to your PypeIt file:\n"
+                        "[calibrations]\n"
+                        "\t[[wavelengths]]\n"
+                        "\t\tmethod = holy-grail")
+                par['calibrations']['wavelengths']['reid_arxiv'] = reids[best_wv]
+            except KeyError:
+                msgs.warn("Your grating " + grating + " doesn't have a template spectrum for the red arm of DBSP.")
+        else:
+            if grating == '316/7500' and dichroic == 'D55':
+                par['calibrations']['wavelengths']['reid_arxiv'] = 'p200_dbsp_red_316_7500_d55.fits'
+            elif grating == '600/10000' and dichroic == 'D55':
+                par['calibrations']['wavelengths']['reid_arxiv'] = 'p200_dbsp_red_600_10000_d55.fits'
+            else:
+                msgs.warn("Your grating " + grating + " doesn't have a template spectrum for the red arm of DBSP.")
+
         return par
 
+    def bpm(self, filename, det, shape=None, msbias=None):
+        """
+        Override parent bpm function with BPM specific to P200 DBSPr.
+
+        Parameters
+        ----------
+        det : int
+            Detector number
+        msbias : numpy.ndarray
+            Processed bias frame used when constructing the bpm (see :func:`bpm_frombias`)
+
+        Returns
+        -------
+        bpix : ndarray
+          0 = ok; 1 = Mask
+        """
+        msgs.info("Custom bad pixel mask for DBSPr")
+        bpm_img = self.empty_bpm(filename, det, shape=shape)
+
+        # Fill in bad pixels if a processed bias frame is provided
+        if msbias is not None:
+            return self.bpm_frombias(msbias, bpm_img)
+
+        # Red CCD detector defect is present in data taken 2020-05-22
+        # and absent in data taken 2020-04-21
+        DEFECT_DATE = Time('2020-05-21')
+        # TODO: Model the growth of the detector defect with time.
+        # TODO: Get more precise date range for detector.
+        with io.fits_open(filename) as hdul:
+            if Time(hdul[0].header['UTSHUT']) > DEFECT_DATE:
+                spec_binning = int(self.get_meta_value([hdul[0].header], 'binning').split(',')[0])
+                bpm_img[464 // spec_binning : 723 // spec_binning, :] = 1
+
+        return bpm_img
+
 
