@@ -2,17 +2,21 @@
 Module to define the SlitMask class
 
 .. include common links, assuming primary doc root is up one directory
-.. include:: ../links.rst
+.. include:: ../include/links.rst
 """
 from collections import OrderedDict
 import warnings
 import numpy
 from scipy import optimize, fftpack, signal
 from matplotlib import pyplot
-from astropy.stats import sigma_clip, sigma_clipped_stats
+
+from astropy.stats import sigma_clip
 
 from pypeit.bitmask import BitMask
 from pypeit.utils import index_of_x_eq_y
+from pypeit import io
+
+from IPython import embed
 
 class SlitMaskBitMask(BitMask):
     """
@@ -84,9 +88,11 @@ class SlitMask:
             position angle of the slit from N through E in degrees.
         objects (`numpy.ndarray`_, optional):
             List of objects observed as a 1D or 2D array with shape
-            :math:`(4,)` or :math:`(N_{\rm obj},4)`. The four
+            :math:`(9,)` or :math:`(N_{\rm obj},9)`. The nine
             elements for each object is the slit id, the object ID,
-            and the right ascension and declination of the target.
+            the right ascension and declination of the target, the
+            object name, the object magnitude and band, and the
+            object top and bottom distances from the slit edges.
             The order of the objects does not have to match that of
             the slit IDs. Also, there can be slits without objects
             and slits with multiple objects; however, objects cannot
@@ -96,7 +102,7 @@ class SlitMask:
     Attributes:
         corners (`numpy.ndarray`_):
             See above.
-        id (`numpy.ndarray`_):
+        slitid (`numpy.ndarray`_):
             See `slitid` above.
         mask (`numpy.ndarray`_):
             Mask bits selecting the type of slit.
@@ -125,6 +131,14 @@ class SlitMask:
             The slit width.
         pa (`numpy.ndarray`_):
             The cartesian rotation angle of the slit in degrees.
+        mask_radec (:obj:`tuple`, optional):
+            RA, Dec (deg) of the pointing of the mask (approximate center)
+        posx_pa (:obj:`float`):
+            Sky PA that points to positive x (spatial) on the detector
+        negx_pa (:obj:`float`):
+            Sky PA that points to negative x (spatial) on the detector
+        object_names (`numpy.ndarray`_):
+            Object names
 
     Raises:
         ValueError:
@@ -133,7 +147,17 @@ class SlitMask:
             slits provided.
     """
     bitmask = SlitMaskBitMask()
-    def __init__(self, corners, slitid=None, align=None, science=None, onsky=None, objects=None):
+    def __init__(self, corners, slitid=None, align=None, science=None, onsky=None, objects=None,
+                 posx_pa=None, object_names=None, mask_radec=None):
+
+        # PA
+        if posx_pa is not None:
+            self.posx_pa, self.negx_pa = positive_pa(posx_pa)
+        else:
+            self.posx_pa, self.negx_pa = None, None
+
+        self.mask_radec = mask_radec
+        self.object_names=object_names
 
         # TODO: Allow random input order and then fix
 
@@ -188,8 +212,9 @@ class SlitMask:
         self.slitindx = None
         if objects is not None:
             self.objects = numpy.atleast_2d(objects)
-            if self.objects.shape[1] != 4:
-                raise ValueError('Must provide the slit ID and sky coordinates for each object.')
+            if self.objects.shape[1] != 9:
+                raise ValueError('Must provide the slit ID, object ID, sky coordinates, object name, '
+                                 'object magnitude and band, top and bottom distance for each object.')
             try:
                 self.slitindx = index_of_x_eq_y(self.slitid, self.objects[:,0].astype(int),
                                                 strict=True)
@@ -217,6 +242,7 @@ class SlitMask:
         self.pa[self.pa < -90] += 180
         self.pa[self.pa > 90] -= 180
 
+
     def __repr__(self):
         return '<{0}: nslits={1}>'.format(self.__class__.__name__, self.nslits)
 
@@ -228,20 +254,20 @@ class SlitMask:
     @property
     def alignment_slit(self):
         """Boolean array selecting the alignment slits."""
-        return self.bitmask.flagged(self.mask, 'ALIGN')
+        return self.bitmask.flagged(self.mask, flag='ALIGN')
 
     def is_alignment(self, i):
         """Check if specific slit is an alignment slit."""
-        return self.bitmask.flagged(self.mask[i], 'ALIGN')
+        return self.bitmask.flagged(self.mask[i], flag='ALIGN')
 
     @property
     def science_slit(self):
         """Boolean array selecting the slits with science targets."""
-        return self.bitmask.flagged(self.mask, 'SCIENCE')
+        return self.bitmask.flagged(self.mask, flag='SCIENCE')
 
     def is_science(self, i):
         """Check if specific slit should have a science target."""
-        return self.bitmask.flagged(self.mask[i], 'SCIENCE')
+        return self.bitmask.flagged(self.mask[i], flag='SCIENCE')
 
 
 class SlitRegister:
@@ -884,4 +910,136 @@ def build_slit_function(edges, size=None, oversample=1, sigma=None):
     return offset, slit_func_x, slit_func_y
 
 
+def positive_pa(pa:float):
+    """ Modify input pa to be positive (0-360)
+
+    Args:
+        pa (float): [description]
+
+    Returns:
+        [type]: [description]
+    """
+    # Require it be positive
+    if pa < 0.:
+        pa += 360.
+    # Now the complement -- also require it be positive
+    comp_pa = pa - 180. if pa > 180. else pa + 180.
+    # Return
+    return pa, comp_pa
+
+
+def correct_slitpa(slitpa, maskpa):
+    """ Flip 180 degree the slit PA if the value recorded
+    in the slitmask design is more than +/-90 degree from the slitmask PA.
+
+    Args:
+        slitpa (:obj:`float` or `numpy.ndarray`_): position angle of the slits.
+        maskpa: (:obj:`float`): position angle of the slitmask.
+
+    Returns:
+        :obj:`float` or `numpy.ndarray`_: flipped slitpa, if it is more than +/-90 from the maskpa,
+        otherwise unchanged slitpa.
+
+    """
+    # Insure maskpa is positive
+    maskpa, _ = positive_pa(maskpa)
+
+    # create a new array of slitpa
+    newslitpa = numpy.atleast_1d(slitpa).copy()
+
+    for i in range(slitpa.size):
+        # make slitpa positive (if it's not)
+        pa, _ = positive_pa(slitpa[i])
+        # flip slitpas that are >90 and <270 degrees from the maskpa
+        if 90 < (pa - maskpa) < 270:
+            newslitpa[i] -= 180
+        # flip slitpas that are <-90 and >-270 degrees from the maskpa
+        elif -90 > (pa - maskpa) > -270:
+            newslitpa[i] += 180
+        # check for value >= 270
+        if newslitpa[i] >= 270:
+            newslitpa[i] -= 360
+        # check for value <= -270
+        elif newslitpa[i] <= -270:
+            newslitpa[i] += 360
+    return newslitpa[0] if newslitpa.size == 1 else newslitpa
+
+
+def load_keck_deimoslris(filename:str, instr:str):
+    """ Load up the mask design info from the header
+    of the file provided
+
+    Args:
+        filename (str): 
+        instr (str): Name of spectrograph
+            Allowed are keck_lris_xxx, keck_deimos
+
+    Returns:
+        [type]: [description]
+    """
+    # Open the file
+    hdu = io.fits_open(filename)
+
+    # Build the object data
+    #   - Find the index of the object IDs in the slit-object
+    #     mapping that match the object catalog
+    mapid = hdu['SlitObjMap'].data['ObjectID']
+    catid = hdu['ObjectCat'].data['ObjectID']
+    indx = index_of_x_eq_y(mapid, catid)
+    # .decode() assumes encoding is the default 'utf-8'
+    objname = [item.strip().decode() if isinstance(item, bytes) else item.strip()
+               for item in hdu['ObjectCat'].data['OBJECT']]
+    # check if each objname is an ascii str (if not BinTableHDU later in the reduction will fail)
+    objname = [name if name.isascii() else name.encode('ascii', 'ignore').decode() for name in objname]
+    #   - Pull out the slit ID, object ID, name, object coordinates, top and bottom distance
+    objects = numpy.array([hdu['SlitObjMap'].data['dSlitId'][indx].astype(int),
+                        catid.astype(int),
+                        hdu['ObjectCat'].data['RA_OBJ'],
+                        hdu['ObjectCat'].data['DEC_OBJ'],
+                        objname,
+                        hdu['ObjectCat'].data['mag'],
+                        hdu['ObjectCat'].data['pBand'],
+                        hdu['SlitObjMap'].data['TopDist'][indx],
+                        hdu['SlitObjMap'].data['BotDist'][indx]]).T
+    #   - Only keep the objects that are in the slit-object mapping
+    objects = objects[mapid[indx] == catid]
+
+    # Match the slit IDs in DesiSlits to those in BluSlits
+    indx = index_of_x_eq_y(hdu['DesiSlits'].data['dSlitId'], 
+                            hdu['BluSlits'].data['dSlitId'],
+                            strict=True)
+
+    # PA corresponding to positive x on detector (spatial)
+    posx_pa = hdu['MaskDesign'].data['PA_PNT'][-1]
+    # Insure it is positive
+    posx_pa, _ = positive_pa(posx_pa)
+
+    # flip 180 degree slit PAs if are > +/-90 degrees from maskpa (posx_pa)
+    slit_pas = correct_slitpa(hdu['DesiSlits'].data['slitLPA'][indx], posx_pa)
+
+    # Instantiate the slit mask object and return it
+    try:
+        hdu['BluSlits'].data['slitX0']
+        indices = numpy.arange(4)
+    except KeyError:
+        indices = numpy.arange(4)+1
+    #indices = numpy.arange(4) if instr == 'keck_deimos' else numpy.arange(4)+1 
+    slit_list = []
+    for index in indices:
+        for cdim in ['X', 'Y']:
+            slit_list.append(hdu['BluSlits'].data[f'slit{cdim}{index}'])
+
+    return SlitMask(numpy.array(slit_list).T.reshape(-1,4,2),
+                    slitid=hdu['BluSlits'].data['dSlitId'],
+                    align=hdu['DesiSlits'].data['slitTyp'][indx] == 'A',
+                    science=hdu['DesiSlits'].data['slitTyp'][indx] == 'P',
+                    onsky=numpy.array([hdu['DesiSlits'].data['slitRA'][indx],
+                                       hdu['DesiSlits'].data['slitDec'][indx],
+                                       hdu['DesiSlits'].data['slitLen'][indx],
+                                       hdu['DesiSlits'].data['slitWid'][indx],
+                                       slit_pas]).T,
+                    objects=objects,
+                    mask_radec=(hdu['MaskDesign'].data['RA_PNT'][0], 
+                                hdu['MaskDesign'].data['DEC_PNT'][0]),
+                    posx_pa=posx_pa)
 
