@@ -3,8 +3,6 @@ Module for VLT X-Shooter
 
 .. include:: ../include/links.rst
 """
-import os
-
 import numpy as np
 
 from astropy.coordinates import SkyCoord
@@ -17,7 +15,7 @@ from pypeit.core import parse
 from pypeit.core import framematch
 from pypeit.spectrographs import spectrograph
 from pypeit.images import detector_container
-from pypeit import data
+from pypeit import dataPaths
 
 from IPython import embed
 
@@ -36,7 +34,7 @@ class VLTXShooterSpectrograph(spectrograph.Spectrograph):
         """
         Define how metadata are derived from the spectrograph files.
 
-        That is, this associates the ``PypeIt``-specific metadata keywords
+        That is, this associates the PypeIt-specific metadata keywords
         with the instrument-specific header cards using :attr:`meta`.
         """
         self.meta = {}
@@ -85,6 +83,26 @@ class VLTXShooterSpectrograph(spectrograph.Spectrograph):
             return parse.binning2string(binspec, binspatial)
         msgs.error("Not ready for this compound meta")
 
+    def config_independent_frames(self):
+        """
+        Define frame types that are independent of the fully defined
+        instrument configuration.
+
+        This method returns a dictionary where the keys of the dictionary are
+        the list of configuration-independent frame types. The value of each
+        dictionary element can be set to one or more metadata keys that can
+        be used to assign each frame type to a given configuration group. See
+        :func:`~pypeit.metadata.PypeItMetaData.set_configurations` and how it
+        interprets the dictionary values, which can be None.
+
+        Returns:
+            :obj:`dict`: Dictionary where the keys are the frame types that
+            are configuration-independent and the values are the metadata
+            keywords that can be used to assign the frames to a configuration
+            group.
+        """
+        return {}
+
     def configuration_keys(self):
         """
         Return the metadata keys that define a unique instrument
@@ -96,14 +114,34 @@ class VLTXShooterSpectrograph(spectrograph.Spectrograph):
 
         Returns:
             :obj:`list`: List of keywords of data pulled from file headers
-            and used to constuct the :class:`~pypeit.metadata.PypeItMetaData`
+            and used to construct the :class:`~pypeit.metadata.PypeItMetaData`
             object.
         """
         return ['arm']
 
+    def raw_header_cards(self):
+        """
+        Return additional raw header cards to be propagated in
+        downstream output files for configuration identification.
+
+        The list of raw data FITS keywords should be those used to populate
+        the :meth:`~pypeit.spectrographs.spectrograph.Spectrograph.configuration_keys`
+        or are used in :meth:`~pypeit.spectrographs.spectrograph.Spectrograph.config_specific_par`
+        for a particular spectrograph, if different from the name of the
+        PypeIt metadata keyword.
+
+        This list is used by :meth:`~pypeit.spectrographs.spectrograph.Spectrograph.subheader_for_spec`
+        to include additional FITS keywords in downstream output files.
+
+        Returns:
+            :obj:`list`: List of keywords from the raw data files that should
+            be propagated in output files.
+        """
+        return ['HIERARCH ESO SEQ ARM']
+
     def pypeit_file_keys(self):
         """
-        Define the list of keys to be output into a standard ``PypeIt`` file.
+        Define the list of keys to be output into a standard PypeIt file.
 
         Returns:
             :obj:`list`: The list of keywords in the relevant
@@ -184,6 +222,7 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
             :class:`~pypeit.images.detector_container.DetectorContainer`:
             Object with the detector metadata.
         """
+        
         # Detector 1
         detector_dict = dict(
             binning         = '1,1',  # No binning in near-IR
@@ -192,21 +231,15 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
             specaxis        = 1,
             specflip        = False,
             spatflip        = False,
-            platescale      = 0.197, # average between order 11 & 30, see manual
-            darkcurr        = 0.0,
+            platescale      = 0.245, # average across all orders, see manual
+            darkcurr        = 72.0,  # e-/pixel/hour
             saturation      = 2.0e5, # I think saturation may never be a problem here since there are many DITs
             nonlinear       = 0.86,
             mincounts       = -1e10,
             numamplifiers   = 1,
-            gain            = np.atleast_1d(2.12), #
-            ronoise         = np.atleast_1d(8.0), # ?? more precise value? #TODO the read noise is exposure time  dependent and should be grabbed from header
+            gain            = np.atleast_1d(2.29),
+            ronoise         = np.atleast_1d(8.0), # Read noise depends on exposure time (see XShooter manual) but this is the typical value.
             datasec         = np.atleast_1d('[4:2044,4:]'), # These are all unbinned pixels
-            # EMA: No real overscan for XSHOOTER-NIR:
-            # See Table 6 in http://www.eso.org/sci/facilities/paranal/instruments/xshooter/doc/VLT-MAN-ESO-14650-4942_P103v1.pdf
-            # The overscan region below contains only zeros
-            # ToDo should we just set it as empty?
-            #  JXP says yes
-            #oscansec        = np.atleast_1d('[4:2044,1:3]'), # These are all unbinned pixels.
             )
         return detector_container.DetectorContainer(**detector_dict)
 
@@ -217,7 +250,7 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         
         Returns:
             :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
-            all of ``PypeIt`` methods.
+            all of PypeIt methods.
         """
         par = super().default_pypeit_par()
 
@@ -225,19 +258,12 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         turn_off = dict(use_illumflat=False, use_biasimage=False, use_overscan=False,
                         use_darkimage=False)
         par.reset_all_processimages_par(**turn_off)
-        # Require dark images to be subtracted from the flat images used for
-        # tracing, pixelflats, and illumflats
-        # par['calibrations']['traceframe']['process']['use_darkimage'] = True
-        # par['calibrations']['pixelflatframe']['process']['use_darkimage'] = True
-        # par['calibrations']['illumflatframe']['process']['use_darkimage'] = True
-        # TODO: `mask_cr` now defaults to True for darks.  Should this be turned off?
 
         # Is this needed below?
         par['scienceframe']['process']['sigclip'] = 20.0
         par['scienceframe']['process']['satpix'] = 'nothing'
         # TODO tune up LA COSMICS parameters here for X-shooter as tellurics are being excessively masked
 
-
         # Adjustments to slit and tilts for NIR
         par['calibrations']['slitedges']['edge_thresh'] = 50.
         par['calibrations']['slitedges']['fit_order'] = 8
@@ -257,24 +283,26 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
 
         # 1D wavelength solution
         par['calibrations']['wavelengths']['lamps'] = ['OH_XSHOOTER']
-        par['calibrations']['wavelengths']['rms_threshold'] = 0.25
+        par['calibrations']['wavelengths']['rms_thresh_frac_fwhm'] = 0.15
         par['calibrations']['wavelengths']['sigdetect'] = 10.0
-        par['calibrations']['wavelengths']['fwhm'] = 5.0
+        par['calibrations']['wavelengths']['fwhm'] = 4.
         par['calibrations']['wavelengths']['n_final'] = 4
         # Reidentification parameters
         par['calibrations']['wavelengths']['method'] = 'reidentify'
         par['calibrations']['wavelengths']['reid_arxiv'] = 'vlt_xshooter_nir.fits'
         par['calibrations']['wavelengths']['cc_thresh'] = 0.50
         par['calibrations']['wavelengths']['cc_local_thresh'] = 0.50
-#        par['calibrations']['wavelengths']['ech_fix_format'] = True
         # Echelle parameters
         par['calibrations']['wavelengths']['echelle'] = True
         par['calibrations']['wavelengths']['ech_nspec_coeff'] = 5
         par['calibrations']['wavelengths']['ech_norder_coeff'] = 5
         par['calibrations']['wavelengths']['ech_sigrej'] = 3.0
+        par['calibrations']['wavelengths']['qa_log'] = False
+        # Measured FWHM is correct, but resulting wavelength solution is poor.
+        # This should be explored further, but for now, turning off fwhm_fromlines helps.
+        par['calibrations']['wavelengths']['fwhm_fromlines'] = False
 
         # Flats
-        #par['calibrations']['standardframe']['process']['illumflatten'] = False
         par['calibrations']['flatfield']['tweak_slits_thresh'] = 0.90
         par['calibrations']['flatfield']['tweak_slits_maxfrac'] = 0.10
 
@@ -285,27 +313,23 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         par['reduce']['skysub']['bspline_spacing'] = 0.8
         par['reduce']['skysub']['global_sky_std']  = False # Do not perform global sky subtraction for standard stars
         par['reduce']['extraction']['model_full_slit'] = True  # local sky subtraction operates on entire slit
-        par['reduce']['findobj']['trace_npoly'] = 8
+        par['reduce']['findobj']['trace_npoly'] = 10
         par['reduce']['findobj']['maxnumber_sci'] = 2  # Assume that there is only one object on the slit.
         par['reduce']['findobj']['maxnumber_std'] = 1  # Assume that there is only one object on the slit.
 
-
-        # The settings below enable X-shooter dark subtraction from the traceframe and pixelflatframe, but enforce
-        # that this bias won't be subtracted from other images. It is a hack for now, because eventually we want to
-        # perform this operation with the dark frame class, and we want to attach individual sets of darks to specific
-        # images.
-        #par['calibrations']['biasframe']['useframe'] = 'bias'
-        #par['calibrations']['traceframe']['process']['bias'] = 'force'
-        #par['calibrations']['pixelflatframe']['process']['bias'] = 'force'
-        #par['calibrations']['arcframe']['process']['bias'] = 'skip'
-        #par['calibrations']['tiltframe']['process']['bias'] = 'skip'
-        #par['calibrations']['standardframe']['process']['bias'] = 'skip'
-        #par['scienceframe']['process']['bias'] = 'skip'
-
         # Sensitivity function parameters
         par['sensfunc']['algorithm'] = 'IR'
         par['sensfunc']['polyorder'] = 8
-        par['sensfunc']['IR']['telgridfile'] = 'TelFit_Paranal_NIR_9800_25000_R25000.fits'
+        par['sensfunc']['IR']['telgridfile'] = 'TellPCA_3000_26000_R25000.fits'
+        par['sensfunc']['IR']['pix_shift_bounds'] = (-10.0,10.0)
+        
+        # Telluric parameters
+        par['telluric']['pix_shift_bounds'] = (-10.0,10.0)
+        par['telluric']['resln_frac_bounds'] = (0.4,2.0)
+
+        # Coadding
+        par['coadd1d']['wave_method'] = 'log10'
+
 
         return par
 
@@ -314,7 +338,7 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         """
         Define how metadata are derived from the spectrograph files.
 
-        That is, this associates the ``PypeIt``-specific metadata keywords
+        That is, this associates the PypeIt-specific metadata keywords
         with the instrument-specific header cards using :attr:`meta`.
         """
         super().init_meta()
@@ -329,7 +353,7 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
 
     def pypeit_file_keys(self):
         """
-        Define the list of keys to be output into a standard ``PypeIt`` file.
+        Define the list of keys to be output into a standard PypeIt file.
 
         Returns:
             :obj:`list`: The list of keywords in the relevant
@@ -339,7 +363,7 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         pypeit_keys = super().pypeit_file_keys()
         # TODO: Why are these added here? See
         # pypeit.metadata.PypeItMetaData.set_pypeit_cols
-        pypeit_keys += ['calib', 'comb_id', 'bkg_id']
+        pypeit_keys += ['comb_id', 'bkg_id']
         return pypeit_keys
 
     def check_frame_type(self, ftype, fitstbl, exprng=None):
@@ -389,7 +413,7 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
                                | (fitstbl['target'] == 'LAMP,FLAT'))
                                & good_flat_seq)
         
-        if ftype in ['dark']:
+        if ftype in ['lampoffflats']:
             # Lamp off flats are taken second (even exposure number)
             return good_exp & (((fitstbl['target'] == 'LAMP,DFLAT')
                                 | (fitstbl['target'] == 'LAMP,QFLAT')
@@ -425,7 +449,7 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
                 Required if filename is None
                 Ignored if filename is not None
             msbias (`numpy.ndarray`_, optional):
-                Master bias frame used to identify bad pixels
+                Processed bias frame used to identify bad pixels
 
         Returns:
             `numpy.ndarray`_: An integer array with a masked value set
@@ -436,32 +460,10 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         bpm_img = super().bpm(filename, det, shape=shape, msbias=msbias)
 
         if det == 1:
-            bpm_dir = os.path.join(data.Paths.static_calibs, 'vlt_xshoooter')
-            try :
-                bpm_loc = np.loadtxt(os.path.join(bpm_dir, 'BP_MAP_RP_NIR.dat'),
-                                     usecols=(0,1))
-            except IOError :
-                msgs.warn('BP_MAP_RP_NIR.dat not present in the static database')
-                bpm_fits = io.fits_open(os.path.join(bpm_dir, 'BP_MAP_RP_NIR.fits.gz'))
-                # ToDo: this depends on datasec, biassec, specflip, and specaxis
-                #       and should become able to adapt to these parameters.
-                # Flipping and shifting BPM to match the PypeIt format
-                y_shift = -2
-                x_shift = 18
-                bpm_data = np.flipud(bpm_fits[0].data)
-                y_len = len(bpm_data[:,0])
-                x_len = len(bpm_data[0,:])
-                bpm_data_pypeit = np.full( ((y_len+abs(y_shift)),(x_len+abs(x_shift))) , 0)
-                bpm_data_pypeit[:-abs(y_shift),:-abs(x_shift)] = bpm_data_pypeit[:-abs(y_shift),:-abs(x_shift)] + bpm_data
-                bpm_data_pypeit = np.roll(bpm_data_pypeit,-y_shift,axis=0)
-                bpm_data_pypeit = np.roll(bpm_data_pypeit,x_shift,axis=1)
-                filt_bpm = bpm_data_pypeit[1:y_len,1:x_len]>100.
-                y_bpm, x_bpm = np.where(filt_bpm)
-                bpm_loc = np.array([y_bpm,x_bpm]).T
-                np.savetxt(os.path.join(bpm_dir, 'BP_MAP_RP_NIR.dat'), bpm_loc,
-                           fmt=['%d','%d'])
-            finally :
-                bpm_img[bpm_loc[:,0].astype(int),bpm_loc[:,1].astype(int)] = 1.
+            # Creates another PypeItDataPath object
+            vlt_sc = dataPaths.static_calibs / 'vlt_xshoooter'
+            bpm_loc = np.loadtxt(vlt_sc.get_file_path('BP_MAP_RP_NIR.dat'), usecols=(0,1))
+            bpm_img[bpm_loc[:,0].astype(int),bpm_loc[:,1].astype(int)] = 1.
 
         return bpm_img
 
@@ -496,8 +498,8 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         Return the minimum and maximum spectral pixel expected for the
         spectral range of each order.
         """
-        spec_max = np.asarray([1467,1502,1540, 1580,1620,1665,1720, 1770,1825,1895, 1966, 2000,2000,2000,2000,2000])
-        spec_min = np.asarray([420 ,390 , 370,  345, 315, 285, 248,  210, 165, 115,   63,   10,   0,   0,   0,   0])
+        spec_max = np.asarray([1477,1513,1547, 1588,1628,1682,1733,1795,1855,1930,2005,2040,2040,2040,2040,2040])
+        spec_min = np.asarray([420 ,390 , 370,  345, 315, 285, 248, 210, 165, 115,  58,   5,   0,   0,   0,   0])
         return np.vstack((spec_min, spec_max))
 
     def order_platescale(self, order_vec, binning=None):
@@ -517,12 +519,9 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
             `numpy.ndarray`_: An array with the platescale for each order
             provided by ``order``.
         """
-        # TODO: Either assume a linear trend or measure this
-        # X-shooter manual says, but gives no exact numbers per order.
-        # NIR: 52.4 pixels (0.210"/pix) at order 11 to 59.9 pixels (0.184"/pix) at order 26.
-
-        # Right now I just assume a simple linear trend
-        plate_scale = 0.184 + (order_vec - 26)*(0.184-0.210)/(26 - 11)
+        # TODO: Figure out the order-dependence of the updated plate scale
+        # From the X-Shooter P113 manual, average over all orders. No order-dependent values given.
+        plate_scale = 0.245*np.ones_like(order_vec)
         return plate_scale
 
     @property
@@ -572,6 +571,11 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
         # Binning
         # TODO: Could this be detector dependent??
         binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
+        
+        # Grab the gain and read noise from the header.
+        # If hdu not present, use typical defaults
+        gain = None if hdu is None else np.atleast_1d(hdu[0].header['HIERARCH ESO DET OUT1 CONAD'])
+        ronoise = None if hdu is None else np.atleast_1d(hdu[0].header['HIERARCH ESO DET OUT1 RON'])
 
         # Detector 1
         detector_dict = dict(
@@ -581,17 +585,17 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
             specaxis        = 0,
             specflip        = False,
             spatflip        = False,
-            platescale      = 0.16, # average from order 17 and order 30, see manual
-            darkcurr        = 0.0,
+            platescale      = 0.154, # average from order 17 and order 30, see manual
+            darkcurr        = 0.0,  # e-/pixel/hour
             saturation      = 65535.,
             nonlinear       = 0.86,
             mincounts       = -1e10,
             numamplifiers   = 1,
-            gain            = np.atleast_1d(0.595), # FITS format is flipped: PrimaryHDU  (2106, 4000) w/respect to Python
-            ronoise         = np.atleast_1d(3.1), # raw unbinned images are (4000,2106) (spec, spat)
-            datasec=np.atleast_1d('[:,11:2058]'),  # pre and oscan are in the spatial direction
-            oscansec=np.atleast_1d('[:,2059:2106]'),
-        )
+            gain            = gain,
+            ronoise         = ronoise,
+            datasec=np.atleast_1d('[:,11:2058]'),    # FITS format is flipped: PrimaryHDU  (2106, 4000) w/respect to Python
+            oscansec=np.atleast_1d('[:,2059:2106]'), # raw unbinned images are (4000,2106) (spec, spat)
+        )                                            # pre and oscan are in the spatial direction
         return detector_container.DetectorContainer(**detector_dict)
 
     @classmethod
@@ -601,7 +605,7 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
         
         Returns:
             :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
-            all of ``PypeIt`` methods.
+            all of PypeIt methods.
         """
         par = super().default_pypeit_par()
 
@@ -641,15 +645,13 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
         # 1D wavelength solution
         par['calibrations']['wavelengths']['lamps'] = ['ThAr_XSHOOTER_VIS']
         # The following is for 1x1 binning. TODO GET BINNING SORTED OUT!!
-        par['calibrations']['wavelengths']['rms_threshold'] = 0.50
+        par['calibrations']['wavelengths']['rms_thresh_frac_fwhm'] = 0.15
+        par['calibrations']['wavelengths']['fwhm'] = 8.0
+        #
         par['calibrations']['wavelengths']['sigdetect'] = 5.0
         par['calibrations']['wavelengths']['n_final'] = [3] + 13*[4] + [3]
-        # This is for 1x1 binning. Needs to be divided by binning for binned data!!
-        par['calibrations']['wavelengths']['fwhm'] = 11.0
         # Reidentification parameters
         par['calibrations']['wavelengths']['method'] = 'reidentify'
-        # TODO: the arxived solution is for 1x1 binning. It needs to be
-        # generalized for different binning!
         par['calibrations']['wavelengths']['reid_arxiv'] = 'vlt_xshooter_vis1x1.fits'
         par['calibrations']['wavelengths']['cc_thresh'] = 0.50
         par['calibrations']['wavelengths']['cc_local_thresh'] = 0.50
@@ -659,6 +661,8 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
         par['calibrations']['wavelengths']['ech_nspec_coeff'] = 4
         par['calibrations']['wavelengths']['ech_norder_coeff'] = 4
         par['calibrations']['wavelengths']['ech_sigrej'] = 3.0
+        par['calibrations']['wavelengths']['qa_log'] = True
+
 
         # Flats
         par['calibrations']['flatfield']['tweak_slits_thresh'] = 0.90
@@ -677,15 +681,24 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
 
         # Sensitivity function parameters
         par['sensfunc']['algorithm'] = 'IR'
-        par['sensfunc']['polyorder'] = [9, 11, 11, 9, 9, 8, 8, 7, 7, 7, 7, 7, 7, 7, 7]
-        par['sensfunc']['IR']['telgridfile'] = 'TelFit_Paranal_VIS_4900_11100_R25000.fits'
+        par['sensfunc']['polyorder'] = 8 #[9, 11, 11, 9, 9, 8, 8, 7, 7, 7, 7, 7, 7, 7, 7]
+        par['sensfunc']['IR']['telgridfile'] = 'TellPCA_3000_26000_R25000.fits'
+        par['sensfunc']['IR']['pix_shift_bounds'] = (-10.0,10.0)
+        
+        # Telluric parameters
+        par['telluric']['pix_shift_bounds'] = (-10.0,10.0)
+        par['telluric']['resln_frac_bounds'] = (0.4,2.0)
+
+        # Coadding
+        par['coadd1d']['wave_method'] = 'log10'
+
         return par
 
     def init_meta(self):
         """
         Define how metadata are derived from the spectrograph files.
 
-        That is, this associates the ``PypeIt``-specific metadata keywords
+        That is, this associates the PypeIt-specific metadata keywords
         with the instrument-specific header cards using :attr:`meta`.
         """
         super().init_meta()
@@ -747,15 +760,12 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
             `numpy.ndarray`_: An array with the platescale for each order
             provided by ``order``.
         """
-        # VIS has no binning, but for an instrument with binning we would do this
         binspectral, binspatial = parse.parse_binning(binning)
-
-        # ToDO Either assume a linear trend or measure this
-        # X-shooter manual says, but gives no exact numbers per order.
-        # VIS: 65.9 pixels (0.167"/pix) at order 17 to 72.0 pixels (0.153"/pix) at order 30.
-
-        # Right now I just assume a simple linear trend
-        plate_scale = 0.153 + (order_vec - 30)*(0.153-0.167)/(30 - 17)
+        
+        # TODO: Figure out the order-dependence of the updated plate scale
+        # From the X-Shooter P113 manual, average over all orders. No order-dependent values given.
+        plate_scale = 0.154*np.ones_like(order_vec)
+        
         return plate_scale*binspatial
 
     @property
@@ -798,7 +808,7 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
                 Required if filename is None
                 Ignored if filename is not None
             msbias (`numpy.ndarray`_, optional):
-                Master bias frame used to identify bad pixels
+                Processed bias frame used to identify bad pixels
 
         Returns:
             `numpy.ndarray`_: An integer array with a masked value set
@@ -854,6 +864,10 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         """
         # Binning
         binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
+        
+        # Grab the gain and read noise from the header.
+        gain = None if hdu is None else np.atleast_1d(hdu[0].header['HIERARCH ESO DET OUT1 CONAD'])
+        ronoise = None if hdu is None else np.atleast_1d(hdu[0].header['HIERARCH ESO DET OUT1 RON'])
 
         # Detector 1
         detector_dict = dict(
@@ -863,14 +877,14 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
             specaxis        = 0,
             specflip        = True,
             spatflip        = True,
-            platescale      = 0.161, # average from order 14 and order 24, see manual
-            darkcurr        = 0.0,
+            platescale      = 0.164, # average from order 14 and order 24, see manual
+            darkcurr        = 0.0,  # e-/pixel/hour
             saturation      = 65000.,
             nonlinear       = 0.86,  
             mincounts       = -1e10,
             numamplifiers   = 1,
-            gain            = np.atleast_1d(1.61),
-            ronoise         = np.atleast_1d(2.60),
+            gain            = gain,
+            ronoise         = ronoise,
             datasec         = np.atleast_1d('[:,49:2096]'), # '[49:2000,1:2999]',
             oscansec        = np.atleast_1d('[:,1:48]'), # '[1:48, 1:2999]',
             )
@@ -884,7 +898,7 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         
         Returns:
             :class:`~pypeit.par.pypeitpar.PypeItPar`: Parameters required by
-            all of ``PypeIt`` methods.
+            all of PypeIt methods.
         """
         par = super().default_pypeit_par()
 
@@ -908,7 +922,10 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         # 1D wavelength solution
         par['calibrations']['wavelengths']['lamps'] = ['ThAr_XSHOOTER_UVB']
         par['calibrations']['wavelengths']['n_final'] = [3] + 10*[4] 
-        par['calibrations']['wavelengths']['rms_threshold'] = 0.60 
+        # This is for 1x1
+        par['calibrations']['wavelengths']['rms_thresh_frac_fwhm'] = 0.184
+        par['calibrations']['wavelengths']['fwhm'] = 3.8
+        #
         par['calibrations']['wavelengths']['sigdetect'] = 3.0 # Pretty faint lines in places
         # Reidentification parameters
         par['calibrations']['wavelengths']['method'] = 'reidentify'
@@ -922,6 +939,11 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         
         par['calibrations']['wavelengths']['cc_thresh'] = 0.50
         par['calibrations']['wavelengths']['cc_local_thresh'] = 0.50
+        par['calibrations']['wavelengths']['qa_log'] = True
+
+        # Flats
+        par['calibrations']['flatfield']['tweak_slits_thresh'] = 0.90
+        par['calibrations']['flatfield']['tweak_slits_maxfrac'] = 0.10
 
         # Right now we are using the overscan and not biases becuase the
         # standards are read with a different read mode and we don't yet have
@@ -947,13 +969,31 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         par['reduce']['findobj']['maxnumber_sci'] = 2  # Assume that there is a max of 2 objects on the slit
         par['reduce']['findobj']['maxnumber_std'] = 1  # Assume that there is only one object on the slit.
 
+        # Coadding
+        par['coadd1d']['wave_method'] = 'log10'
+
+
+        # Sensitivity function parameters
+        par['sensfunc']['algorithm'] = 'IR'
+        par['sensfunc']['polyorder'] =  8
+        par['sensfunc']['IR']['telgridfile'] = 'TellPCA_3000_26000_R25000.fits'
+        par['sensfunc']['IR']['pix_shift_bounds'] = (-8.0,8.0)
+        
+        # Telluric parameters
+        par['telluric']['pix_shift_bounds'] = (-8.0,8.0)
+
+        # Coadding
+        par['coadd1d']['wave_method'] = 'log10'
+
+
+
         return par
 
     def init_meta(self):
         """
         Define how metadata are derived from the spectrograph files.
 
-        That is, this associates the ``PypeIt``-specific metadata keywords
+        That is, this associates the PypeIt-specific metadata keywords
         with the instrument-specific header cards using :attr:`meta`.
         """
         super().init_meta()
@@ -979,7 +1019,7 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         .. code-block:: python
 
             from pypeit import edgetrace
-            edges = edgetrace.EdgeTraceSet.from_file('MasterEdges_A_1_DET01.fits.gz')
+            edges = edgetrace.EdgeTraceSet.from_file('Edges_A_1_DET01.fits.gz')
 
             nrm_edges = edges.edge_fit[edges.nspec//2,:] / edges.nspat
             slit_cen = ((nrm_edges + np.roll(nrm_edges,1))/2)[np.arange(nrm_edges.size//2)*2+1]
@@ -1029,15 +1069,11 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         """
         binspectral, binspatial = parse.parse_binning(binning)
 
-        # ToDO Either assume a linear trend or measure this
-        # X-shooter manual says, but gives no exact numbers per order.
-        # UVB: 65.9 pixels (0.167“/pix) at order 14 to 70.8 pixels (0.155”/pix) at order 24
+        # TODO: Figure out the order-dependence of the updated plate scale
+        # From the X-Shooter P113 manual, average over all orders. No order-dependent values given.
+        plate_scale = 0.164*np.ones_like(order_vec)
 
-        # Assume a simple linear trend
-        plate_scale = 0.155 + (order_vec - 24)*(0.155-0.167)/(24 - 14)
-
-        # Right now I just took the average
-        return np.full(self.norders, 0.161)*binspatial
+        return plate_scale*binspatial
 
     def bpm(self, filename, det, shape=None, msbias=None):
         """
@@ -1059,7 +1095,7 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
                 Required if filename is None
                 Ignored if filename is not None
             msbias (`numpy.ndarray`_, optional):
-                Master bias frame used to identify bad pixels
+                Processed bias frame used to identify bad pixels
 
         Returns:
             `numpy.ndarray`_: An integer array with a masked value set
